﻿if (typeof JSON !== "object") {
    JSON = {};
}

(function () {
    "use strict";

    var rx_one = /^[\],:{}\s]*$/;
    var rx_two = /\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g;
    var rx_three = /"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g;
    var rx_four = /(?:^|:|,)(?:\s*\[)+/g;
    var rx_escapable = /[\\"\u0000-\u001f\u007f-\u009f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;
    var rx_dangerous = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;

    function f(n) {
        // Format integers to have at least two digits.
        return (n < 10)
            ? "0" + n
            : n;
    }

    function this_value() {
        return this.valueOf();
    }

    if (typeof Date.prototype.toJSON !== "function") {

        Date.prototype.toJSON = function () {

            return isFinite(this.valueOf())
                ? (
                    this.getUTCFullYear()
                    + "-"
                    + f(this.getUTCMonth() + 1)
                    + "-"
                    + f(this.getUTCDate())
                    + "T"
                    + f(this.getUTCHours())
                    + ":"
                    + f(this.getUTCMinutes())
                    + ":"
                    + f(this.getUTCSeconds())
                    + "Z"
                )
                : null;
        };

        Boolean.prototype.toJSON = this_value;
        Number.prototype.toJSON = this_value;
        String.prototype.toJSON = this_value;
    }

    var gap;
    var indent;
    var meta;
    var rep;


    function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

        rx_escapable.lastIndex = 0;
        return rx_escapable.test(string)
            ? "\"" + string.replace(rx_escapable, function (a) {
                var c = meta[a];
                return typeof c === "string"
                    ? c
                    : "\\u" + ("0000" + a.charCodeAt(0).toString(16)).slice(-4);
            }) + "\""
            : "\"" + string + "\"";
    }


    function str(key, holder) {

// Produce a string from holder[key].

        var i;          // The loop counter.
        var k;          // The member key.
        var v;          // The member value.
        var length;
        var mind = gap;
        var partial;
        var value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

        if (
            value
            && typeof value === "object"
            && typeof value.toJSON === "function"
        ) {
            value = value.toJSON(key);
        }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

        if (typeof rep === "function") {
            value = rep.call(holder, key, value);
        }

// What happens next depends on the value's type.

        switch (typeof value) {
        case "string":
            return quote(value);

        case "number":

// JSON numbers must be finite. Encode non-finite numbers as null.

            return (isFinite(value))
                ? String(value)
                : "null";

        case "boolean":
        case "null":

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce "null". The case is included here in
// the remote chance that this gets fixed someday.

            return String(value);

// If the type is "object", we might be dealing with an object or an array or
// null.

        case "object":

// Due to a specification blunder in ECMAScript, typeof null is "object",
// so watch out for that case.

            if (!value) {
                return "null";
            }

// Make an array to hold the partial results of stringifying this object value.

            gap += indent;
            partial = [];

// Is the value an array?

            if (Object.prototype.toString.apply(value) === "[object Array]") {

// The value is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || "null";
                }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

                v = partial.length === 0
                    ? "[]"
                    : gap
                        ? (
                            "[\n"
                            + gap
                            + partial.join(",\n" + gap)
                            + "\n"
                            + mind
                            + "]"
                        )
                        : "[" + partial.join(",") + "]";
                gap = mind;
                return v;
            }

// If the replacer is an array, use it to select the members to be stringified.

            if (rep && typeof rep === "object") {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    if (typeof rep[i] === "string") {
                        k = rep[i];
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (
                                (gap)
                                    ? ": "
                                    : ":"
                            ) + v);
                        }
                    }
                }
            } else {

// Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.prototype.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (
                                (gap)
                                    ? ": "
                                    : ":"
                            ) + v);
                        }
                    }
                }
            }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

            v = partial.length === 0
                ? "{}"
                : gap
                    ? "{\n" + gap + partial.join(",\n" + gap) + "\n" + mind + "}"
                    : "{" + partial.join(",") + "}";
            gap = mind;
            return v;
        }
    }

// If the JSON object does not yet have a stringify method, give it one.

    if (typeof JSON.stringify !== "function") {
        meta = {    // table of character substitutions
            "\b": "\\b",
            "\t": "\\t",
            "\n": "\\n",
            "\f": "\\f",
            "\r": "\\r",
            "\"": "\\\"",
            "\\": "\\\\"
        };
        JSON.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

            var i;
            gap = "";
            indent = "";

// If the space parameter is a number, make an indent string containing that
// many spaces.

            if (typeof space === "number") {
                for (i = 0; i < space; i += 1) {
                    indent += " ";
                }

// If the space parameter is a string, it will be used as the indent string.

            } else if (typeof space === "string") {
                indent = space;
            }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

            rep = replacer;
            if (replacer && typeof replacer !== "function" && (
                typeof replacer !== "object"
                || typeof replacer.length !== "number"
            )) {
                throw new Error("JSON.stringify");
            }

// Make a fake root object containing our value under the key of "".
// Return the result of stringifying the value.

            return str("", {"": value});
        };
    }


// If the JSON object does not yet have a parse method, give it one.

    if (typeof JSON.parse !== "function") {
        JSON.parse = function (text, reviver) {

// The parse method takes a text and an optional reviver function, and returns
// a JavaScript value if the text is a valid JSON text.

            var j;

            function walk(holder, key) {

// The walk method is used to recursively walk the resulting structure so
// that modifications can be made.

                var k;
                var v;
                var value = holder[key];
                if (value && typeof value === "object") {
                    for (k in value) {
                        if (Object.prototype.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }


// Parsing happens in four stages. In the first stage, we replace certain
// Unicode characters with escape sequences. JavaScript handles many characters
// incorrectly, either silently deleting them, or treating them as line endings.

            text = String(text);
            rx_dangerous.lastIndex = 0;
            if (rx_dangerous.test(text)) {
                text = text.replace(rx_dangerous, function (a) {
                    return (
                        "\\u"
                        + ("0000" + a.charCodeAt(0).toString(16)).slice(-4)
                    );
                });
            }

// In the second stage, we run the text against regular expressions that look
// for non-JSON patterns. We are especially concerned with "()" and "new"
// because they can cause invocation, and "=" because it can cause mutation.
// But just to be safe, we want to reject all unexpected forms.

// We split the second stage into 4 regexp operations in order to work around
// crippling inefficiencies in IE's and Safari's regexp engines. First we
// replace the JSON backslash pairs with "@" (a non-JSON character). Second, we
// replace all simple value tokens with "]" characters. Third, we delete all
// open brackets that follow a colon or comma or that begin the text. Finally,
// we look to see that the remaining characters are only whitespace or "]" or
// "," or ":" or "{" or "}". If that is so, then the text is safe for eval.

            if (
                rx_one.test(
                    text
                        .replace(rx_two, "@")
                        .replace(rx_three, "]")
                        .replace(rx_four, "")
                )
            ) {

// In the third stage we use the eval function to compile the text into a
// JavaScript structure. The "{" operator is subject to a syntactic ambiguity
// in JavaScript: it can begin a block or an object literal. We wrap the text
// in parens to eliminate the ambiguity.

                j = eval("(" + text + ")");

// In the optional fourth stage, we recursively walk the new structure, passing
// each name/value pair to a reviver function for possible transformation.

                return (typeof reviver === "function")
                    ? walk({"": j}, "")
                    : j;
            }

// If the text is not JSON parseable, then a SyntaxError is thrown.

            throw new SyntaxError("JSON.parse");
        };
    }
}());

function jqExtend() {
    // 默认不进行深拷贝
    var deep = false;
    var name, options, src, copy, clone, copyIsArray;
    var length = arguments.length;
    // 记录要复制的对象的下标
    var i = 1;
    // 第一个参数不传布尔值的情况下，target 默认是第一个参数
    var target = arguments[0] || {};
    // 如果第一个参数是布尔值，第二个参数是 target
    if (typeof target == 'boolean') {
        deep = target;
        target = arguments[i] || {};
        i++;
    }
    // 如果target不是对象，我们是无法进行复制的，所以设为 {}
    if (typeof target !== "object" && !isFunction(target)) {
        target = {};
    }
    // 循环遍历要复制的对象们
    for (; i < length; i++) {
        // 获取当前对象
        options = arguments[i];
        // 要求不能为空 避免 extend(a,,b) 这种情况
        if (options != null) {
            for (name in options) {
                // 目标属性值
                src = target[name];
                // 要复制的对象的属性值
                copy = options[name];
                // 解决循环引用
                if (target === copy) {
                    continue;
                }
                // 要递归的对象必须是 plainObject 或者数组
                if (deep && copy && (isPlainObject(copy) ||
                        (copyIsArray = Array.isArray(copy)))) {
                    // 要复制的对象属性值类型需要与目标属性值相同
                    if (copyIsArray) {
                        copyIsArray = false;
                        clone = src && Array.isArray(src) ? src : [];

                    } else {
                        clone = src && isPlainObject(src) ? src : {};
                    }
                    target[name] = extend(deep, clone, copy);
                } else if (copy !== undefined) {
                    target[name] = copy;
                }
            }
        }
    }
    return target;
};





var treeSetting; //树形菜单的间距;属性treeTop记录tree高度，属性layerNum记录画一个树需要多少个图层;
var uedTreeSetting;
var dlg;
var doc, layer;

var windowBounds = {}; //属性top,right,bottom,left记录拉框的数值;
var geoSetting = {}; //地图画线的颜色，画面的颜色，宽度等等配置项;
var oldRuleUnits //记录原来的标尺,函数运行过程中，改成px,函数结束，改回px;
/*var FONT = {
    //默认思源黑体;
    family: "SourceHanSansCN-Normal",
    //默认fontAwesome;
    iconFamily: "FontAwesome5FreeSolid"
};*/
var UED = {
    dashedLine: [4, 2], //所有虚线都是4px点，然后2px空白;
    decimal: 0, //图表上线条的数字保留几位小数,分别是0,1,2;
    font: {
        //默认思源黑体;
        family: "SourceHanSansCN-Normal",
        //默认fontAwesome;
        iconFamily: "FontAwesome5FreeSolid"
    },
    btn: {
        normal:{
            fontSize: 12,
            fontColor: '515a6e',
            bgColor: 'ffffff',
            borderRadius: '4,4,4,4',
            padding: '10,16,10,16',
            borderWidth: 1,
            borderType: 'solid',
            borderColor: 'dcdee2',
            iconFontSize: 12,
            iconMargin: 8
        },
        primary:{
            fontSize: 12,
            fontColor: 'ffffff',
            bgColor: '2d8cf0',
            borderRadius: '4,4,4,4',
            padding: '10,16,10,16',
            borderWidth: 0,
            borderType: 'solid',
            borderColor: '2d8cf0',
            iconFontSize: 12,
            iconMargin: 8
        },
        success:{
            fontSize: 12,
			fontColor: "ffffff",
			bgColor: "22a240",
			borderRadius: "4,4,4,4",
			padding: "10,16,10,16",
			borderWidth: 0,
			borderType: "solid",
			borderColor: "22a240",
			iconFontSize: 12,
			iconMargin: 8
        },
        info:{
            fontSize: 12,
            fontColor: 'ffffff',
            bgColor: '2db7f5',
            borderRadius: '4,4,4,4',
            padding: '10,16,10,16',
            borderWidth: 0,
            borderType: 'solid',
            borderColor: '2db7f5',
            iconFontSize: 12,
            iconMargin: 8
        },
        warning:{
            fontSize: 12,
            fontColor: 'ffffff',
            bgColor: 'ff9900',
            borderRadius: '4,4,4,4',
            padding: '10,16,10,16',
            borderWidth: 0,
            borderType: 'solid',
            borderColor: 'ff9900',
            iconFontSize: 12,
            iconMargin: 8
        },
        danger:{
            fontSize: 12,
            fontColor: 'ffffff',
            bgColor: 'ed4014',
            borderRadius: '4,4,4,4',
            padding: '10,16,10,16',
            borderWidth: 0,
            borderType: 'solid',
            borderColor: 'ed4014',
            iconFontSize: 12,
            iconMargin: 8
        }
    },
    btnGroup:{
		base: {
			fontSize: 12,
			fontColor: "515a6e",
            bgColor: "ffffff",
            borderRadius: '0,0,0,0',
			padding: "10,16,10,16",
			borderWidth: 1,
			borderType: "solid",
			borderColor: "dcdee2",
			margin: -1
        },
        first: {
            borderRadius: '4,0,0,4',
            icon: '',
            iconMargin: 8
        },
        last: {
            borderRadius: '0,4,4,0',
            icon: '',
            iconMargin: 8
        },
        select: {
            fontColor: 'ffffff',
            bgColor: '2d8cf0',
            borderColor: '2d8cf0'
        }
    },
    input: {
		fontSize: 12,
		fontColor: "515a6e",
		bgColor: "ffffff",
		borderRadius: "4,4,4,4",
		padding: "10,9,10,9",
		borderWidth: 1,
		borderType: "solid",
		borderColor: "dcdee2"
	},
    select: {
        fontSize: 12,
		fontColor: "515a6e",
		bgColor: "ffffff",
		borderRadius: "4,4,4,4",
		padding: "10,9,10,9",
		borderWidth: 1,
		borderType: "solid",
		borderColor: "dcdee2"
    },
    radio:{
		base: {
			fontSize: 12,
			fontColor: "333333",
			iconMargin: 4
		},
		select: {
			outerDiam: 14,
			outerBorderWidth: 1,
			outerBorderColor: "2d8cf0",
			outerBg: null,
			innerDiam: 8,
			innerBorderWidth: 0,
			innerBorderColor: "2d8cf0",
			innerBg: "2d8cf0"
		},
		unselect: {
			outerDiam: 14,
			outerBorderWidth: 1,
			outerBorderColor: "cccccc",
			outerBg: null,
			innerDiam: 8,
			innerBorderWidth: 0,
			innerBorderColor: "2d8cf0",
			innerBg: null
		}
    },
    checkbox: {
        base: {
            fontSize: 12,
			fontColor: "333333",
            iconMargin: 4,
            rectSize: 12,
            rectRadius: 4
        },
        checked:{
            borderWidth: 2,
            borderColor: '2d8cf0',
            bgColor: '2d8cf0',
            icon: '',
            iconSize: 10,
            iconColor: 'ffffff'
        },
        unchecked:{
            borderWidth: 1,
            borderColor: '333333',
            bgColor: 'ffffff'
        }
    },
    tooltip: {
        fontSize: 12,
        fontColor: 'ffffff',
        bgColor: '585e6b',
        borderRadius: '4,4,4,4',
        padding: '12,15,11,15',
        borderWidth: 0,
        borderColor: '585e6b',
        triangleAngle: 90,
        triangleHeight: 5,
        triangleDistance: 8
    },
    alert: {
        base: {
            titleFontSize: 16,
            titleFontBolder: true,
            titleFontMargin: 4,
            fontSize: 12,
            borderRadius: '4,4,4,4',
            borderWidth: 1,
            padding: '14,16,14,16',
            largeIconSize: 20,
            largeIconMargin: 10,
            smallIconSize: 12,
            smallIconMargin: 6,
            closeIcon: '',
            closeIconSize: 10,
            closeIconMargin: 6
        },
        success: {
            titleColor: '17233d',
            fontColor: '515a6e',
            borderColor: '8ce6b0',
            bgColor: 'edfff3',
            largeIcon: '',
            largeIconColor: '2dbe74',
            smallIcon: '',
            smallIconColor: '2dbe74',
            closeIconColor: '515a6e'
        },
        info: {
            titleColor: '17233d',
            fontColor: '515a6e',
            borderColor: 'abdcff',
            bgColor: 'f0faff',
            largeIcon: '',
            largeIconColor: '338cf0',
            smallIcon: '',
            smallIconColor: '338cf0',
            closeIconColor: '515a6e'
        },
        warning: {
            titleColor: '17233d',
            fontColor: '515a6e',
            borderColor: 'ffd77a',
            bgColor: 'fff9e6',
            largeIcon: '',
            largeIconColor: 'ff9c16',
            smallIcon: '',
            smallIconColor: 'ff9c16',
            closeIconColor: '515a6e'
        },
        error: {
            titleColor: '17233d',
            fontColor: '515a6e',
            borderColor: 'ffb08f',
            bgColor: 'ffefe6',
            largeIcon: '',
            largeIconColor: 'ef4b14',
            smallIcon: '',
            smallIconColor: 'ef4b14',
            closeIconColor: '515a6e'
        }
    },
    table: {
        base: {
            rules: 'all',
            borderRadius: '2,2,2,2',
            borderWidth: 1,
            borderColor: 'dee2e6'
        },
        th: {
            borderWidth: 1,
            borderColor: 'dee2e6',
            bgColor: 'f8f8f9',
            padding: '16,5,16,5',
            fontSize: 14,
            fontColor: '515a6e',
            fontBolder: true,
            align: 'center'
        },
        td: {
            borderWidth: 1,
            borderColor: 'dee2e6',
            bgColor: 'ffffff',
            padding: '16,5,16,5',
            fontSize: 12,
            fontColor: '515a6e',
            highlightBgColor: 'ebf7ff',
            highlightFontColor: '515a6e'
        }
    },
    modal: {
        base: {
            borderRadius: '4,4,4,4',
            borderWidth: 1,
            borderColor: 'e8eaec',
            bgColor: 'ffffff'
        },
        th: {
            borderWidth: 1,
            borderColor: 'e8eaec',
            bgColor: null,
            padding: '18,18,18,18',
            fontSize: 14,
            fontColor: '17233d',
            fontBolder: true,
            align: 'left',
            closeIconShow: true,
            closeIcon: '',
            closeIconSize: 14,
            closeIconColor: '999999'
        },
        tfoot: {
            show: true,
            borderWidth: 1,
            borderColor: 'e8eaec',
            bgColor: null,
            padding: '12,18,12,18',
            align: 'right',
            btnShow: 'all'
        },
        tfootBtns: [{
            txt: '取消',
            type: 'normal'
        },{
            txt: '确定',
            type: 'primary'
        },4]
    },
    tree: {
        fontSize: 12,
        fontColor: '515a6e',
        openIcon: '',
        openIconColor: '0072ff',
        closeIcon: '',
        closeIconColor: '0072ff',        
        openCloseIconSize: 14,
        iconMargin: 2,
        fontMarginLeft: 4,
        levelMarginLeft: 20,
        levelMarginBottom: 10,
        withLine: true,
        lineColor: 'd9d9d9',
        lineType: 'dashed'
    }
}
//读取config.txt的配置文件;
function changeConfigUED(uedStr){
    var obj = JSON.parse(uedStr);
    UED = jqExtend({}, UED, obj);
    //alert(JSON.stringify(UED));
}


init();
function init(){
    oldRuleUnits = Units.PIXELS;
    treeSetting = {
        top: 26,
        left: 22,
        treeTop: 0,
        layerNum: 0 
    }
    uedTreeSetting = {
        top: 0, //top是固定的，但是left则每个不同，这里可以共用top;
        left: 0,
        treeTop: 0,
        layerNum: 0 
    }
    //将地图的配置项存储为全局变量;
    geoSetting = {
        "geo":{
            "itemStyle": {
                "show": true,
                "color": "0e9cff",
                "borderColor": "000000",
                "borderWidth": 2,
                "borderLocation": "outerside"
            },
            "label":{
                "show": true,
                "color": "000000",
                "fontSize": 12
            }
         }   
    }
    //loadJSX("json2.js");
}
//设置选项里面改变字体的配置;
function changeFontFamily(obj){
    changeConfigUED(obj);
    /*var o = JSON.parse(obj);
    if(o && o.family && o.iconFamily){
        FONT.family = o.family;
        FONT.iconFamily = o.iconFamily;
        completeFn();
        alert('保存成功');
    }*/
};

//测试字体名称;
function getFontPostScriptName(){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    return layer.textItem.font;
    //alert("该字体的postScript名称为：" + layer.textItem.font);
}

//吸管;
function useColorPicker(json){
    setForegroundColor(json);
    app.showColorPicker();
    var color = app.foregroundColor;
    var returnValue = color.rgb.hexValue;
    return returnValue;
}



//创建UED按钮;
function createUedBtn(obj){
    var errObj  = checkError(obj);
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var obj = JSON.parse(obj);
    //不拉框的时候，可以将范围传过来;
    if(obj.bounds){ 
        var w = windowBounds = obj.bounds
    }else{
        var w = windowBounds = getSelectionBounds();
    };
    
    var btnObj = {
        content: obj.content,
        icon: obj.icon,
        type: obj.type,
        w: w
    }
    try{
        doc.selection.deselect();    
    }catch(e){}
    addUedBtn(btnObj);
}
function addUedBtn(obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var type = obj.type;

    var uedBtnType = UED.btn[type];
    //虚线框范围传递过来;
    var w = obj.w;
    var txtLen = 0;
    var txtWh = getTxtLen({
        content: obj.content,
        fontSize: uedBtnType.fontSize
    });
    txtLen = txtWh.width;
    var txtHeight = txtWh.height;
    var layerNum = 0;
    var paddingArr = uedBtnType.padding.split(',');
    var paddingLR = parseInt(paddingArr[1], 10) + parseInt(paddingArr[3], 10);
    var paddingTB = parseInt(paddingArr[0], 10) + parseInt(paddingArr[2], 10);
    var dotsR = w.left + txtLen + paddingLR;
    var iconWh;
    if(obj.icon != ''){
        iconWh = getTxtLen({
            content: obj.icon,
            fontSize: uedBtnType.iconFontSize,
            font: UED.font.iconFamily
        });
        dotsR += (iconWh.width + uedBtnType.iconMargin);
    }
    /*var dots = [
        [w.left, w.top],
        [w.left, w.top + uedBtnType.fontSize + paddingTB],
        [dotsR, w.top + uedBtnType.fontSize + paddingTB],
        [dotsR, w.top]
    ];*/
    var dots = [
        [w.left, w.top],
        [w.left, w.top + txtHeight + paddingTB],
        [dotsR, w.top + txtHeight + paddingTB],
        [dotsR, w.top]
    ];
    //如果圆角值不是0，则需要绘制成圆角;
    if(uedBtnType.borderRadius != '0,0,0,0'){
        var borderArr = splitToNumber(uedBtnType.borderRadius, ',');
        dots = getConnerDots(dots, borderArr);
    }
    //渐变角度控件的角度是从90度开始的，我设计的渐变角度控件是从0度开始的;    
    /*if(typeof(uedBtnType.bgColor) == 'object'){
        if(uedBtnType.bgColor != null && /^\d+$/.test(uedBtnType.bgColor.angle)){
            uedBtnType.bgColor.angle = correctAngle(uedBtnType.bgColor.angle);
        }
    }*/
    
    //如果存在边框,也就是说borderWidth>0, 那么需要绘制边框图层;
    if(uedBtnType.borderWidth > 0){
        addLayer({
            name: 'btn-Border-'+ uedBtnType.borderWidth +'px '+ uedBtnType.borderType +' #'+ uedBtnType.borderColor
        });
        //也可能存在没有背景，只有边线的按钮;
        if(uedBtnType.bgColor == 'transparent' || uedBtnType.bgColor == null){
            drawVectorShape({
                dots: dots,
                type: 'stroke', 
                hexValue: uedBtnType.borderColor,
                solidWidth: uedBtnType.borderWidth,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                strokeType: uedBtnType.borderType,
                closed: true
            });
        }else{
            //有背景，又有边框的按钮;
            drawVectorShape({
                dots: dots,
                type: 'all', 
                hexValue: uedBtnType.borderColor,
                hexValueFill: uedBtnType.bgColor,
                solidWidth: uedBtnType.borderWidth,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                strokeType: uedBtnType.borderType,
                closed: true
            });
        }
    //不存在边框，那么直需要绘制按钮背景那个色块;
    }else{
        addLayer({
            name: 'btn-Bg-#' + uedBtnType.bgColor
        })
        drawVectorShape({
            dots: dots,
            type: 'fill',
            hexValueFill: uedBtnType.bgColor,
            closed: true
        });
    }
    layerNum++;
    var contentLeftPos = w.left + parseInt(paddingArr[3], 10);
    //如果要绘制图标;
    if(obj.icon != ''){
        addTxtLayer({
            name: '',
            font: UED.font.iconFamily,
            color: uedBtnType.fontColor,
            size: uedBtnType.iconFontSize,
            position: [w.left + parseInt(paddingArr[3], 10), w.top + parseInt(paddingArr[0], 10)],
            txtAlign: 'left',
            content: obj.icon,
            width: iconWh.width + uedBtnType.iconMargin
        });
        contentLeftPos += (iconWh.width + uedBtnType.iconMargin);
        layerNum++;
    }
    addTxtLayer({
        name: 'btn-text',
        font: UED.font.family,
        color: uedBtnType.fontColor,
        size: uedBtnType.fontSize,
        position: [contentLeftPos, w.top + parseInt(paddingArr[0], 10)],
        txtAlign: 'left',
        content: obj.content,
        width: txtLen
    });
    layerNum++;
    createLayerSet({
        name: 'button',
        number: layerNum
    });


};
//将一个字符串分割，并且分割后每项都是数字;
function splitToNumber(str, splitStr){
    var arr = [];
    var strArr = str.split(splitStr);
    for(var i=0; i<strArr.length; i++){
        arr.push(parseInt(strArr[i], 10));
    }
    return arr;
}



function addBtn(obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;

    var borderColor = '075d76';
    var txtColor = '09a7b0';
    //var txtLen = obj.content.length * 14; // 每个文字14个像素;
    var txtLen = 0;
    for(var j=0; j<obj.content.length; j++){
        var reg = /[\u4e00-\u9fa5]/;
        if(reg.test(obj.content[j])){ //是中文汉字;
            txtLen += 14;
        }else{
            txtLen += 10;
        }
    }
    var points = [
        [1, 0],
        [0, 1],
        [0, 20],
        [10, 30],
        [42+txtLen+20, 30],
        [42+txtLen+21, 29],
        [42+txtLen+21, 9],
        [42+txtLen+12, 0]
    ];
    addLayer({
        name: 'btn-Bg'
    });
    switch(obj.type){
        case 'ok':
            borderColor = '89afb0';
            txtColor = '89afb0';
            drawShape({
                type: 'selection', 
                dots: points
            });
            setForegroundColor('30444f');
            setBackgroundColor('172b36');
            createLineGradual({
                startY: 0,
                endY: 30
                });                
            doc.selection.deselect();
            break;
        case 'cancel':
            drawShape({
                type: 'fill', 
                dots: points,
                hexValue: "052834",
                opacity: 100
            });
            break;
        case 'primary':
            txtColor = 'ffffff';
            drawShape({
                type: 'fill', 
                dots: points,
                hexValue: "075d76",
                opacity: 100
            });
            break;
        default:
            drawShape({
                type: 'fill', 
                dots: points,
                hexValue: "075d76",
                opacity: 30
            });
            break;
    }
    addLayer({
        name: 'btn-2SolidLine'
    });
    drawShape({
        type: 'stroke', 
        dots: points,
        hexValue: borderColor,
        solidWidth: 2,
        location: 'inside'
    });
    drawShape({
        type: 'fill', 
        dots: [[2,4], [4,5], [4, 16], [2, 17]],
        hexValue: borderColor,
        opacity: 100
    });
    addTxtLayer({
        name: 'btn-icon',
        font: UED.font.iconFamily,
        color: txtColor,
        size: 12,
        position: [22, 20],
        txtAlign: 'left',
        content: obj.iconTxt
    });
    addTxtLayer({
        name: 'btn-txt',
        font: UED.font.family,
        color: txtColor,
        size: 14,
        position: [44, 20],
        txtAlign: 'left',
        content: obj.content
    })    
}
//绘制系统UI控件中的输入框;
function createUedInput(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var uedInput = UED.input;
    var paddingArr = uedInput.padding.split(',');
    var paddingLR = parseInt(paddingArr[1], 10) + parseInt(paddingArr[3], 10);
    var paddingTB = parseInt(paddingArr[0], 10) + parseInt(paddingArr[2], 10);
    var txtObj = getTxtLen({
        content: obj.txt || 'input',
        fontSize: uedInput.fontSize
    });
    var inputHeight = txtObj.height + paddingTB;
    var dots = [
        [w.left, w.top], 
        [w.left, w.top + inputHeight], 
        [w.left+w.width, w.top + inputHeight], 
        [w.left+w.width, w.top]
    ];    
    //如果圆角值不是0，则需要绘制成圆角;
    if(uedInput.borderRadius != '0,0,0,0'){
        var borderArr = splitToNumber(uedInput.borderRadius, ',');
        dots = getConnerDots(dots, borderArr);
    }
    addLayer({
        name: 'input'
    });
    var layerNum = 1;
    //如果存在边框,也就是说borderWidth>0, 那么需要绘制边框图层;
    if(uedInput.borderWidth > 0){
        //也可能存在没有背景，只有边线的按钮;
        if(uedInput.bgColor == 'transparent' || uedInput.bgColor == null){
            drawVectorShape({
                dots: dots,
                type: 'stroke', 
                hexValue: uedInput.borderColor,
                solidWidth: uedInput.borderWidth,
                strokeType: uedInput.borderType,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: true
            });
        }else{
            //有背景，又有边框的按钮;
            drawVectorShape({
                dots: dots,
                type: 'all', 
                hexValue: uedInput.borderColor,
                hexValueFill: uedInput.bgColor,
                solidWidth: uedInput.borderWidth,
                strokeType: uedInput.borderType,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: true
            });
        }
    //不存在边框，那么直需要绘制输入框背景那个色块（极端情况，没有边框又没有背景,直接绘制个白色背景）;
    }else{
        //没有边框，只有背景;
        drawVectorShape({
            dots: dots,
            type: 'fill',
            hexValueFill: uedInput.bgColor != null ? uedInput.bgColor : 'ffffff',
            closed: true
        });
    }
    //输入框里面存在文字;
    if(obj.txt.length > 0){
        addTxtLayer({
            name: obj.txt,
            font: UED.font.family,
            color: uedInput.fontColor,
            size: uedInput.fontSize,
            position: [w.left + parseInt(paddingArr[3], 10), w.top + parseInt(paddingArr[0], 10)],
            txtAlign: 'left',
            content: obj.txt,
            width: (w.width > paddingLR) ? (w.width - paddingLR) : w.width
        });
        layerNum++;
    }
    if(obj.icon){
        var iconObj = getTxtLen({
            content: obj.icon,
            fontSize: uedInput.fontSize,
            font: UED.font.iconFamily
        });
        var leftPos = w.left + w.width - parseInt(paddingArr[3], 10) - iconObj.width;
        addTxtLayer({
            name: '',
            font: UED.font.iconFamily,
            color: uedInput.fontColor,
            size: uedInput.fontSize,
            position: [leftPos, w.top + parseInt(paddingArr[0], 10)],
            txtAlign: 'left',
            content: obj.icon,
            width: iconObj.width + parseInt(paddingArr[3], 10)
        });
        layerNum++;
    }
    createLayerSet({
        name: 'input',
        number: layerNum
    });
}
//系统ui控件,绘制下拉框;
function createUedSelect(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var uedSelect = UED.select;
    var paddingArr = uedSelect.padding.split(',');
    var paddingLR = parseInt(paddingArr[1], 10) + parseInt(paddingArr[3], 10);
    var paddingTB = parseInt(paddingArr[0], 10) + parseInt(paddingArr[2], 10);
    var txtObj = getTxtLen({
        content: (obj.txt.length > 0) ? obj.txt : 'psCharts',
        fontSize: uedSelect.fontSize
    });
    var selectHeight = txtObj.height + paddingTB;
    var dots = [
        [w.left, w.top], 
        [w.left, w.top + selectHeight], 
        [w.left+w.width, w.top + selectHeight], 
        [w.left+w.width, w.top]
    ];    
    //如果圆角值不是0，则需要绘制成圆角;
    if(uedSelect.borderRadius != '0,0,0,0'){
        var borderArr = splitToNumber(uedSelect.borderRadius, ',');
        dots = getConnerDots(dots, borderArr);
    }
    addLayer({
        name: 'select'
    });
    var layerNum = 1;
    //如果存在边框,也就是说borderWidth>0, 那么需要绘制边框图层;
    if(uedSelect.borderWidth > 0){
        //也可能存在没有背景，只有边线;
        if(uedSelect.bgColor == 'transparent' || uedSelect.bgColor == null){
            drawVectorShape({
                dots: dots,
                type: 'stroke', 
                hexValue: uedSelect.borderColor,
                solidWidth: uedSelect.borderWidth,
                strokeType: uedSelect.borderType,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: true
            });
        }else{
            //有背景，又有边框;
            drawVectorShape({
                dots: dots,
                type: 'all', 
                hexValue: uedSelect.borderColor,
                hexValueFill: uedSelect.bgColor,
                solidWidth: uedSelect.borderWidth,
                strokeType: uedSelect.borderType,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: true
            });
        }
    //不存在边框，那么直需要绘制输入框背景那个色块（极端情况，没有边框又没有背景,直接绘制个白色背景）;
    }else{
        //没有边框，只有背景;
        drawVectorShape({
            dots: dots,
            type: 'fill',
            hexValueFill: uedSelect.bgColor != null ? uedSelect.bgColor : 'ffffff',
            closed: true
        });
    }
    //下拉框里面存在文字;
    if(obj.txt.length > 0){
        addTxtLayer({
            name: obj.txt,
            font: UED.font.family,
            color: uedSelect.fontColor,
            size: uedSelect.fontSize,
            position: [w.left + parseInt(paddingArr[3], 10), w.top + parseInt(paddingArr[0], 10)],
            txtAlign: 'left',
            content: obj.txt,
            width: (w.width > paddingLR) ? (w.width - paddingLR) : w.width
        });
        layerNum++;
    }
    var iconObj = getTxtLen({
        content: '',
        font: UED.font.iconFamily,
        fontSize: uedSelect.fontSize
    });
    var leftPos = w.left + w.width - parseInt(paddingArr[3], 10) - iconObj.width;
    addTxtLayer({
        name: '',
        font: UED.font.iconFamily,
        color: uedSelect.fontColor,
        size: uedSelect.fontSize,
        position: [leftPos, w.top + parseInt(paddingArr[0], 10)],
        txtAlign: 'left',
        content: '',
        width: iconObj.width + parseInt(paddingArr[3], 10)
    });
    layerNum++;
    createLayerSet({
        name: 'select',
        number: layerNum
    });
}
//系统UI控件，绘制单选框;
function createUedRadio(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var layerNum = 0;    
    var uedRadio = UED.radio;
    var ck = obj.checked ? 'select' : 'unselect';
    var withOuter = (UED.radio[ck].outerBorderWidth > 0 || UED.radio[ck].outerBg != null) ? true : false;
    if(withOuter){
        addLayer({
            name: '单选框-外圈'
        });
        drawVerctorCircle({
            bgColor: UED.radio[ck].outerBg,
            borderWidth: UED.radio[ck].outerBorderWidth,
            borderColor: UED.radio[ck].outerBorderColor,
            diam: UED.radio[ck].outerDiam,
            top: w.top,
            left: w.left
        });
        layerNum ++;
    }
    var withInner = (UED.radio[ck].innerBorderWidth > 0 || UED.radio[ck].innerBg != null) ? true : false;
    if(withInner){
        addLayer({
            name: "单选框-内圈"
        });
        var topPose = withOuter ? w.top + (UED.radio[ck].outerDiam - UED.radio[ck].innerDiam)/2 : w.top;
        var leftPos = withOuter ? w.left + (UED.radio[ck].outerDiam - UED.radio[ck].innerDiam)/2 : w.left;
        drawVerctorCircle({
            bgColor: UED.radio[ck].innerBg,
            borderWidth: UED.radio[ck].innerBorderWidth,
            borderColor: UED.radio[ck].innerBorderColor,
            diam: UED.radio[ck].innerDiam,
            top: topPose,
            left: leftPos
        });
        layerNum ++;
    }
    if(obj.txt && obj.txt.length > 0){
        //内外圆圈都没有;
        var txtLeft = w.left;
        //有外圆圈;
        if(withOuter){
            txtLeft += UED.radio.base.iconMargin + UED.radio[ck].outerDiam;
        //有内圆圈;
        }else if(withInner){
            txtLeft += UED.radio.base.iconMargin + UED.radio[ck].innerDiam;
        }
        var txtWidth = getTxtLen({
            content: obj.txt,
            fontSize: UED.radio.base.fontSize
        }).width;
        addTxtLayer({
            name: obj.txt,
            content: obj.txt,
            font: UED.font.family,
            size: UED.radio.base.fontSize,
            color: UED.radio.base.fontColor,
            txtAlign: 'left',
            width: txtWidth,
            position: [txtLeft, w.top]
        });
        layerNum ++;
    }
    createLayerSet({
        name: '单选框',
        number: layerNum
    });
    completeFn();
}
//系统UI控件，绘制多选;
function createUedCheckbox(obj){
    var errObj  = checkError(obj);
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    if(obj.bounds){
        var w = windowBounds = obj.bounds;
    }else{
        var w = windowBounds = getSelectionBounds();
        doc.selection.deselect();
    }
    var layerNum = 0;    
    var uedCheckbox = UED.checkbox;
    var ck = obj.checked ? 'checked' : 'unchecked';
    var withRect = (UED.checkbox[ck].borderWidth > 0 || UED.checkbox[ck].bgColor != null) ? true : false;
    if(withRect){
        addLayer({
            name: '多选框'
        });
        var rectSize = UED.checkbox.base.rectSize;
        var rectRadius = UED.checkbox.base.rectRadius;
        var dots = [
            [w.left, w.top],
            [w.left, w.top + rectSize],
            [w.left + rectSize, w.top + rectSize],
            [w.left + rectSize, w.top]
        ];
        if(rectRadius != 0){
            dots = getConnerDots(dots, rectRadius);
        }
        //有背景又有边框;
        if(UED.checkbox[ck].borderWidth > 0 && UED.checkbox[ck].bgColor != null){
            drawVectorShape({
                dots: dots,
                type: 'all', 
                hexValue: UED.checkbox[ck].borderColor,
                hexValueFill: UED.checkbox[ck].bgColor,
                solidWidth: UED.checkbox[ck].borderWidth,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: true
            });
        //只有边框;
        }else if(UED.checkbox[ck].borderWidth > 0){
            drawVectorShape({
                dots: dots,
                type: 'stroke', 
                hexValue: UED.checkbox[ck].borderColor,
                solidWidth: UED.checkbox[ck].borderWidth,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: true
            });
        //只有背景;
        }else{
            drawVectorShape({
                dots: dots,
                type: 'fill',
                hexValueFill: UED.checkbox[ck].bgColor,
                closed: true
            });
        }
        layerNum ++;
    }
    //绘制勾选的那个打钩的图标;
    if(obj.checked){
        var txtLeft = withRect ? w.left + (UED.checkbox.base.rectSize - UED.checkbox.checked.iconSize) / 2 : w.left;
        var txtTop = withRect ? w.top + (UED.checkbox.base.rectSize - UED.checkbox.checked.iconSize) / 2 : w.top;
        var iconObj = getTxtLen({
            content: UED.checkbox.checked.icon,
            fontSize: UED.checkbox.checked.iconSize,
            font: UED.font.iconFamily
        });
        addTxtLayer({
            name: 'checked',
            content: UED.checkbox.checked.icon,
            font: UED.font.iconFamily,
            size: UED.checkbox.checked.iconSize,
            color: UED.checkbox.checked.iconColor,
            txtAlign: 'left',
            width: iconObj.width,
            position: [txtLeft, txtTop]
        });
        layerNum ++;
    }
    if(obj.txt && obj.txt.length > 0){
        var txtWidth = getTxtLen({
            content: obj.txt,
            fontSize: UED.checkbox.base.fontSize
        }).width;
        var txtLeft = withRect ? 
            w.left + UED.checkbox.base.rectSize + UED.checkbox.base.iconMargin : 
            w.left;
        var txtTop = (withRect && (UED.checkbox.base.rectSize > UED.checkbox.base.fontSize)) ? 
            w.top + (UED.checkbox.base.rectSize - UED.checkbox.base.fontSize) / 2 :
            w.top;
        addTxtLayer({
            name: obj.txt,
            content: obj.txt,
            font: UED.font.family,
            size: UED.checkbox.base.fontSize,
            color: UED.checkbox.base.fontColor,
            txtAlign: 'left',
            width: txtWidth,
            position: [txtLeft, txtTop]
        });
        layerNum ++;        
    }
    createLayerSet({
        name: '多选框',
        number: layerNum
    });
    //如果需要合并，将这个文件夹合并，主要应用于表格中;
    if(obj.merge){ 
        doc.layerSets[0].merge();
    }
    //如果是绘制树或表格的时候，内部绘制多选框，就不要播放声音;
    if(typeof(notNeedParse) == 'undefined' || notNeedParse == false){
        completeFn();
    }
}
//系统ui控件，绘制tooltip;
function createUedTooltip(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var txt = obj.txt;
    var positon = obj.position;
    var layerNum = 0;    
    var uedTooltip = UED.tooltip;
    var txtObj = getTxtLen({
        content: txt,
        fontSize: uedTooltip.fontSize
    });
    //先画出整个矩形的4个顶点;
    var txtWidth = txtObj.width;
    var txtHeight = txtObj.height;
    var paddingArr = uedTooltip.padding.split(',');
    var paddingLR = parseInt(paddingArr[1], 10) + parseInt(paddingArr[3], 10);
    var paddingTB = parseInt(paddingArr[0], 10) + parseInt(paddingArr[2], 10);
    var tooltipHeight = txtHeight + paddingTB;
    var tooltipWidth = txtWidth + paddingLR;
    var dots = [
        [w.left, w.top], 
        [w.left, w.top + tooltipHeight], 
        [w.left + tooltipWidth, w.top + tooltipHeight], 
        [w.left + tooltipWidth, w.top]
    ];
    //通过position来改变这4个点的位置;
    //上面不需要改变，右侧x都加上triangleHeight，底部y都加上triangleHeight;
    if(positon == 'right' || positon == 'rt' || positon == 'rb'){
        for(var i=0; i<dots.length; i++){
            dots[i][0] += uedTooltip.triangleHeight;
        }
    }else if(positon == 'bottom' || positon == 'bl' || positon == 'br'){
        for(var i=0; i<dots.length; i++){
            dots[i][1] += uedTooltip.triangleHeight;
        }
    }
    //如果圆角值不是0，则需要绘制成圆角;
    if(uedTooltip.borderRadius != '0,0,0,0'){
        var borderArr = splitToNumber(uedTooltip.borderRadius, ',');
        dots = getConnerDots(dots, borderArr);
    }
    var middleX = w.left + tooltipWidth / 2;
    var middleY = w.top + tooltipHeight / 2;
    var dotLen = dots.length;
    //根据位置，决定要绘制小三角的三个点;
    var x1 = Math.tan(angleToRadian(uedTooltip.triangleAngle/2)) * uedTooltip.triangleHeight;
    switch(positon){
        case 'top':
            var p1 = [middleX - x1, w.top + tooltipHeight];
            var p2 = [middleX, w.top + tooltipHeight + uedTooltip.triangleHeight];
            var p3 = [middleX + x1, w.top + tooltipHeight];
            //如果没有圆角，则在位置2添加，如果有圆角，则在位置4添加;
            var insertNum = (dotLen == 4) ? 2 : 4;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'tl':
            var p1 = [w.left + uedTooltip.triangleDistance, w.top + tooltipHeight];
            var p2 = [w.left + uedTooltip.triangleDistance + x1, w.top + tooltipHeight + uedTooltip.triangleHeight];
            var p3 = [w.left + uedTooltip.triangleDistance + x1 + x1, w.top + tooltipHeight];
            //如果没有圆角，则在位置2添加，如果有圆角，则在位置4添加;
            var insertNum = (dotLen == 4) ? 2 : 4;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'tr':
            var p1 = [w.left + tooltipWidth - uedTooltip.triangleDistance - x1 - x1,  w.top + tooltipHeight];
            var p2 = [w.left + tooltipWidth - uedTooltip.triangleDistance - x1, w.top + tooltipHeight + uedTooltip.triangleHeight];
            var p3 = [w.left + tooltipWidth - uedTooltip.triangleDistance, w.top + tooltipHeight];
            //如果没有圆角，则在位置2添加，如果有圆角，则在位置4添加;
            var insertNum = (dotLen == 4) ? 2 : 4;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'right':
            var p1 = [w.left + uedTooltip.triangleHeight, middleY - x1];
            var p2 = [w.left, middleY];
            var p3 = [w.left + uedTooltip.triangleHeight, middleY + x1];
            //如果没有圆角，则在位置1添加，如果有圆角，则在位置2添加;
            var insertNum = (dotLen == 4) ? 1 : 2;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'rt':
            var p1 = [w.left + uedTooltip.triangleHeight, w.top + uedTooltip.triangleDistance];
            var p2 = [w.left, w.top + uedTooltip.triangleDistance + x1];
            var p3 = [w.left + uedTooltip.triangleHeight, w.top + uedTooltip.triangleDistance + x1 + x1];
            //如果没有圆角，则在位置1添加，如果有圆角，则在位置2添加;
            var insertNum = (dotLen == 4) ? 1 : 2;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'rb':
            var p1 = [w.left + uedTooltip.triangleHeight, w.top +  tooltipHeight - uedTooltip.triangleDistance - x1 - x1];
            var p2 = [w.left, w.top + tooltipHeight - uedTooltip.triangleDistance - x1];
            var p3 = [w.left + uedTooltip.triangleHeight, w.top + tooltipHeight - uedTooltip.triangleDistance];
            //如果没有圆角，则在位置1添加，如果有圆角，则在位置2添加;
            var insertNum = (dotLen == 4) ? 1 : 2;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'bottom':
            var p1 = [middleX + x1, w.top + uedTooltip.triangleHeight];
            var p2 = [middleX, w.top];
            var p3 = [middleX - x1, w.top + uedTooltip.triangleHeight];
            dots.push(p1);
            dots.push(p2);
            dots.push(p3);
        break;
        case 'bl':
            var p1 = [w.left + uedTooltip.triangleDistance + x1 + x1, w.top + uedTooltip.triangleHeight];
            var p2 = [w.left + uedTooltip.triangleDistance + x1, w.top];
            var p3 = [w.left + uedTooltip.triangleDistance, w.top + uedTooltip.triangleHeight];
            dots.push(p1);
            dots.push(p2);
            dots.push(p3);
        break;
        case 'br':
            var p1 = [w.left + tooltipWidth - uedTooltip.triangleDistance,  w.top + uedTooltip.triangleHeight];
            var p2 = [w.left + tooltipWidth - uedTooltip.triangleDistance - x1, w.top];
            var p3 = [w.left + tooltipWidth - uedTooltip.triangleDistance - x1 - x1, w.top + uedTooltip.triangleHeight];
            dots.push(p1);
            dots.push(p2);
            dots.push(p3);
        break;
        case 'left':
            var p1 = [w.left + tooltipWidth, middleY + x1];
            var p2 = [w.left + tooltipWidth + uedTooltip.triangleHeight, middleY];
            var p3 = [w.left + tooltipWidth, middleY - x1];
            //如果没有圆角，则在位置3添加，如果有圆角，则在位置6添加;
            var insertNum = (dotLen == 4) ? 3 : 6;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'lt':
            var p1 = [w.left + tooltipWidth, w.top + uedTooltip.triangleDistance + x1 + x1];
            var p2 = [w.left + tooltipWidth + uedTooltip.triangleHeight, w.top + uedTooltip.triangleDistance + x1];
            var p3 = [w.left + tooltipWidth, w.top + uedTooltip.triangleDistance];
            //如果没有圆角，则在位置3添加，如果有圆角，则在位置6添加;
            var insertNum = (dotLen == 4) ? 3 : 6;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
        case 'lb':
            var p1 = [w.left + tooltipWidth, w.top + tooltipHeight - uedTooltip.triangleDistance];
            var p2 = [w.left + tooltipWidth + uedTooltip.triangleHeight, w.top + tooltipHeight - uedTooltip.triangleDistance - x1];
            var p3 = [w.left + tooltipWidth, w.top + tooltipHeight - uedTooltip.triangleDistance - x1 - x1];
            //如果没有圆角，则在位置3添加，如果有圆角，则在位置6添加;
            var insertNum = (dotLen == 4) ? 3 : 6;
            dots.splice(insertNum, 0, p1, p2, p3);
        break;
    }
    //有边框，又有背景色;
    if(uedTooltip.borderWidth > 0 && uedTooltip.bgColor != null){
        addLayer({name: 'tooltip'});
        drawVectorShape({
            dots: dots,
            type: 'all', 
            hexValue: uedTooltip.borderColor,
            hexValueFill: uedTooltip.bgColor,
            solidWidth: uedTooltip.borderWidth,
            angleDot: 'idstrokeStyleRoundJoin',
            strokeStyleAlign: 'strokeStyleAlignInside',
            closed: true
        });
        layerNum ++;
    //只有边框;
    }else if(uedTooltip.borderWidth > 0){
        addLayer({name: 'tooltip'});
        drawVectorShape({
            dots: dots,
            type: 'stroke', 
            hexValue: uedTooltip.borderColor,
            solidWidth: uedTooltip.borderWidth,
            angleDot: 'idstrokeStyleRoundJoin',
            strokeStyleAlign: 'strokeStyleAlignInside',
            closed: true
        });
        layerNum ++;
    //只有背景色;
    }else if(uedTooltip.bgColor != null){
        addLayer({name: 'tooltip'});
        drawVectorShape({
            dots: dots,
            type: 'fill',
            hexValueFill: uedTooltip.bgColor,
            closed: true
        });
        layerNum ++;
    }
    var txtTop =  w.top + parseInt(paddingArr[0], 10);
    if(positon == 'bottom' || positon == 'bl' || positon == 'br'){
        txtTop += uedTooltip.triangleHeight;
    }
    var txtLeft = w.left + parseInt(paddingArr[3], 10);
    if(positon == 'right' || positon == 'rt' || positon == 'rb'){
        txtLeft += uedTooltip.triangleHeight;
    }
    addTxtLayer({
        name: txt,
        content: txt,
        font: UED.font.family,
        size: uedTooltip.fontSize,
        color: uedTooltip.fontColor,
        width: txtWidth,
        txtAlign: 'left',
        position: [txtLeft, txtTop]
    });
    layerNum ++;
    createLayerSet({
        name: '提示',
        number: layerNum
    });
    completeFn();
}
//系统UI控件，绘制Alert;
function createUedAlert(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var layerNum = 0;
    var uedAlert = UED.alert;
    var txt = obj.txt;
    var type = obj.type;
    var withTitle = obj.title ? true : false;
    var withClose = obj.withClose;
    var withIcon = obj.withIcon;
    var txtObj = getTxtLen({
        content: txt,
        fontSize: uedAlert.base.fontSize
    });
    var txtWidth = txtObj.width;
    var txtHeight = txtObj.height;
    var paddingArr = uedAlert.base.padding.split(',');
    var paddingLR = parseInt(paddingArr[1], 10) + parseInt(paddingArr[3], 10);
    var paddingTB = parseInt(paddingArr[0], 10) + parseInt(paddingArr[2], 10);
    var titleWidth = 0;
    var titleHeight = 0;
    var alertWidth = 0;
    var alertHeight = 0;
    var txtObj = getTxtLen({
        content: obj.txt,
        fontSize: uedAlert.base.fontSize
    });
    var iconObj = (withTitle) ?
        getTxtLen({
            content: uedAlert[type].largeIcon,
            fontSize: uedAlert.base.largeIconSize,
            font: UED.font.iconFamily
        }) :
        getTxtLen({
            content: uedAlert[type].smallIcon,
            fontSize: uedAlert.base.smallIconSize,
            font: UED.font.iconFamily
        });


    //画有标题的，大图标;
    if(withTitle){
        var titleObj = getTxtLen({
            content: obj.title,
            fontSize: uedAlert.base.titleFontSize
        });
        titleWidth = titleObj.width;
        titleHeight = titleObj.height;
        alertWidth = withIcon ? 
                     Math.max(titleWidth, txtWidth) + paddingLR + iconObj.width + uedAlert.base.largeIconMargin :
                     Math.max(titleWidth, txtWidth) + paddingLR;
    //没有标题，小图标;
    }else{
        alertWidth = withIcon ?
                     txtWidth + paddingLR + iconObj.width + uedAlert.base.smallIconMargin : 
                     txtWidth + paddingLR;
    }
    if(withClose){
        var colseIconObj = getTxtLen({
            content: uedAlert.base.closeIcon,
            fontSize: uedAlert.base.closeIconSize,
            font: UED.font.iconFamily
        });
        alertWidth += (colseIconObj.width + uedAlert.base.closeIconMargin);
    }
    alertHeight = withTitle ? 
                  uedAlert.base.titleFontMargin + titleHeight + txtObj.height + paddingTB :
                  txtObj.height + paddingTB;
    var dots = [
        [w.left, w.top],
        [w.left, w.top + alertHeight], 
        [w.left + alertWidth, w.top + alertHeight], 
        [w.left + alertWidth, w.top] 
    ];
    //如果圆角值不是0，则需要绘制成圆角;
    if(uedAlert.base.borderRadius != '0,0,0,0'){
        var borderArr = splitToNumber(uedAlert.base.borderRadius, ',');
        dots = getConnerDots(dots, borderArr);
    }    
    var uedType = uedAlert[type];
    if(uedType.bgColor != null && uedAlert.base.borderWidth > 0){
        addLayer({name: 'Alert警告'});
        drawVectorShape({
            dots: dots,
            type: 'all', 
            hexValue: uedType.borderColor,
            hexValueFill: uedType.bgColor,
            solidWidth: uedAlert.base.borderWidth,
            angleDot: 'idstrokeStyleRoundJoin',
            strokeStyleAlign: 'strokeStyleAlignInside',
            closed: true
        });
        layerNum ++;
    }else if(uedType.bgColor != null){
        addLayer({name: 'Alert警告'});
        drawVectorShape({
            dots: dots,
            type: 'fill',
            hexValueFill: uedType.bgColor,
            closed: true
        });
        layerNum ++;
    }else if(uedAlert.base.borderWidth > 0){
        addLayer({name: 'Alert警告'});
        drawVectorShape({
            dots: dots,
            type: 'stroke', 
            hexValue: uedType.borderColor,
            solidWidth: uedAlert.base.borderWidth,
            angleDot: 'idstrokeStyleRoundJoin',
            strokeStyleAlign: 'strokeStyleAlignInside',
            closed: true
        });
        layerNum ++;
    }
    //有标题，绘制大图标和标题;
    if(withTitle){
        if(withIcon){
            var iconLeft = w.left + parseInt(paddingArr[3], 10);
            var iconTop = (txtObj.height + titleHeight + paddingTB - iconObj.height) / 2 + w.top;
            addTxtLayer({
                name: 'Icon-' + obj.type,
                content: uedType.largeIcon,
                font: UED.font.iconFamily,
                size: uedAlert.base.largeIconSize,
                color: uedType.largeIconColor,
                txtAlign: 'left',
                width: iconObj.width,
                position: [iconLeft, iconTop]
            });
            layerNum ++;
        }
        var titleLeft = withIcon ? 
        w.left + parseInt(paddingArr[3], 10) + iconObj.width + uedAlert.base.largeIconMargin :
                        w.left + parseInt(paddingArr[3], 10);
        var titleTop = w.top + parseInt(paddingArr[0], 10);
        addTxtLayer({
            name: obj.title,
            content: obj.title,
            font: UED.font.family,
            size: uedAlert.base.titleFontSize,
            color: uedType.titleColor,
            fauxBold: uedAlert.base.titleFontBolder,
            txtAlign: 'left',
            width: titleWidth * 1.2,
            position: [titleLeft, titleTop]
        });
        layerNum ++;
        var txtLeft = titleLeft;
        var txtTop = w.top + parseInt(paddingArr[0], 10) + titleObj.height + uedAlert.base.titleFontMargin;
        addTxtLayer({
            name: obj.txt,
            content: obj.txt,
            font: UED.font.family,
            size: uedAlert.base.fontSize,
            color: uedType.fontColor,
            txtAlign: 'left',
            width: txtWidth,
            position: [txtLeft, txtTop]
        });
        layerNum ++;
    //没有标题，绘制小图标和正文;
    }else{
        //有小图标;
        if(withIcon){
            addTxtLayer({
                name: 'Icon-' + obj.type,
                content: uedType.smallIcon,
                font: UED.font.iconFamily,
                size: uedAlert.base.smallIconSize,
                color: uedType.smallIconColor,
                txtAlign: 'left',
                width: iconObj.width + uedAlert.base.smallIconMargin,
                position: [
                    w.left + parseInt(paddingArr[3], 10),
                    w.top + parseInt(paddingArr[0], 10)
                ]
            });
            layerNum ++;
        }
        var txtLeft = withIcon ?
        w.left + parseInt(paddingArr[3], 10) + iconObj.width + uedAlert.base.smallIconMargin : 
                      w.left + parseInt(paddingArr[3], 10);
        var txtTop = w.top + parseInt(paddingArr[0], 10);
        addTxtLayer({
            name: obj.txt,
            content: obj.txt,
            font: UED.font.family,
            size: uedAlert.base.fontSize,
            color: uedType.fontColor,
            txtAlign: 'left',
            width: txtWidth,
            position: [txtLeft, txtTop]
        });
        layerNum ++;
    }
    //绘制关闭按钮;
    if(withClose){
        addTxtLayer({
            name: '关闭',
            content: uedAlert.base.closeIcon,
            font: UED.font.iconFamily,
            size: uedAlert.base.closeIconSize,
            color: uedType.closeIconColor,
            txtAlign: 'right',
            width: colseIconObj.width,
            position: [
                w.left + alertWidth - parseInt(paddingArr[1], 10) - colseIconObj.width,
                w.top + parseInt(paddingArr[0], 10)
            ]
        });
        layerNum ++;
    }
    createLayerSet({
        name: '警告',
        number: layerNum
    });
    completeFn();
}

//绘制输入框;
function addTextInput(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();    
    addLayer({
        name: '输入框-背景'
    });
    var dots = [[w.left, w.top], [w.left, w.top+34], [w.left+w.width, w.top+34], [w.left+w.width, w.top]];
    drawShape({
        dots: dots,
        type: 'fill',  
        hexValue: "0f1a1f",
        opacity: 50
    });
    addLayer({
        name: '输入框-边框'
    });
    drawShape({
        dots: dots,
        type: 'stroke',  
        solidWidth: 2,
        hexValue: "075d76",
        location: 'inside'
    });
    var txtColor = '099ea6';
    var layerName = '输入框-文字';
    if(obj.placeholder){
        txtColor = '075d76';
        layerName = '输入框-placeholder'
    }
    var layerNum = 2;
    if(obj.content.length >= 1){
        layerNum = 3;
        addTxtLayer({
            name: layerName,
            content: obj.content,
            font: UED.font.family,
            size: 14,
            color: txtColor,
            txtAlign: 'left',
            position: [w.left+12, w.top+23]
        });
    }
    createLayerSet({
        name: '输入框',
        number: layerNum
    });
    completeFn();
}
//绘制下拉框;
function addSelect(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
   var obj = JSON.parse(obj);
   var doc = app.activeDocument;
   var layer = doc.activeLayer;
   var w = windowBounds = getSelectionBounds();
   doc.selection.deselect();    
   addLayer({
       name: '下拉框-背景'
   });
   var dots = [[w.left, w.top], [w.left, w.top+34], [w.left+w.width, w.top+34], [w.left+w.width, w.top]];
   drawShape({
       dots: dots,
       type: 'fill',  
       hexValue: "0f1a1f",
       opacity: 50
   });
   addLayer({
       name: '下拉框-边框'
   });
   drawShape({
       dots: dots,
       type: 'stroke',  
       solidWidth: 2,
       hexValue: "075d76",
       location: 'inside'
   });
   var txtColor = '099ea6';
   var layerName = '输入框-文字'
   if(obj.placeholder){
       txtColor = '075d76';
       layerName = '输入框-placeholder'
   }
   var layerNum = 3;
   if(obj.content.length >= 1){
       layerNum++;
       addTxtLayer({
            name: layerName,
            content: obj.content,
            font: UED.font.family,
            size: 14,
            color: txtColor,
            txtAlign: 'left',
            position: [w.left+12, w.top+23]
       });
   }
   addTxtLayer({
        name: 'ICON-下拉箭头',
        content: '',
        font: UED.font.iconFamily,
        size: 14,
        color: '099ea6',
        txtAlign: 'right',
        position: [w.right-14, w.top+20]
   });
   createLayerSet({
        name: '下拉框',
        number: layerNum
   });
   completeFn();
}
//绘制单选框;
function addRadio(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();    
    addLayer({
        name: '单选框-圆圈'
    });
    drawCirclePath({
        startX: w.left,
        startY: w.top + 1,
        endX: w.left+16,
        endY: w.top+ 16 +1,
        type: 'stroke',
        location: 'inside',
        hexValue: '075d76',
        solidWidth: 1
    });
    if(!obj.checked){
        drawCirclePath({
            startX: w.left+4,
            startY: w.top+4 +1,
            endX: w.left+4+8,
            endY: w.top+4+8 +1,
            type: 'stroke',
            location: 'inside',
            hexValue: '075d76',
            solidWidth: 1
        });
    }else{
        drawCirclePath({
            startX: w.left+4,
            startY: w.top+4 +1,
            endX: w.left+4+8,
            endY: w.top+4+8 +1,
            type: 'fill',
            hexValue: '075d76',
            opacity: 100
        });
    }
    var layerNum = 1;
    if(obj.content.length >= 1){
        layerNum++;
        addTxtLayer({
            name: obj.content,
            content: obj.content,
            font: UED.font.family,
            size: 14,
            color: '09a7b0',
            txtAlign: 'left',
            position: [w.left+20, w.top+14]
        });
    }
    createLayerSet({
        name: '单选框',
        number: layerNum
    })
    completeFn();
};
//绘制多选框,如果有notNeedParse,则说明传过来的是对象，不需要JSON.parse，也不需要绘制结束播放声音;
function addCheckBox(obj, notNeedParse){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var obj = (notNeedParse == true) ? obj : JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    try{
        var w = windowBounds = getSelectionBounds();
        doc.selection.deselect();    
    }catch(e){}
    //如果没有绘制蚂蚁线，而是直接传递参数来绘制特定位置的多选框(用于表格中);
    if(obj.bounds){
        w = obj.bounds;
    }
    addLayer({
        name: '多选框-背景'
    });
    var dots = [[w.left, w.top], [w.left, w.top+18], [w.left+18, w.top+18], [w.left+18, w.top]];
    drawShape({
        dots: dots,
        type: 'fill',  
        hexValue: "082a45",
        opacity: 50
    });
    addLayer({
        name: '多选框-边框'
    });
    drawShape({
        dots: dots,
        type: 'stroke',  
        solidWidth: 1,
        hexValue: "075d76",
        location: 'inside'
    });
    var layerNum = 2;
    if(obj.checked){
        layerNum++;
        var ckIcon = '';
        var leftPos = 3;
        var topPos = 14;
        if(obj.check_Child_State && obj.check_Child_State == 1){////-1没有子节点，0没有子节点被勾选，1，部分子节点被勾选，2全部子节点被勾选;
            ckIcon = '';
            leftPos = 4;
            topPos = 13;
        }
        addTxtLayer({
            name: 'ICON-选中',
            content: ckIcon,
            font: UED.font.iconFamily,
            size: 12,
            color: '099ea6',
            txtAlign: 'left',
            position: [w.left+leftPos, w.top+topPos]
        });
    }
    if(obj.content.length >= 1){
        layerNum++;
        addTxtLayer({
            name: obj.content,
            content: obj.content,
            font: UED.font.family,
            size: 14,
            color: '09a7b0',
            txtAlign: 'left',
            position: [w.left+22, w.top+14]
        });
    }
    createLayerSet({
        name: '多选框',
        number: layerNum
    })
    //如果需要合并，将这个文件夹合并，主要应用于表格中;
    if(obj.merge){ 
        doc.layerSets[0].merge();
    }
    //如果是绘制树或表格的时候，内部绘制多选框，就不要播放声音;
    if(typeof(notNeedParse) == 'undefined' || notNeedParse == false){
        completeFn();
    }
}
//系统UI控件，添加按钮组;
function createUedBtnGroup(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    
    var obj = JSON.parse(obj);
    var layerNum = 0;
    var selects = obj.selects;
    var selectedArray = selects.split(/[,]+/);
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var btns = obj.btnTxts;
    var startX = w.left;
    var layerNum = 0;
    for(var i=0; i<btns.length; i++){
        (function(i){
            var tempObj = {
                startX: startX,
                txt: btns[i],
                w: w
            }
            //是选中的那个按钮;
            if(parseInt(selects, 10)-1 == i){
                tempObj.selected = true;
            }
            //是第一个按钮;
            if(i == 0){
                tempObj.first = true;
            }
            //是最后一个按钮;
            if(i == btns.length-1){
                tempObj.last = true;
            }
            var returnObj = addOneUedBtnGroup(tempObj);
            startX = returnObj.startX + UED.btnGroup.base.margin;
            layerNum += returnObj.layerNum;
        })(i);
    }
    createLayerSet({
        name: 'buttonGroup',
        number: layerNum
    });
}
//系统UI控件，添加按钮组，添加其中一个按钮;
function addOneUedBtnGroup(obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var BG = UED.btnGroup;
    var base = BG.base;
    var firstBtn = BG.first;
    var lastBtn = BG.last;
    var select = BG.select;
    layerNum = 0;
    //先计算文字的宽度;
    var txtWh = getTxtLen({
        content: obj.txt,
        fontSize: base.fontSize
    });    
    var txtLen = txtWh.width;
    var txtHeight = txtWh.height;
    var paddingArr = base.padding.split(',');
    var paddingLR = parseInt(paddingArr[1], 10) + parseInt(paddingArr[3], 10);
    var paddingTB = parseInt(paddingArr[0], 10) + parseInt(paddingArr[2], 10);
    var dotsR = obj.startX + txtLen + paddingLR;
    var firstBtnWh, lastBtnWh;
    //如果是首尾按钮，并且配有图标，则需要加上这部分距离;
    if(obj.first && typeof(firstBtn.icon) != 'undefined'){
        firstBtnWh = getTxtLen({
            content: firstBtn.icon,
            fontSize: base.fontSize,
            font: UED.font.iconFamily
        });   
        dotsR += (firstBtnWh.width + firstBtn.iconMargin);
    }
    if(obj.last && typeof(lastBtn.icon) != 'undefined'){
        lastBtnWh = getTxtLen({
            content: lastBtn.icon,
            fontSize: base.fontSize,
            font: UED.font.iconFamily
        });   
        dotsR += (lastBtnWh.width + lastBtn.iconMargin);
    }
    var dots = [
        [obj.startX, obj.w.top],
        [obj.startX, obj.w.top + txtHeight + paddingTB],
        [dotsR, obj.w.top + txtHeight + paddingTB],
        [dotsR, obj.w.top]
    ];
    //如果圆角值不是0，则需要绘制成圆角;
    if(obj.first){
        if(firstBtn.borderRadius != '0,0,0,0'){
            var borderArr = splitToNumber(firstBtn.borderRadius, ',');
            dots = getConnerDots(dots, borderArr);
        }
    }else if(obj.last){
        if(lastBtn.borderRadius != '0,0,0,0'){
            var borderArr = splitToNumber(lastBtn.borderRadius, ',');
            dots = getConnerDots(dots, borderArr);
        }
    }else{
        if(base.borderRadius != '0,0,0,0'){
            var borderArr = splitToNumber(base.borderRadius, ',');
            dots = getConnerDots(dots, borderArr);
        }
    }
    //渐变角度控件的角度是从90度开始的，我设计的渐变角度控件是从0度开始的;    
    /*if(typeof(base.bgColor) == 'object'){
        if(base.bgColor != null && /^\d+$/.test(base.bgColor.angle)){
            base.bgColor.angle = correctAngle(base.bgColor.angle);
        }
    }*/
    //如果存在边框,也就是说borderWidth>0, 那么需要绘制边框图层;
    if(base.borderWidth > 0){
        addLayer({
            name: (obj.selected) ? ('btn-Border-'+ base.borderWidth +'px '+ base.borderType +' #' + select.borderColor) : ('btn-Border-'+ base.borderWidth +'px '+ base.borderType +' #' + base.borderColor)
        });
        if(obj.selected){
            if(select.bgColor == 'transparent' || select.bgColor == null){
                drawVectorShape({
                    dots: dots,
                    type: 'stroke', 
                    hexValue: select.borderColor,
                    solidWidth: base.borderWidth,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignInside',
                    strokeType: base.borderType,
                    closed: true
                });
            }else{
                drawVectorShape({
                    dots: dots,
                    type: 'all', 
                    hexValue: select.borderColor,
                    hexValueFill: select.bgColor,
                    solidWidth: base.borderWidth,
                    strokeType: base.borderType,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignInside',
                    closed: true
                });
            }
        }else{
            //也可能存在没有背景，只有边线的按钮;
            if(base.bgColor == 'transparent' || base.bgColor == null){
                drawVectorShape({
                    dots: dots,
                    type: 'stroke', 
                    hexValue: (obj.selected) ? select.borderColor : base.borderColor,
                    solidWidth: base.borderWidth,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignInside',
                    strokeType: base.borderType,
                    closed: true
                });
            }else{
                //有背景，又有边框的按钮;
                drawVectorShape({
                    dots: dots,
                    type: 'all', 
                    hexValue: (obj.selected) ? select.borderColor : base.borderColor,
                    hexValueFill: (obj.selected) ? select.bgColor : base.bgColor,
                    solidWidth: base.borderWidth,
                    strokeType: base.borderType,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignInside',
                    closed: true
                });
            }
        }        
    //不存在边框，那么直需要绘制按钮背景那个色块;
    }else{
        addLayer({
            name: (obj.selected) ? ('btn-Bg-#' + select.bgColor) : ('btn-Bg-#' + base.bgColor)
        })
        var fill = (obj.selected) ? select.bgColor : base.bgColor;
        //没有边框，又没有背景，那么就什么也不画;
        if(fill != null && fill != 'transparent'){
            drawVectorShape({
                dots: dots,
                type: 'fill',
                hexValueFill: fill,
                closed: true
            });
        }
    }
    layerNum++;
    var contentLeftPos = obj.startX + parseInt(paddingArr[3], 10);
    //首按钮,如果要绘制图标;
    if(obj.first && typeof(firstBtn.icon) != 'undefined'){
        contentLeftPos += (firstBtnWh.width + firstBtn.iconMargin);
        var iconLeft = obj.startX + parseInt(paddingArr[3], 10);
        var iconTop = obj.w.top + parseInt(paddingArr[0], 10);
        addTxtLayer({
            name: 'icon',
            font: UED.font.iconFamily,
            color: (obj.selected) ? select.fontColor : base.fontColor,
            size: base.fontSize,
            position: [iconLeft, iconTop],
            txtAlign: 'left',
            content: firstBtn.icon,
            width: firstBtnWh.width + firstBtn.iconMargin
        });
        layerNum++;
    }
    //尾按钮,如果需要绘制图标;
    if(obj.last && typeof(lastBtn.icon) != 'undefined'){
        var iconLeft = dotsR - parseInt(paddingArr[1], 10) - lastBtnWh.width;
        var iconTop = obj.w.top + parseInt(paddingArr[0], 10);
        addTxtLayer({
            name: 'icon',
            font: UED.font.iconFamily,
            color: (obj.selected) ? select.fontColor : base.fontColor,
            size: base.fontSize,
            position: [iconLeft, iconTop],
            txtAlign: 'right',
            content: lastBtn.icon,
            width: lastBtnWh.width + lastBtn.iconMargin
        });
        layerNum++;
    }
    addTxtLayer({
        name: 'btn-text',
        font: UED.font.family,
        color: (obj.selected) ? select.fontColor: base.fontColor,
        size: base.fontSize,
        position: [contentLeftPos, obj.w.top + parseInt(paddingArr[0], 10)],
        txtAlign: 'left',
        content: obj.txt,
        width: txtLen
    });
    layerNum++;
    return {
        startX: dotsR,
        layerNum: layerNum
    }


}
//添加按钮组;
function crteateBtnGroup(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var obj = JSON.parse(obj);
    var layerNum = 0;
    var selects = obj.selects;
    var selectedArray = selects.split(/[,]+/);
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var btns = obj.btnTxts;
    var startX = w.left;
    for(var i=0; i<btns.length; i++){
            var btnBgOpaticy = 30;
            var txtColor = '075d76';
        
            for(var j=0; j<selectedArray.length; j++){ //循环所有选中的数组;
                    if(parseInt(selectedArray[j], 10)-1 == i){
                            btnBgOpaticy = 80;
                            txtColor = '09a7b0';
                    }
            }
            addLayer({name: 'BtnGroup-背景-' + btns[i]});
            //var btnW = btns[i].length * 14 + 24; //padding左边和右边12像素,所以是24;
            //小写字母是8px,大写字母10px,中文14px;
            var btnW = 24;
            for(var j=0; j<btns[i].length; j++){
                var reg = /[\u4e00-\u9fa5]/;
                if(reg.test(btns[i][j])){ //是中文汉字;
                    btnW += 14;
                }else{
                    btnW += 10;
                }
            }
            var dots = [[startX, w.top], [startX, w.top+30], [startX+btnW, w.top+30], [startX+btnW, w.top]]
            drawShape({
                dots: dots,
                type: 'fill', 
                hexValue: "082a45",
                opacity: btnBgOpaticy
            });
            addLayer({name: 'BtnGroup-边框-' + btns[i]});
            drawShape({
                dots: dots,
                type: 'stroke', 
                hexValue: "082a45",
                solidWidth: 2,
                location: 'inside'
            });
            addTxtLayer({
                    name: btns[i],
                    font: UED.font.family,
                    color: txtColor,
                    size: 14,
                    position: [startX + 12, w.top+20],
                    txtAlign: 'left',
                    content: btns[i]
            })    
            startX += btnW - 2;
            layerNum += 3; //每个按钮有3个图层;
    }
    createLayerSet({
        name: 'Btn Group',
        number: layerNum
    });
    completeFn();
}
function createWindow(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var obj = JSON.parse(obj);
    var footBtnType = obj.btnType;
    var inputTitle = obj.title;
    windowBounds = getSelectionBounds();
    //取消掉选区;
    doc.selection.deselect();
    var w = windowBounds;
    addLayer({name: '黑色背景遮罩'});
    var sel = doc.selection.selectAll();
    var bgColor = new SolidColor();
    bgColor.rgb.hexValue = '000000';
    doc.selection.fill(bgColor, ColorBlendMode.NORMAL, 75, false);
    doc.selection.deselect();
    addLayer({name: 'Window外发光底板'});
    //绘制带外发光的黑色填充80的底板;
    drawShape({
        dots: [[w.left + 11, w.top], [w.left, w.top+11], [w.left, w.bottom-14], [w.left+14, w.bottom], [w.right-14, w.bottom], [w.right, w.bottom-14], [w.right, w.top]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "04131a",
        opacity: 100
    })
    doc.activeLayer.fillOpacity = 80;
    addOuterLight();
    //绘制斜条纹图案层;
    addLayer({name: 'Window斜条纹图案'});
    var gap = 5;
    var rectMax = windowBounds.width > windowBounds.height ? windowBounds.width : windowBounds.height;
    var maxStep = rectMax /gap * 1.5;     //bug复制一个图层，将2个图层合并;

    var color = new SolidColor();
    color.rgb.hexValue = '001627';
    app.foregroundColor = color;
    var dots = [];
    for(var i=0; i<=maxStep; i++){
        var p1 = [0, i*gap];
        var p2 = [i*gap, 0];
        if(i%2 == 1){
            dots.push(p1);
            dots.push(p2);
        }else{
            dots.push(p2);            
            dots.push(p1);
        }
    }
    drawShape({
        dots: dots,
        type: 'pencil'
    });

    doc.activeLayer.translate(windowBounds.left, windowBounds.top);
    drawShape({
        dots: [[w.left + 11, w.top], [w.left, w.top+11], [w.left, w.bottom-14], [w.left+14, w.bottom], [w.right-14, w.bottom], [w.right, w.bottom-14], [w.right, w.top]],
        type: 'selection'
    });
    doc.selection.invert();
    doc.selection.clear();
    doc.selection.deselect();

    addLayer({name: 'Window Title'});
    drawShape({
        dots: [[w.left + 11, w.top], [w.left, w.top+11], [w.left, w.top+60], [w.right, w.top+60],  [w.right, w.top]],
        type: 'selection'
    });
    setForegroundColor ('002b37');
    setBackgroundColor ('064e63');
    createLineGradual({
        startY: w.top,
        endY: w.top+60
    });
    doc.selection.deselect();

    //给thead添加杂色 / 1.5  / 平均分布 / 单色;
    doc.activeLayer.applyAddNoise(1.5, NoiseDistribution.UNIFORM , true);
    //创建thead下面2px solid #075d76线条图层
    drawShape({
        dots: [[w.left, w.top+60], [w.left, w.top+62], [w.right, w.top+62], [w.right, w.top+60]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "075d76",
        opacity: 100
    });

    //创建柔光的椭圆渐变alt+点击到底下图层;
    addLayer({name: '柔光椭圆渐变', blendMode: BlendMode.SOFTLIGHT});
    //doc.activeLayer.grouped = true;

    setForegroundColor('ffffff');
    createCirGradual();
    //高度上缩放到30%;
    doc.activeLayer.resize(100, 30, AnchorPosition.MIDDLECENTER);
    //移动到正中心;
    doc.activeLayer.translate(windowBounds.left+windowBounds.width/2-120,  windowBounds.top-60);
    //创建thead选区，反选，删掉不需要的;
    drawShape({
        dots: [[w.left + 11, w.top], [w.left, w.top+11], [w.left, w.top+61], [w.right, w.top+61],  [w.right, w.top]],
        type: 'selection'
    });
    doc.selection.invert();
    doc.selection.clear();
    doc.selection.deselect();

    //创建title上4个装饰菱形;
    addLayer({name: 'thead four rect'});
    drawShape({
        dots: [[w.left+13, w.top+4], [w.left+5, w.top+13], [w.left+104, w.top+13], [w.left+112, w.top+4]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });
    drawShape({
        dots: [[w.left+3, w.top+17],[w.left+3, w.top+21], [w.left+97, w.top+21], [w.left+101, w.top+17]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });
    drawShape({
        dots: [[w.right-110 , w.top+4], [w.right-100, w.top+13], [w.right-58, w.top+13], [w.right-68, w.top+4]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });
    drawShape({
        dots: [[w.right-99 , w.top+17], [w.right-94, w.top+21], [w.right-52, w.top+21], [w.right-57, w.top+17]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });

    //创建标题的文字;
    addLayer({name: inputTitle, kind: LayerKind.TEXT});
    var txtItem = doc.activeLayer.textItem;
    txtItem.contents = inputTitle;
    txtItem.font = UED.font.family;
    txtItem.size = 22;
    txtItem.color.rgb.hexValue ="FFFFFF";
    //txtItem.antiAliasMethod = AntiAlias.STRONG;
    txtItem.justification = Justification.CENTER;
    txtItem.tracking = 100;
    txtItem.position = [w.left + w.width/2, w.top + 39];

    var idsetd = charIDToTypeID( "setd" );
    var desc = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref = new ActionReference();
    var idPrpr = charIDToTypeID( "Prpr" );
    var idAntA = charIDToTypeID( "AntA" );
    ref.putProperty( idPrpr, idAntA );
    var idTxLr = charIDToTypeID( "TxLr" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref.putEnumerated( idTxLr, idOrdn, idTrgt );
    desc.putReference( idnull, ref);
    var idT = charIDToTypeID( "T   " );
    var idAnnt = charIDToTypeID( "Annt" );
    //antiAliasPlatformGray 是“Windows”，antiAliasPlatformLCD 是“Windows LCD”
    var idantiAliasPlatformLCD = stringIDToTypeID( "antiAliasPlatformLCD" );
    desc.putEnumerated( idT, idAnnt, idantiAliasPlatformLCD );

    executeAction( idsetd, desc, DialogModes.NO );

    //创建title左侧高光小竖线条;
    addLayer({name: 'Window Title 左侧高光条'});
    drawShape({
        dots: [[w.left+2, w.top+13], [w.left+2, w.top+60], [w.left+4, w.top+60], [w.left+4, w.top+13]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "085e77",
        opacity: 80
    });


    //创建窗口的tfoot;
    addLayer({name: 'Window Tfooter'});
    drawShape({
        dots: [[w.left, w.bottom-62], [w.left, w.bottom-15], [w.left+14, w.bottom], [w.right-14, w.bottom], [w.right, w.bottom-15], [w.right, w.bottom-62]],
        type: 'selection'
    });
    setForegroundColor ('064e63');
    setBackgroundColor ('053543');
    createLineGradual({
            startY: w.bottom-62,
            endY: w.bottom
    });
    doc.selection.deselect();

    //给tfooter添加杂色 / 1.5  / 平均分布 / 单色;
    doc.activeLayer.applyAddNoise(1.5, NoiseDistribution.UNIFORM , true);

    //创建tfooter上4个装饰菱形;
    addLayer({name: 'tfooter four rect'});
    drawShape({
        dots: [[w.left+5, w.bottom-21], [w.left+5, w.bottom-17], [w.left+103, w.bottom-17], [w.left+98, w.bottom-21]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });
    drawShape({
        dots: [[w.left+6, w.bottom-13], [w.left+14, w.bottom-4], [w.left+114, w.bottom-4], [w.left+106, w.bottom-13]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });
    drawShape({
        dots: [[w.right-105, w.bottom-13], [w.right-114, w.bottom-4], [w.right-15, w.bottom-4], [w.right-6, w.bottom-13]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });
    drawShape({
        dots: [[w.right-97, w.bottom-21], [w.right-102, w.bottom-17], [w.right-5, w.bottom-17], [w.right-5, w.bottom-21]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 30
    });

    //创建tfooter上高光线 柔光40%的不透明度;
    addLayer({name: 'tfooter highlight line'});
    drawShape({
        dots: [[w.left+2, w.bottom-62], [w.left+2, w.bottom-60], [w.right-2, w.bottom-60], [w.right-2, w.bottom-62]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "ffffff",
        opacity: 40
    });
    doc.activeLayer.blendMode = BlendMode.SOFTLIGHT;

    //创建tfooter上暗部黑线;
    addLayer({name: 'tfooter dark line'});
    drawShape({
        dots: [[w.left, w.bottom-64], [w.left+2, w.bottom-62], [w.right-2, w.bottom-62], [w.right-2, w.bottom-64]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "000000",
        opacity: 100
    });

    //创建tfooer上2个渐变竖条，都是白色柔光，40%的不透明度;
    addLayer({name: 'tfooter left&right highlight line'});
    drawShape({
        dots: [[w.left+2, w.bottom-60], [w.left+2, w.bottom-14], [w.left+4, w.bottom-14], [w.left+4, w.bottom-60]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "ffffff",
        opacity: 40
    });
    drawShape({
        dots: [[w.right-4, w.bottom-60], [w.right-4, w.bottom-14], [w.right-2, w.bottom-14], [w.right-2, w.bottom-60]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "ffffff",
        opacity: 40
    });
    doc.activeLayer.blendMode = BlendMode.SOFTLIGHT;


    //创建最外层2px solid #075d76 蓝色边框;
    addLayer({name: 'Window outerLine'});
    drawShape({
        dots: [[w.left + 11, w.top], [w.left, w.top+11], [w.left, w.bottom-14], [w.left+14, w.bottom], [w.right-14, w.bottom], [w.right, w.bottom-14], [w.right, w.top]],
        type: 'stroke',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: '075d76',
        solidWidth: 2,
        location: 'outside' //分别是outside,inside,center
    });

    //创建内层2px solid #000000 黑色边框;
    addLayer({name: 'Window innerLine'});
    drawShape({
        dots: [[w.left + 11, w.top], [w.left, w.top+11], [w.left, w.bottom-14], [w.left+14, w.bottom], [w.right-14, w.bottom], [w.right, w.bottom-14], [w.right, w.top]],
        type: 'stroke',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: '000000',
        solidWidth: 2,
        location: 'inside' //分别是outside,inside,center
    });

    //创建复杂蓝黑色装饰角;
    addLayer({name: '左上角装饰角'});
    drawShape({
        dots: [[w.left-6, w.top+5], [w.left-6, w.top+21], [w.left, w.top+25], [w.left, w.top+19], [w.left-1, w.top+19],[w.left-1, w.top+13], [w.left+14, w.top-2], [w.left+21, w.top-2], [w.left+21, w.top], [w.left+27, w.top],
            [w.left+23, w.top-6], [w.left+5, w.top-6]
        ],
        type: 'fill',
        hexValue: '075d76',
        opacity: 100
    });
    drawShape({
        dots: [[w.left-6, w.top+9], [w.left-6, w.top+13], [w.left+14, w.top-6], [w.left+9, w.top-6]
        ],
        type: 'fill',
        hexValue: '000000',
        opacity: 100
    });
    drawShape({
        dots: [[w.left-6, w.top+17], [w.left-6, w.top+19], [w.left-1, w.top+19], [w.left-1, w.top+17]
        ],
        type: 'fill',
        hexValue: '000000',
        opacity: 100
    });
    drawShape({
        dots: [[w.left+19, w.top-6], [w.left+19, w.top-2], [w.left+21, w.top-2], [w.left+21, w.top-6]],
        type: 'fill',
        hexValue: '000000',
        opacity: 100
    });
    var l = doc.activeLayer.duplicate(doc.layers[0], ElementPlacement.PLACEBEFORE);
    l.name = "左下角装饰角";
    l.rotate(-90, AnchorPosition.MIDDLECENTER);
    l.translate(-1,windowBounds.height-19);
    var l2 = doc.activeLayer.duplicate(doc.layers[0], ElementPlacement.PLACEBEFORE);
    l2.name = "右下角装饰角";
    l2.rotate(180, AnchorPosition.MIDDLECENTER);
    l2.translate(windowBounds.width-21, windowBounds.height-19);


    //创建关闭红色斜角;
    addLayer({name: 'Red CloseBg'});
    drawShape({
        dots: [[w.right-75, w.top-5], 
            [w.right+5, w.top+75], 
        {
            anchor: [w.right+5, w.top+3],
            left: [w.right+5, w.top-2],
            right: [w.right+5, w.top+3]
        },{
            anchor: [w.right-3, w.top-5],
            left: [w.right-3, w.top-5],
            right: [w.right+2, w.top-5]
        }],
        type: 'fill',
        hexValue: "6e1d1c",
        opacity: 100
    });

    //创建关闭浅红色内部线条;
    addLayer({name: 'Red Close line'});
    drawShape({
        dots: [[w.right-66, w.top-2], 
            [w.right+2, w.top+66], 
        {
            anchor: [w.right+2, w.top+6],
            left: [w.right+2, w.top-2],
            right: [w.right+2, w.top+6]
        },{
            anchor: [w.right-6, w.top-2],
            left: [w.right-6, w.top-2],
            right: [w.right-2, w.top-2]
        }],
        type: 'stroke',
        hexValue: "df7c79",
        solidWidth: 2,
        location: 'inside' //分别是outside,inside,center
    });

    //创建×图标;
    addLayer({name: 'closeIcon'});
    drawShape({
        dots: [[w.right-20, w.top+9], [w.right-20, w.top+16], [w.right-28, w.top+16], [w.right-28, w.top+19], [w.right-20, w.top+19], [w.right-20, w.top+27]
        , [w.right-17, w.top+27], [w.right-17, w.top+19], [w.right-10, w.top+19], [w.right-10, w.top+16], [w.right-17, w.top+16]
        , [w.right-17, w.top+9]],
        type: 'fill',  //分别是fill或者stroke,填充区域或者绘制线条;
        hexValue: "ffffff",
        opacity: 100
    });
    doc.activeLayer.rotate(45, AnchorPosition.MIDDLECENTER);
    //给关闭按钮加个外发光;
    addCloseOuterLight();
    
    createLayerSet({
        name: 'Window',
        number: 20
    });
    if(footBtnType == 'close'){
        addBtn({
            iconTxt: '',
            content: '关闭',
            type: 'cancel'
        });
        createLayerSet({
            name: 'Close Btn',
            number: 4
        });
        doc.layerSets[0].translate(windowBounds.left + windowBounds.width/2 -46, windowBounds.bottom-46);
    }else{
        addBtn({
            iconTxt: '',
            content: '确定',
            type: 'ok'
        });
        createLayerSet({
            name: 'OK Btn',
            number: 4
        });
        doc.layerSets[0].translate(windowBounds.left + windowBounds.width/2 - 94, windowBounds.bottom-46);
        
        addBtn({
            iconTxt: '',
            content: '取消',
            type: 'cancel'
        })
        createLayerSet({
            name: 'Cancel Btn',
            number: 4
        });
        doc.layerSets[0].translate(windowBounds.left + windowBounds.width/2 + 5, windowBounds.bottom-46);
    }
    completeFn();
}
function createUedTree(withCkBox, treeJson, jsonHasLevel){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var treeJson = JSON.parse(treeJson);
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();   
    //对比多选框的高度和tree文字的高度;
    uedTreeSetting.top = Math.max(UED.checkbox.base.rectSize, UED.tree.fontSize, UED.tree.openCloseIconSize) + UED.tree.levelMarginBottom;
    uedTreeSetting.left = UED.tree.levelMarginLeft;
    //先给所有的数据加上level,ztree传过来的参数自带level所以不需要去循环加level;
    if(jsonHasLevel != true){
        putJsonLevel(treeJson.nodes);
    }
    for(var i=0; i<treeJson.nodes.length; i++){
        getUedPostion(treeJson.nodes[i], w)
    }   
    uedTreeSetting.layerNum = 0;
    for(var i=0; i<treeJson.nodes.length; i++){
        //alert(JSON.stringify(treeJson.nodes[i]))
        drawUedTreeNode(treeJson.nodes[i], w, withCkBox);
    }   
    createLayerSet({
        name: 'Tree',
        number: uedTreeSetting.layerNum
    });
    uedTreeSetting.treeTop = 0;        
    uedTreeSetting.layerNum = 0;
    completeFn();
}

//参数代表是否带复选框;
function createTree(withCkBox, treeJson, jsonHasLevel){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var treeJson = JSON.parse(treeJson);
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();        
    //先给所有的数据加上level,ztree传过来的参数自带level所以不需要去循环加level;
    if(jsonHasLevel != true){
        putJsonLevel(treeJson.nodes);
    }
    for(var i=0; i<treeJson.nodes.length; i++){
        getPostion(treeJson.nodes[i], w)
    }   
    treeSetting.layerNum = 0;
    for(var i=0; i<treeJson.nodes.length; i++){
       drawTreeNode(treeJson.nodes[i], w, withCkBox);
    }   
    createLayerSet({
        name: 'Tree',
        number: treeSetting.layerNum
    });
    treeSetting.treeTop = 0;        
    treeSetting.layerNum = 0;
    completeFn();
}
function drawUedTreeNode(obj, w, withCkBox, notRoot){
    var _tree = UED.tree;
    var maxHeight = Math.max(_tree.openCloseIconSize, _tree.fontSize, UED.checkbox.base.rectSize);
    //只要不是最顶级菜单，则每个标签前面都要绘制一个短的虚线;
    if(notRoot && _tree.withLine){
        uedTreeSetting.layerNum++;
        addLayer({name: 'Tree-hLine-'+obj.name});
        var leftPos = _tree.levelMarginLeft - _tree.openCloseIconSize / 2;
        var topPos = uedTreeSetting.top / 2 - _tree.levelMarginBottom / 2;
        var hLineArr = [
                        [obj.left - leftPos, obj.top + topPos], 
                        [obj.left, obj.top + topPos]
                    ];
        drawVectorShape({
            dots: hLineArr,
            type: 'stroke',
            hexValue: _tree.lineColor,
            solidWidth: 1,
            angleDot: 'idstrokeStyleRoundJoin',
            strokeStyleAlign: 'strokeStyleAlignInside',
            strokeType: _tree.lineType,
            close: false
        });
    }    
    //有子节点，绘制下拉展开的图标;
    if(obj.children){
        uedTreeSetting.layerNum++;
        var txtTop = (maxHeight == _tree.openCloseIconSize) ? 
                     0 :
                     (maxHeight - _tree.openCloseIconSize) / 2;
        addTxtLayer({
            name: 'Tree-Icon-open',
            font: UED.font.iconFamily,
            color: _tree.openIconColor,
            size: _tree.openCloseIconSize,
            width: _tree.openCloseIconSize,
            position: [obj.left, obj.top + txtTop],
            txtAlign: 'left',
            content: _tree.openIcon
        });
    }
    //带有层级复选框;
    var ck = (obj.checked) ? true : false;
    if(withCkBox){
        var boundsLeft = obj.children ?
                         obj.left + _tree.openCloseIconSize + _tree.iconMargin :
                         obj.left ;
        var ckObj = {
            bounds: {
                left: boundsLeft,
                top: obj.top
            },
            checked: ck,
            merge: true,
            notNeedParse: true
        };
        var str = JSON.stringify(ckObj);
        createUedCheckbox(str);
        uedTreeSetting.layerNum++;
    }

    //绘制文字;
    var leftPos = withCkBox ? 
                  obj.left + _tree.fontMarginLeft + UED.checkbox.base.rectSize : 
                  obj.left;
    if(obj.children){
        leftPos += (_tree.openCloseIconSize + _tree.iconMargin);
    }

    var topPos = (maxHeight == _tree.fontSize) ?
                 0 :
                 (maxHeight - _tree.fontSize) / 2;

    var txtWidth = getTxtLen({
        content: obj.name,
        fontSize: _tree.fontSize
    }).width;
    addTxtLayer({
        name: obj.name,
        font: UED.font.family,
        color: _tree.fontColor,
        size: _tree.fontSize,
        position: [leftPos, obj.top + topPos],
        txtAlign: 'left',
        width: txtWidth,
        content: obj.name
    });
    uedTreeSetting.layerNum++;
    if(obj.children){
        for (var i = 0; i < obj.children.length; i++){
            drawUedTreeNode(obj.children[i], w, withCkBox, true); //notRoot表示不是第一级别的item;
            if(i == (obj.children.length-1) && _tree.withLine){ //有子节点的那些父级,需要以他们为起始点向下画线;
                var lastChild = obj.children[obj.children.length-1];
                uedTreeSetting.layerNum++;
                addLayer({name: 'Tree-vLine-'+obj.name+'-'+ lastChild.name});
                var leftPos = _tree.openCloseIconSize / 2 - 1;
                var topPos = (maxHeight - _tree.openCloseIconSize)/2 + _tree.openCloseIconSize;
                var vLineArr = [
                    [obj.left+leftPos, obj.top+topPos], 
                    [obj.left+leftPos, lastChild.top+maxHeight/2+1]
                    //, 
                    //[obj.left+leftPos+1, lastChild.top+maxHeight/2+1], 
                    //[obj.left+leftPos+1, obj.top+topPos]
                ];
                drawVectorShape({
                    dots: vLineArr,
                    type: 'stroke',
                    hexValue: _tree.lineColor,
                    solidWidth: 1,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignOuterSide',
                    strokeType: _tree.lineType,
                    close: false
                });

                /*drawShape({
                    dots: vLineArr,
                    type: 'fill', 
                    hexValue: _tree.lineColor,
                    opacity: 100
                });   
                if(_tree.lineType == 'dashed'){
                    //每隔4个像素创建一个2个像素的选区，然后删掉这个选区，再取消选择;
                    vlineToDashed({
                        top: obj.top + leftPos,
                        right: obj.left + leftPos+1,
                        bottom: lastChild.top + maxHeight/2+1,
                        left: obj.left + leftPos
                    });                          
                }*/
                
            }
        }
    }
}
function drawTreeNode(obj, w, withCkBox, notRoot){
    var bgColor = '082a45';
    var txtColor = '09a7b0';
    if(obj.active){
        bgColor = '8c0202';
        txtColor = 'ffffff';
    }
    treeSetting.layerNum++;
    //绘制底部的图形;
    addLayer({name: 'Tree-Item-'+obj.name});
    var txtWidth = obj.name.toString().length * 12 + 44;  //44是因为左右padding:22px;
    if(txtWidth < 120){ txtWidth = 120}
    if(withCkBox){ txtWidth += 22};
    var treeItemBgs = [[obj.left, obj.top], [obj.left, obj.top+20], 
        {
            anchor: [obj.left+txtWidth-8, obj.top+20],
            left: [obj.left+txtWidth, obj.top+20],
            right: [obj.left+txtWidth-8, obj.top+20]
        },{
            anchor: [obj.left+txtWidth, obj.top+12],
            left: [obj.left+txtWidth, obj.top+4],
            right: [obj.left+txtWidth, obj.top+12]
        },[obj.left+txtWidth,  obj.top]
    ]
    drawShape({
        dots: treeItemBgs,
        type: 'fill', 
        hexValue: bgColor,
        opacity: 100
    });   
   
    //只要不是最顶级菜单，则每个标签前面都要绘制一个短的虚线;
    if(notRoot){
        treeSetting.layerNum++;
        addLayer({name: 'Tree-hLine-'+obj.name});
        var hLineArr = [[obj.left-11, obj.top+10], [obj.left-11, obj.top+11], [obj.left, obj.top+11], [obj.left, obj.top+10]];
        drawShape({
            dots: hLineArr,
            type: 'fill', 
            hexValue: '075d76',
            opacity: 100
       });          
       //横向虚线，4个实体像素，2个空白像素;
       hlineToDashed({
            top: obj.top+10,
            right: obj.left,
            bottom: obj.top+11,
            left: obj.left-11
       });
    }
    //绘制下拉展开的箭头;
    if(obj.children){
        treeSetting.layerNum++;
        addTxtLayer({
            name: 'Tree-Icon-open',
            font: UED.font.iconFamily,
            color: '075d76',
            size: 12,
            position: [obj.left+8, obj.top+14],
            txtAlign: 'left',
            content: ''
       })   
   }
   //带有层级复选框;
   var ck = (obj.checked) ? true : false;
   if(withCkBox){
        addCheckBox({
            bounds: {
                left: obj.left+22,
                top: obj.top + 1
            },
            checked: ck,
            check_Child_State: obj.check_Child_State,
            content: '',
            merge: true
        }, true);
        treeSetting.layerNum++;
    }
    //绘制文字
    treeSetting.layerNum++;
    var leftPos =withCkBox ? obj.left + 22 + 22 : obj.left+22;
    addTxtLayer({
        name: obj.name,
        font: UED.font.family,
        color: txtColor,
        size: 12,
        position: [leftPos, obj.top+14],
        txtAlign: 'left',
        content: obj.name
    })   
    if(obj.children){
        for (var i = 0; i < obj.children.length; i++){
            drawTreeNode(obj.children[i], w, withCkBox, true); //notRoot表示不是第一级别的item;
            if(i == (obj.children.length-1)){ //有子节点的那些父级,需要以他们为起始点向下画线;
                var lastChild = obj.children[obj.children.length-1];
                treeSetting.layerNum++;
                addLayer({name: 'Tree-vLine-'+obj.name+'-'+ lastChild.name});
                var vLineArr = [[obj.left+11, obj.top+20], [obj.left+11, lastChild.top+11], [obj.left+12, lastChild.top+11], [obj.left+12, obj.top+20]];
                drawShape({
                    dots: vLineArr,
                    type: 'fill', 
                    hexValue: '075d76',
                    opacity: 100
                });   
                //每隔4个像素创建一个2个像素的选区，然后删掉这个选区，再取消选择;
                vlineToDashed({
                    top: obj.top+20,
                    right: obj.left+12,
                    bottom: lastChild.top+11,
                    left: obj.left+11
                });                          
            }
        }
    }
}
//添加tooltip;
function addTooltip(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var obj = JSON.parse(obj);
    var bgColor, outerLineColor, lineColor, iconTxt;
    switch(obj.type){
        case 'yellow':
            bgColor = 'ee8800';
            outerLineColor = '221400'
            lineColor = 'be6d00';
            iconTxt = '';
            break;
        case 'green':
            bgColor = '00a157';
            outerLineColor = '00241b';
            lineColor = '00894a';
            iconTxt = '';
            break;
        case 'red':
            bgColor = '8c0202';
            outerLineColor = '280000'
            lineColor = '700202';
            iconTxt = '';
            break;
    }
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();    
    var tooltipWidth = 14 * obj.content.length + 80;
    addLayer({
        name: 'Tooltip-背景'
    });
    var points = [[w.left+2, w.top], [w.left, w.top+2],[w.left, w.top+30],[w.left+14, w.top+44],[w.left+tooltipWidth-2, w.top+44],[w.left+tooltipWidth, w.top+44-2],[w.left+tooltipWidth, w.top+14],[w.left+tooltipWidth-14, w.top]];
    drawShape({
        dots: points,
        type: 'fill',  
        hexValue: bgColor,
        opacity: 100
    });
    //加外发光;
    addOuterLight();
    //加2px 深色描边(外);
    addLayer({
        name: 'Tooltip-2px外描边'
    });
    drawShape({
        dots: points,
        type: 'stroke',  
        hexValue: outerLineColor,
        location: 'outside',
        solidWidth: 2
    });
    //加入斜条纹;
    addLayer({name: 'Tooltip-斜条纹图案'});
    var gap = 3;
    var rectMax = w.left+tooltipWidth;
    var maxStep = rectMax /gap * 1.5;     //bug复制一个图层，将2个图层合并;

    var color = new SolidColor();
    color.rgb.hexValue = lineColor;
    app.foregroundColor = color;

    var dots = [];
    for(var i=0; i<=maxStep; i++){
        var p1 = [0, i*gap];
        var p2 = [i*gap, 0];
        if(i%2 == 1){
            dots.push(p1);
            dots.push(p2);
        }else{
            dots.push(p2);            
            dots.push(p1);
        }
    }
    drawShape({
        dots: dots,
        type: 'pencil'
    });

    doc.activeLayer.translate(w.left-2, w.top-2);
    drawShape({
        dots: points,
        type: 'selection'
    })
    doc.selection.invert();
    doc.selection.clear();
    doc.selection.deselect();
    
    //高亮描边;
    addLayer({
        name: 'Tooltip-1px高亮内描边'
    });
    doc.activeLayer.blendMode = BlendMode.SOFTLIGHT;
    drawShape({
        dots: points,
        type: 'stroke',  
        hexValue: "ffffff",
        location: 'inside',
        solidWidth: 1
    });
    //添加图标;
    addTxtLayer({
        name: 'Tooltip-Icon',
        content: iconTxt,
        font: UED.font.iconFamily,
        size: 18,
        color: 'ffffff',
        txtAlign: 'left',
        position: [w.left+21, w.top+28]
    });
    addTxtLayer({
        name: 'Tooltip-提示文字',
        font: UED.font.family,
        color: 'ffffff',
        size: 14,
        position: [w.left+50, w.top+26],
        txtAlign: 'left',
        content: obj.content
    });
    createLayerSet({
        name: 'Tooltip-'+obj.type,
        number: 6
    });
    completeFn();
}
//系统UI板块，绘制modal;
function createUedModal(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();  
    var layerNum = 0; 
    var modalBase = UED.modal.base;
    var modalTh = UED.modal.th;
    var modalTfoot = UED.modal.tfoot;
    var modalTfootBtns = UED.modal.tfootBtns;
    var dots = [
        [w.left, w.top],
        [w.left, w.bottom],
        [w.right, w.bottom],
        [w.right, w.top]
    ];
    if(modalBase.borderRadius != '0,0,0,0'){
        var borderArr = splitToNumber(modalBase.borderRadius, ',');
        dots = getConnerDots(dots, borderArr);
    }
    //先绘制整个弹窗的背景颜色;
    if(modalBase.bgColor != null){
        addLayer({
            name: 'modal-bg'
        });
        drawVectorShape({
            dots: dots,
            type: 'fill',
            hexValueFill: modalBase.bgColor,
            closed: true
        });
        layerNum ++;
    }
    //绘制头部;
    var thPaddingArr = modalTh.padding.split(',');
    var thPaddingLR = parseInt(thPaddingArr[1], 10) + parseInt(thPaddingArr[3], 10);
    var thPaddingTB = parseInt(thPaddingArr[0], 10) + parseInt(thPaddingArr[2], 10);
    var thObj = getTxtLen({
        content: json.title,
        fontSize: modalTh.fontSize,
        fauxBold: modalTh.fontBolder
    });
    var thHeight = thObj.height + thPaddingTB;
    var thDots = [
        [w.left, w.top],
        [w.left, w.top + thHeight],
        [w.right, w.top + thHeight],
        [w.right, w.top]
    ];
    if(modalBase.borderRadius != '0,0,0,0'){
        var thBorderRadius = modalBase.borderRadius.slice(0,4)+'0,0';
        var borderArr = splitToNumber(thBorderRadius, ',');
        thDots = getConnerDots(thDots, borderArr);
    }
    //只需要绘制背景，并不需要绘制边线;
    if(modalTh.bgColor != null){
        addLayer({
            name: 'modal-thead'
        });
        drawVectorShape({
            dots: thDots,
            type: 'fill',
            hexValueFill: modalTh.bgColor,
            closed: true
        });
        layerNum ++;
    }
    
    //绘制标题文字;
    var topPos = w.top + parseInt(thPaddingArr[0], 10);
    if(modalTh.align == 'left'){
        var txtWidth = thObj.width;
        var leftPos = w.left + parseInt(thPaddingArr[3], 10);
        addTxtLayer({
            name: json.title,
            font: UED.font.family,
            fauxBold: modalTh.fontBolder,
            color: modalTh.fontColor,
            size: modalTh.fontSize,
            position: [leftPos, topPos],
            txtAlign: 'left',
            content: json.title,
            width: txtWidth * 1.2
        });
    }else if(modalTh.align == 'center'){
        var txtWidth = modalTh.closeIconShow ? w.width - modalTh.closeIconSize * 2 : w.width;
        var leftPos = modalTh.closeIconShow ? w.left + modalTh.closeIconSize : w.left;
        addTxtLayer({
            name: json.title,
            font: UED.font.family,
            fauxBold: modalTh.fontBolder,
            color: modalTh.fontColor,
            size: modalTh.fontSize,
            position: [leftPos, topPos],
            txtAlign: 'center',
            content: json.title,
            width: txtWidth
        });
    }
    layerNum ++;
    //绘制关闭按钮;
    if(modalTh.closeIconShow){
        var closeIconObj = getTxtLen({
            content: modalTh.closeIcon,
            fontSize: modalTh.closeIconSize,
            font: UED.font.iconFamily
        });
        var iconLeftPos = w.right - parseInt(thPaddingArr[1], 10) - closeIconObj.width;
        var iconTopPos = w.top + parseInt(thPaddingArr[0], 10);
        
        addTxtLayer({
            name: 'close',
            font: UED.font.iconFamily,
            color: modalTh.closeIconColor,
            size: modalTh.closeIconSize,
            position: [iconLeftPos, iconTopPos],
            txtAlign: 'right',
            content: modalTh.closeIcon,
            width: closeIconObj.width
        });
        layerNum ++;
    }
    //绘制表头的下边框;
    if(modalTh.borderWidth > 0){
        var lineDots = [
            [w.left + modalBase.borderWidth, w.top + thHeight],
            [w.left + modalBase.borderWidth, w.top + thHeight + modalTh.borderWidth],
            [w.right - modalBase.borderWidth, w.top + thHeight + modalTh.borderWidth],
            [w.right - modalBase.borderWidth, w.top + thHeight]
        ];
        addLayer({
            name: 'modal-thead-line'
        });
        drawVectorShape({
            dots: lineDots,
            type: 'fill',
            hexValueFill: modalTh.borderColor,
            closed: true
        });
        layerNum ++;
    }
    
    var tfootPaddingArr = modalTfoot.padding.split(',');
    var tfootPaddingLR = parseInt(tfootPaddingArr[1], 10) + parseInt(tfootPaddingArr[3], 10);
    var tfootPaddingTB = parseInt(tfootPaddingArr[0], 10) + parseInt(tfootPaddingArr[2], 10);

    var btnPaddingArr = UED.btn.normal.padding.split(',');
    var btnPaddingLR = parseInt(btnPaddingArr[1], 10) + parseInt(btnPaddingArr[3], 10);
    var btnPaddingTB = parseInt(btnPaddingArr[0], 10) + parseInt(btnPaddingArr[2], 10);
    var btnTxtObj = getTxtLen({
        content: UED.modal.tfootBtns[0].txt + UED.modal.tfootBtns[1].txt,
        fontSize: UED.btn.normal.fontSize
    });
    var tfootHeight = btnTxtObj.height + btnPaddingTB + tfootPaddingTB;
    if(modalTfoot.show){
        //绘制底部;
        if(modalTfoot.bgColor != null){
            var tfootDots = [
                [w.left, w.bottom - tfootHeight],
                [w.left, w.bottom],
                [w.right, w.bottom],
                [w.right, w.bottom - tfootHeight]
            ]
            if(modalBase.borderRadius != '0,0,0,0'){
                var tfootBorderRadius = '0,0,' + modalBase.borderRadius.slice(4);
                var borderArr = splitToNumber(tfootBorderRadius, ',');
                tfootDots = getConnerDots(tfootDots, borderArr);
            }
            addLayer({
                name: 'modal-tfoot'
            });
            drawVectorShape({
                dots: tfootDots,
                type: 'fill',
                hexValueFill: modalTfoot.bgColor,
                closed: true
            });
            layerNum ++;
        }
        //绘制tfoot上面的线条;
        if(modalTfoot.borderWidth > 0){
            var footLineDots = [
                [w.left + modalBase.borderWidth, w.bottom - tfootHeight - modalTfoot.borderWidth],
                [w.left + modalBase.borderWidth, w.bottom - tfootHeight],
                [w.right - modalBase.borderWidth, w.bottom - tfootHeight],
                [w.right - modalBase.borderWidth, w.bottom - tfootHeight - modalTfoot.borderWidth]
            ];
            addLayer({
                name: 'modal-tfoot-line'
            });
            drawVectorShape({
                dots: footLineDots,
                type: 'fill',
                hexValueFill: modalTfoot.borderColor,
                closed: true
            });
            layerNum ++;
        }
    }
    //如果整个弹窗有边框，则绘制边框;
    if(modalBase.borderWidth > 0){
        addLayer({
            name: 'modal-border'
        });
        drawVectorShape({
            dots: dots,
            type: 'stroke', 
            hexValue: modalBase.borderColor,
            solidWidth: modalBase.borderWidth,
            angleDot: 'idstrokeStyleRoundJoin',
            strokeStyleAlign: 'strokeStyleAlignInside',
            closed: true
        });
        layerNum ++;
    }
    createLayerSet({
        name: '对话框 Modal',
        number: layerNum
    });
    if(modalTfoot.show == false){ return;}
    
    //绘制按钮;
    if(modalTfoot.btnShow == '0'){
        return;
    }else if(modalTfoot.btnShow == '1' || modalTfoot.btnShow == '2'){
        var index = parseInt(modalTfoot.btnShow, 10) - 1;
        var _type = modalTfootBtns[index].type;
        var _w = getTxtLen({
            content: modalTfootBtns[index].txt,
            fontSize: UED.btn[_type].fontSize
        }).width;
        var btnPaddingArr = UED.btn[_type].padding.split(',');
        var btnPaddingLR = parseInt(btnPaddingArr[1], 10) + parseInt(btnPaddingArr[3], 10);
        _w += btnPaddingLR;
        switch(modalTfoot.align){
            case 'left':
                var leftPos = w.left + parseInt(tfootPaddingArr[3], 10);                                
                break;
            case 'center':
                var leftPos = w.left + (w.width - _w)/2;
                break;
            case 'right':
                var leftPos = w.right - parseInt(tfootPaddingArr[1], 10) - _w;
                break;
        }
        var topPos = w.bottom - tfootHeight + parseInt(tfootPaddingArr[0], 10);
        var paraObj = {
            bounds: {
                left: leftPos,
                top: topPos
            },
            content: modalTfootBtns[index].txt,
            type: modalTfootBtns[index].type,
            icon: ''
        }
        var str = JSON.stringify(paraObj);
        createUedBtn(paraObj);
    }else if(modalTfoot.btnShow == 'all'){
        var twoBtnWidth = [];
        for(var i=0; i<2; i++){
            var _type = modalTfootBtns[i].type;
            var _w = getTxtLen({
                content: modalTfootBtns[i].txt,
                fontSize: UED.btn[_type].fontSize
            }).width;
            var btnPaddingArr = UED.btn[_type].padding.split(',');
            var thPaddingLR = parseInt(btnPaddingArr[1], 10) + parseInt(btnPaddingArr[3], 10);
            _w += thPaddingLR;
            twoBtnWidth.push(_w);
        }
        for(var i=0; i<2; i++){
            var _type = modalTfootBtns[i].type;
            var _w = getTxtLen({
                content: modalTfootBtns[i].txt,
                fontSize: UED.btn[_type].fontSize
            }).width;
            var btnPaddingArr = UED.btn[_type].padding.split(',');
            var thPaddingLR = parseInt(btnPaddingArr[1], 10) + parseInt(btnPaddingArr[3], 10);
            _w += thPaddingLR;
            switch(modalTfoot.align){
                case 'left':
                    var leftPos = w.left + parseInt(tfootPaddingArr[3], 10);
                    if(i>0){
                        leftPos += (twoBtnWidth[0] + modalTfootBtns[2]);
                    }                        
                    break;
                case 'center':
                    var leftPos = w.left + (w.width - twoBtnWidth[0] - twoBtnWidth[1] - modalTfootBtns[2])/2;
                    if(i>0){
                        leftPos += (twoBtnWidth[0] + modalTfootBtns[2]);
                    } 
                    break;
                case 'right':
                    var leftPos = w.right - parseInt(tfootPaddingArr[1], 10) - _w - twoBtnWidth[1] - modalTfootBtns[2];
                    if(i>0){
                        leftPos += (twoBtnWidth[1] + modalTfootBtns[2]);
                    }
                    break;
            }
            var topPos = w.bottom - tfootHeight + parseInt(tfootPaddingArr[0], 10);
            var paraObj = {
                bounds: {
                    left: leftPos,
                    top: topPos
                },
                content: modalTfootBtns[i].txt,
                type: modalTfootBtns[i].type,
                icon: ''
            }
            var str = JSON.stringify(paraObj);
            createUedBtn(str);
        }
        
    }
    
    



}
//系统UI板块，绘制表格;
function createUedTable(obj){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(obj);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();  
    var layerNum = 0; 
    var tableBase = UED.table.base;
    var tableTh = UED.table.th;
    var tableTd = UED.table.td;
    var thPaddingArr = tableTh.padding.split(',');
    var thPaddingLR = parseInt(thPaddingArr[1], 10) + parseInt(thPaddingArr[3], 10);
    var thPaddingTB = parseInt(thPaddingArr[0], 10) + parseInt(thPaddingArr[2], 10);
    //如果不是72像素英寸;
    var is72pxInch = getNowReslution() == 1 ? true : false;
    if(!is72pxInch){
        var thObj = getTxtLen({
            content: '灰色执照abcdefghijklmnABCDEFGHIJKLMN',
            fontSize: tableTh.fontSize,
            fauxBold: tableTh.fontBolder
        });
        var tdObj = getTxtLen({
            content: '灰色执照abcdefghijklmnABCDEFGHIJKLMN',
            fontSize: tableTd.fontSize
        });
    }
    var thHeight = is72pxInch ? 
                   tableTh.fontSize + thPaddingTB :
                   thObj.height + thPaddingTB;
        
    var tdPaddingArr = tableTd.padding.split(',');
    var tdPaddingLR = parseInt(tdPaddingArr[1], 10) + parseInt(tdPaddingArr[3], 10);
    var tdPaddingTB = parseInt(tdPaddingArr[0], 10) + parseInt(tdPaddingArr[2], 10);
    var tdHeight = is72pxInch ?
                   tableTd.fontSize + tdPaddingTB :
                   tdObj.height + tdPaddingTB;
    var jsonTh = json.th;
    var jsonTr = json.tr;
    var jsonTdsAlign = json.tdsAlign;
    var trNum = Math.floor((w.height - thHeight - tableBase.borderWidth * 2) / (tdHeight + tableTd.borderWidth));
    if(trNum > jsonTr.length){
        trNum = jsonTr.length;
    }

    var hoverTr = json.hoverTrs[0] - 1;
    //绘制表头th;
    if(tableTh.bgColor != null){
        var thDots = [
            [w.left, w.top],
            [w.left, w.top + tableBase.borderWidth + thHeight],
            [w.right, w.top + tableBase.borderWidth + thHeight],
            [w.right, w.top]
        ];
        if(tableBase.borderRadius != '0,0,0,0'){
            var thBorderRadius = tableBase.borderRadius.slice(0,4)+'0,0';
            var borderArr = splitToNumber(thBorderRadius, ',');
            thDots = getConnerDots(thDots, borderArr);
        }
        addLayer({name: '表格-头部'});
        drawVectorShape({
            dots: thDots,
            type: 'fill',
            hexValueFill: tableTh.bgColor,
            closed: true
        });
        layerNum ++;
    }
    //绘制th的下边框线条;
    if(tableTh.borderWidth > 0 && (tableBase.rules == 'rows' || tableBase.rules == 'all')){
        var startTop = w.top + tableBase.borderWidth + thHeight;
        var thDots = [
            [w.left + tableBase.borderWidth, startTop],
            [w.left + tableBase.borderWidth, startTop + tableTh.borderWidth],
            [w.right - tableBase.borderWidth, startTop + tableTh.borderWidth],
            [w.right - tableBase.borderWidth, startTop]
        ];
        addLayer({name: '表格-头部-下边框线'});
        drawVectorShape({
            dots: thDots,
            type: 'fill',
            hexValueFill: tableTh.borderColor,
            closed: true
        });
        layerNum ++;
    }
    //绘制每行tr的背景;
    if(tableTd.bgColor != null){
        var startTop = w.top + tableBase.borderWidth + thHeight + tableTh.borderWidth;
        var oneRowHeight = tdHeight + tableTd.borderWidth;
        for(var i=0; i<trNum; i++){
            (function(i){
                var tdDots = [
                    [w.left + tableBase.borderWidth, startTop + oneRowHeight*i],
                    [w.left + tableBase.borderWidth, startTop + oneRowHeight*i + tdHeight],
                    [w.right - tableBase.borderWidth, startTop + oneRowHeight*i + tdHeight],
                    [w.right - tableBase.borderWidth, startTop + oneRowHeight*i]
                ];
                //最后一行的td,如果是圆角的，需要绘制成底部圆角;
                if((i == trNum-1) && (tableBase.borderRadius != '0,0,0,0')){
                    tdDots = [
                        [w.left, startTop + oneRowHeight*i],
                        [w.left, startTop + oneRowHeight*i + tdHeight],
                        [w.right, startTop + oneRowHeight*i + tdHeight],
                        [w.right, startTop + oneRowHeight*i]
                    ]
                    var tdBorderRadius = '0,0,' + tableBase.borderRadius.slice(4,7);
                    var borderArr = splitToNumber(tdBorderRadius, ',');
                    tdDots = getConnerDots(tdDots, borderArr);
                }
                addLayer({name: '表格-行' + (i+1)});
                drawVectorShape({
                    dots: tdDots,
                    type: 'fill',
                    hexValueFill: (hoverTr != i) ? tableTd.bgColor : tableTd.highlightBgColor,
                    closed: true
                });
                layerNum ++;
            })(i);
        }
    }
    //绘制每行的tr底部的边线;
    if(tableTd.borderWidth > 0  && (tableBase.rules == 'rows' || tableBase.rules == 'all')){
        var startTop = w.top + tableBase.borderWidth + thHeight + tableTh.borderWidth + tdHeight;
        var oneRowHeight = tdHeight + tableTd.borderWidth;
        //最后一条边线是不用绘制的;
        for(var i=0; i<trNum-1; i++){ 
            (function(i){
                var tdDots = [
                    [w.left + tableBase.borderWidth, startTop + oneRowHeight*i],
                    [w.left + tableBase.borderWidth, startTop + oneRowHeight*i + tableTd.borderWidth],
                    [w.right - tableBase.borderWidth, startTop + oneRowHeight*i + tableTd.borderWidth],
                    [w.right - tableBase.borderWidth, startTop + oneRowHeight*i]
                ];
                addLayer({name: '表格-行' + (i+1) + "下边框线条"});
                drawVectorShape({
                    dots: tdDots,
                    type: 'fill',
                    hexValueFill: tableTd.borderColor,
                    closed: true
                });
                layerNum ++;
            })(i);
        }
    }
    var allWidth = [];
    var allFixedWidth = 0;
    var noFixedNumber = 0;
    var allBorderWidth = (tableTd.borderWidth > 0  && (tableBase.rules == 'cols' || tableBase.rules == 'all')) ? 
                         jsonTh.length *  tableTd.borderWidth : 
                         0;
    //循环一次，计算不确定宽度的实际值;
    for(var i=0; i<jsonTh.length; i++){
        if(jsonTh[i].width){
            allFixedWidth += jsonTh[i].width;
        }else{
            noFixedNumber ++;
        }
    }
    var noFixWidth =Math.floor((w.width - allBorderWidth - allFixedWidth) / noFixedNumber);
    for(var i=0; i<jsonTh.length; i++){
        if(jsonTh[i].width){
            allWidth.push(jsonTh[i].width);
        }else{
            allWidth.push((noFixWidth>0) ? noFixWidth :  10);
        }
    }
    //绘制竖线;
    if(tableTd.borderWidth > 0  && (tableBase.rules == 'cols' || tableBase.rules == 'all')){
        for(var i=0; i<allWidth.length-1; i++){
            (function(i){
                var startLeft = w.left + tableBase.borderWidth;
                var widthCount = getArrayIndexCount({
                    array: allWidth,
                    index: i
                });
                var tableBottom = w.top + tableBase.borderWidth + thHeight + tableTh.borderWidth + trNum * tdHeight + trNum * tableTd.borderWidth - tableTd.borderWidth;
                var colsDots = [
                    [startLeft + widthCount + tableTd.borderWidth * i, w.top],
                    [startLeft + widthCount + tableTd.borderWidth * i, tableBottom],
                    [startLeft + widthCount + tableTd.borderWidth * i + tableTd.borderWidth, tableBottom],
                    [startLeft + widthCount + tableTd.borderWidth * i + tableTd.borderWidth, w.top]
                ]
                addLayer({name: '表格-列-线条'});
                drawVectorShape({
                    dots: colsDots,
                    type: 'stroke', 
                    hexValue: tableTd.borderColor,
                    solidWidth: tableTd.borderWidth,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignInside',
                    closed: true
                });
                layerNum ++;
            })(i);
        }
    }
    //绘制外边框;
    if(tableBase.borderWidth > 0){
        var outerBoderDots = [
            [w.left, w.top],
            [w.left, w.top + tableBase.borderWidth * 2 + thHeight + (tdHeight + tableTd.borderWidth) * trNum],
            [w.right, w.top + tableBase.borderWidth * 2 + thHeight + (tdHeight + tableTd.borderWidth) * trNum],
            [w.right, w.top]
        ]
        if(tableBase.borderRadius != '0,0,0,0'){
            var borderArr = splitToNumber(tableBase.borderRadius, ',');
            outerBoderDots = getConnerDots(outerBoderDots, borderArr);
        }
        addLayer({name: '表格-外边框'});
        drawVectorShape({
            dots: outerBoderDots,
            type: 'stroke', 
            hexValue: tableBase.borderColor,
            solidWidth: tableBase.borderWidth,
            angleDot: 'idstrokeStyleRoundJoin',
            strokeStyleAlign: 'strokeStyleAlignInside',
            closed: true
        });
        layerNum ++;
    }
    //绘制表头的所有文字;
    for(var i=0; i<jsonTh.length; i++){
        var startLeft = w.left + tableBase.borderWidth;
        var leftPos = startLeft + tableTd.borderWidth * i + parseInt(thPaddingArr[3], 10);
        if(i>0){
            leftPos += getArrayIndexCount({
                array: allWidth,
                index: i-1
            });
        }
        var topPos = w.top + tableBase.borderWidth + parseInt(thPaddingArr[0], 10);
        addTxtLayer({
            name: jsonTh[i].txt,
            font: UED.font.family,
            fauxBold: tableTh.fontBolder,
            color: tableTh.fontColor,
            size: tableTh.fontSize,
            position: [leftPos, topPos],
            txtAlign: tableTh.align,
            content: jsonTh[i].txt,
            width: allWidth[i] - thPaddingLR
        });
        layerNum ++;
    }
    //绘制表格里面的所有文字;
    var trLen = (jsonTr.length > trNum) ? trNum : jsonTr.length;
    for(var i=0; i<trLen; i++){
        var oneTr = jsonTr[i];
        for(var j=0; j<oneTr.length; j++){
            var startLeft = w.left + tableBase.borderWidth;
            var leftPos = startLeft + tableTd.borderWidth * j + parseInt(tdPaddingArr[3], 10);
            if(j>0){
                leftPos += getArrayIndexCount({
                    array: allWidth,
                    index: j-1
                });
            }
            var startTop = w.top + tableBase.borderWidth + thHeight + tableTh.borderWidth + parseInt(tdPaddingArr[0], 10);
            var topPos = startTop + (tableTd.borderWidth + tdHeight) * i;
            addTxtLayer({
                name: oneTr[j],
                font: UED.font.family,
                color: (hoverTr == i) ? tableTd.highlightFontColor : tableTd.fontColor,
                size: tableTd.fontSize,
                position: [leftPos, topPos],
                txtAlign: jsonTdsAlign[j],
                content: oneTr[j],
                width: allWidth[j] - tdPaddingLR
            });
            layerNum ++;
        }
    }
    createLayerSet({
        name: 'Table',
        number: layerNum
    });
    completeFn();
};

//绘制表格,每行需要占38个像素（37个像素+底部一个像素边框）;
function createTable(tableJson){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    tableJson = JSON.parse(tableJson);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var artLayerNum = 2; //需要合并入文件夹的图层个数,背景和线条是2个图层，所以默认值是2;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();    
    var trNum = Math.floor((w.height - 39) / 38);
    //绘制表头;
    var th = [[w.left+1, w.top+1], [w.left+1, w.top+38], [w.right-1, w.top+38], [w.right-1, w.top+1]];
    addLayer({name: 'Table-th-背景'});
    drawShape({
            dots: th,
            type: 'fill', 
            hexValue: "082a45",
            opacity: 50
    });
    //绘制hover的tr;
    if(tableJson.hoverTrs && tableJson.hoverTrs.length > 0){
        for(var i=0; i<tableJson.hoverTrs.length; i++){//循环需要绘制的tr,但是如果tr大于trNum-1,则不绘制;
            var hoverTr = tableJson.hoverTrs[i];
            if(hoverTr>0 && hoverTr <= trNum){
                    artLayerNum++;
                    var hoverDots = [[w.left+1, w.top+39+(hoverTr-1)*38], [w.left+1, w.top+39+(hoverTr-1)*38 + 37], [w.right-1, w.top+39+(hoverTr-1)*38 + 37], [w.right-1, w.top+39+(hoverTr-1)*38]];
                    addLayer({name: 'Table-hover-tr-' + hoverTr});
                    drawShape({
                            dots: hoverDots,
                            type: 'fill', 
                            hexValue: "082a45",
                            opacity: 50
                    });
            }
        }
    }
    addLayer({name: 'Table-tr-线条'});
    doc.activeLayer.opacity = 50;
    //绘制一根根的横线;
    for(var i=1; i<=trNum; i++){
            var tr = [[w.left+1, w.top + i*38], [w.left+1, w.top+i*38+1], [w.right-1, w.top+i*38+1], [w.right-1, w.top+i*38]];
            drawShape({
                dots: tr,
                type: 'fill', 
                hexValue: "075d76",
                opacity: 100
        });
    }

    //绘制table的borderWidth=1;
    var bw = [[w.left, w.top], [w.left, w.top+trNum*38+39], [w.right, w.top+trNum*38+39], [w.right, w.top]];
    drawShape({
        dots: bw,
        type: 'stroke',
        hexValue: '075d76',
        solidWidth: 1,
        location: 'inside'
    });
    var allType = [];//记录每列的type;
    //绘制th 的文字;
    var th = tableJson.th;
    var paddingLeft = 12;
    var allWidth = [];
    var allFixedWidth = 0;
    var noFixedNumber = 0;
    //循环一次，计算不确定宽度的实际值;
    for(var i=0; i<th.length; i++){
        if(th[i].width){
            allFixedWidth += th[i].width;
        }else{
            noFixedNumber ++;
        }
    }
    var noFixWidth =Math.floor((w.width - paddingLeft - allFixedWidth) / noFixedNumber);
    for(var i=0; i<th.length; i++){
        if(th[i].width){
            allWidth.push(th[i].width);
        }else{
            allWidth.push((noFixWidth>0) ? noFixWidth :  10);
        }
    }
    //单个th的宽度;
    //var tdWidth = Math.floor((w.width - paddingLeft) / th.length);
    for(var i=0; i<th.length; i++){
        artLayerNum++;
        allType.push(th[i].type || 'string');
        if(!th[i].type){ th[i].type == 'string'}
        switch(th[i].type){
                case "checkbox":
                    addCheckBox({
                            bounds: {
                                    left: w.left + getArrayNumCount(allWidth, i) + paddingLeft,
                                    top: w.top + 10
                            },
                            checked: false,
                            content: '',
                            merge: true
                    });
                    break;
                default:
                    addTxtLayer({
                        name: th[i].txt,
                        font: UED.font.family,
                        color: '075d76',
                        size: 14,
                        position: [w.left + getArrayNumCount(allWidth, i) + paddingLeft, w.top+14],
                        txtAlign: 'left',
                        content: th[i].txt,
                        width: allWidth[i]-2
                    });
                    break;
        }
    }
    //表头绘制完毕，开始绘制每个td里面的内容;
    var tr = tableJson.tr;
    var len = (tr.length <= trNum) ? tr.length : trNum; //如果数据太多，但是显示不下，就不需要绘制那么多个td;
    for(var i=0; i<len; i++){
            var td = tr[i];
            for(var j=0; j<td.length; j++){
                    if(allType[j] == 'checkbox'){
                            artLayerNum++;
                            addCheckBox({
                                bounds: {
                                        left: w.left + getArrayNumCount(allWidth, j) + paddingLeft,
                                        top: w.top + i*38 + 10 + 39
                                },
                                checked: (td[j] == true) ? true : false,
                                content: '',
                                merge: true
                        })
                    }else if(allType[j] == 'int'){
                            artLayerNum++;
                            addTxtLayer({
                                name: (i+1).toString(),
                                font: 'OCRAStd',
                                color: 'eb7b2d',
                                size: 14,
                                position: [w.left + getArrayNumCount(allWidth, j) + paddingLeft, w.top+i*38+24 + 39],
                                txtAlign: 'left',
                                content: (i+1).toString()
                            })
                    }else if(allType[j] == 'icon'){
                            var icons = Array.prototype.slice.call(td[j]);
                            for(var m=0; m<icons.length; m++){
                                artLayerNum++;
                                addTxtLayer({
                                    name: 'Icon',
                                    font: UED.font.iconFamily,
                                    color: '09a7b0',
                                    size: 14,
                                    position: [w.left + getArrayNumCount(allWidth, j) + paddingLeft + m*26, w.top + i*38 + 24 + 39],
                                    txtAlign: 'left',
                                    content: icons[m]
                                })
                            }
                    }else{ //string 或 ip 或 time;
                            var content;
                            switch(allType[j]){
                                    case 'ip':
                                        content = createRandomIp();
                                        break;
                                    case 'time':
                                        content = createRandomTime();
                                        break;
                                    case 'date':
                                        content = createRandomDate();
                                        break;
                                    case 'dateTime':
                                        content = createRandomDateTime();
                                        break;
                                    default:
                                        content = td[j];
                                        break;
                            }
                            artLayerNum++;
                            addTxtLayer({
                                name: td[j],
                                font: UED.font.family,
                                color: '09a7b0',
                                size: 14,
                                position: [w.left + getArrayNumCount(allWidth, j) + paddingLeft, w.top + i*38 + 24 + 39-12],
                                txtAlign: 'left',
                                content: content,
                                width: allWidth[j]-2
                            })
                    }
            }
    }
    createLayerSet({
            name: 'Table',
            number: artLayerNum
    });
    completeFn();
}



/*
  ** photoshop拉框助手 - UI设计师的智能助手;
  ** 作者：灰色执照;
  ** 时间：2018-12-10;
  ** Q  Q：26304971;
  ** 该文件主要是拉框助手的工具类函数;
*/

/*======================================================================================================== 公共部分类，包含很多数学的计算 */
function s2t(stringID) {
  return app.stringIDToTypeID(stringID);
};
//所有数组中的Y都加上一个值(ypos其实是一个负数)，用于堆叠柱子图;
function arrayAddYNum(arr, ypos){
    var returnArr = [];
    for(var i=0; i<arr.length; i++){
        returnArr[i] = [arr[i][0], arr[i][1]+ypos];
    }
    return returnArr;
}

//所有数组中的X都加上一个值(xpos其实是一个正数)，用于堆叠柱子图;
function arrayAddXNum(arr, xpos){
    var returnArr = [];
    for(var i=0; i<arr.length; i++){
        returnArr[i] = [arr[i][0]+xpos, arr[i][1]];
    }
    return returnArr;
}


//通过文字的大小和文字的个数，获取文字的长度;
function getTxtLen(obj){
    var topPos = obj.fontSize;
    addTxtLayer({
        name: obj.content,
        content: obj.content,
        font: obj.font || UED.font.family,
        fauxBold: obj.fauxBold || false,
        size: obj.fontSize,
        color: '000000',
        txtAlign: 'left',
        position: [10, topPos]
    });
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var width = layer.bounds[2] - layer.bounds[0];
    var height = layer.bounds[3] - layer.bounds[1];
    //alert(width)
    layer.remove();
    return {
        width: (parseInt(width, 10) + 2),
        height: parseInt(height, 10)
    };
}
//获取文字的宽度;
/*function getTxtWidth(){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var newLayer = layer.duplicate();
    newLayer.rasterize(RasterizeType.ENTIRELAYER);
    var width = newLayer.bounds[2] - newLayer.bounds[0];
    newLayer.remove();
}*/
//返回整数;
function returnInterger(value){
    return parseInt(value, 10);
}
//返回1位小数;
function returnFloatOne(value){
    var value = Math.round(parseFloat(value)*10)/10;
    var xsd = value.toString().split(".");
    if(xsd.length == 1){
        value=value.toString()+".0";
        return value;
    }
    if(xsd.length > 1){
        return value; 
    }
}
//返回后面带2位小数的值,不足就补充0;
function returnFloat(value){
    var value=Math.round(parseFloat(value)*100)/100;
    var xsd=value.toString().split(".");
    if(xsd.length==1){
        value=value.toString()+".00";
        return value;
    }
    if(xsd.length>1){
        if(xsd[1].length<2){
            value=value.toString()+"0";
        }
        return value;
    }
}

//返回后面带2位小数的值,不足就补充0;
function returnFloat(value){
    var value=Math.round(parseFloat(value)*100)/100;
    var xsd=value.toString().split(".");
    if(xsd.length==1){
        value=value.toString()+".00";
        return value;
    }
    if(xsd.length>1){
        if(xsd[1].length<2){
            value=value.toString()+"0";
        }
        return value;
    }
}

//我设计的渐变角度控件的角度和ps是不一致的，需要纠正一下;
function correctAngle(angle){
    if(angle >= 0 && angle <= 90){
        return 90 - angle;
    }else if(angle > 90 && angle <= 270){
        return 90 - angle;
    }else if(angle > 270 && angle < 360){
        return 450 - angle;
    }
}

//加载json数据;
function loadJson(relPath){
        var script = new File($.fileName);
        var jsonFile = new File(script.path + '/' + relPath);
        jsonFile.open('r');
        var str = jsonFile.read();
        jsonFile.close();
        return JSON.parse(str);
}
function loadJSX(fileName){
        var extensionRoot = cs.getSystemPath(SystemPath.EXTENSION) + "/jsx/";// 这里是指插件目录下的 jsx 文件夹，可自行设为任意目录
        cs.evalScript('$.evalFile("' + extensionRoot + fileName + '")');
}
//16进制颜色转rgb颜色 ffffff => 255,255,255;
function hex2rgb(hexValue){    
    var reg = /^([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
    var sColor = hexValue.toLowerCase();  
    if(sColor && reg.test(sColor)){  
            //处理六位的颜色值  
            var sColorChange = [];  
            for(var i=0; i<6; i+=2){  
                sColorChange.push(parseInt("0x"+sColor.slice(i,i+2)));    
            }  
            return sColorChange.join(","); 
        }else{  
            return sColor;    
    }  
}
//获取数组中的求和;
//var arr = [2,8,4,5];
//getArrayNumCount(arr, 2); 则得到2+8+4=14；
function getArrayNumCount(arr, num){
        var v = 0;
        for(var i=0; i<num; i++){
                v += arr[i]
        }
        return v;
}
//获取数组中的求和;
function getArrayIndexCount(obj){
    var arr = obj.array;
    var index = obj.index;
    var num = 0;
    for(var i=0; i<=index; i++){
        num += arr[i];
    }
    return num;
}

//将线条的描边inside center outerside转换成ps中的格式;
function changeToStrokeLocation(para){
    var value = StrokeLocation.INSIDE;
    switch(para){
        case 'inside': 
            value = StrokeLocation.INSIDE;
            break;
        case 'center':
            value = StrokeLocation.CENTER;
            break;
        case 'outerside':
            value = StrokeLocation.OUTSIDE;
            break;
    }
    return value;
}
//将线条的描边inside center outerside转换成ps中的格式;
function changeToStrokeStyleAlign(para){
    var value = 'strokeStyleAlignCenter';
    switch(para){
        case 'inside': 
            value = 'strokeStyleAlignInside';
            break;
        case 'center':
            value = 'strokeStyleAlignCenter';
            break;
        case 'outerside':
            value = 'strokeStyleAlignOuterSide';
            break;
    }
    return value;
}
//通过圆中心点，和圆的半径，获取一条用贝塞尔曲线的圆圈路径(实际上是4个点坐标,以逆时针的顺序);
//公式出处 https://blog.csdn.net/RichieZhu/article/details/52954152
function getCirclePath(obj){
    var cx = obj.cx;
    var cy = obj.cy;
    var r = obj.r;
    var c = 0.551915024494 * r;
    var topDot = {
        anchor: [cx, cy-r],
        left: [cx - c, cy-r],
        right: [cx + c, cy - r]
    }
    var leftDot = {
        anchor: [cx-r, cy],
        right: [cx-r, cy-c],
        left: [cx-r, cy+c]
    }
    var bottomDot = {
        anchor: [cx, cy+r],
        right: [cx-c, cy+r],
        left: [cx+c, cy+r]
    }
    var rightDot = {
        anchor: [cx+r, cy],
        right: [cx+r, cy+c],
        left: [cx+r, cy-c]
    }
    return [topDot, leftDot, bottomDot, rightDot];
}


//通过矩形的4个角的点，得到圆角矩形;
//需要注意的是，我绘制矩形是上左下右，而通常规则是上右下左;
function getConnerDots(dots, radius){
        var top,right,bottom,left;
        if(typeof(radius) == 'number'){
                 top = radius;
                 right = radius;
                 bottom = radius;
                 left = radius;
        }else{
                top = radius[0];
                 right = radius[1];
                 bottom = radius[2];
                 left = radius[3];
        }
        var newDots = [];
        if(top > 0){
                var top1 = {
                        anchor: [dots[0][0] + top, dots[0][1]],
                        left: [dots[0][0] + top/2, dots[0][1]],
                        right: [dots[0][0] + top, dots[0][1]]
                }
                var top2 = {
                        anchor: [dots[0][0], dots[0][1] + top],
                        left: [dots[0][0], dots[0][1] + top],
                        right: [dots[0][0], dots[0][1] + top - top/2]
                }
                newDots.push(top1, top2);
        }else{
                newDots.push(dots[0]);
        }
        if(left > 0){
                var left1 = {
                        anchor: [dots[1][0], dots[1][1] - left],
                        right: [dots[1][0], dots[1][1] - left],
                        left: [dots[1][0], dots[1][1] - left + left/2]
                }
                var left2 = {
                        anchor: [dots[1][0] + left, dots[1][1]],
                        right: [dots[1][0] + left - left/2, dots[1][1]],
                        left: [dots[1][0] + left, dots[1][1]]
                }
                newDots.push(left1, left2);
        }else{
                newDots.push(dots[1]);
        }
        if(bottom > 0){
                var bottom1 = {
                        anchor: [dots[2][0] - bottom, dots[2][1]],
                        right: [dots[2][0] - bottom, dots[2][1]],
                        left: [dots[2][0] - bottom + bottom/2, dots[2][1]]
                }
                var bottom2 = {
                        anchor: [dots[2][0] , dots[2][1] - bottom],
                        right: [dots[2][0], dots[2][1] - bottom + bottom/2],
                        left: [dots[2][0], dots[2][1] - bottom]    
                }
                newDots.push(bottom1, bottom2);
        }else{
                newDots.push(dots[2]);
        }
        if(right > 0){
                var right1 = {
                        anchor: [dots[3][0], dots[3][1] + right],
                        right: [dots[3][0], dots[3][1] + right],
                        left: [dots[3][0], dots[3][1] + right - right/2]
                }
                var right2 = {
                        anchor: [dots[3][0] - right, dots[3][1]],
                        right: [dots[3][0] - right + right/2, dots[3][1]],
                        left: [dots[3][0] - right, dots[3][1]]
                }
                newDots.push(right1, right2);
        }else{
                newDots.push(dots[3])
        }
    return newDots;
}

//[该功能已废弃，在界面上就做好了，这样速度更快一些]通过地图数据，得到地图的左上角，右下角坐标（xMin xMax yMin yMax）方便绘制等比例地图转换用;
function getMapRage(json){
        var xMin,yMin,xMax,yMax;
        var features = json.features;
        for(var i=0; i<features.length; i++){
            var geometry = features[i].geometry;
            var geometryType = geometry.type;
            var coordinatesArr = geometry.coordinates;
            for(var j=0; j<coordinatesArr.length; j++){
                var coordinates = coordinatesArr[j];
                for(var m=0; m<coordinates.length; m++){
                var coordinates2 = coordinates[m];
                        if(geometryType == 'Polygon'){
                                if(typeof(xMin) == 'undefined' || coordinates2[0]<xMin){
                                        xMin = coordinates2[0];
                                }
                                if(typeof(yMin) == 'undefined' || coordinates2[1]<yMin){
                                        yMin = coordinates2[1];
                                }
                                if(typeof(xMax) == 'undefined' || coordinates2[0]>xMax){
                                        xMax = coordinates2[0];
                                }
                                if(typeof(yMax) == 'undefined' || coordinates2[1]>yMax){
                                        yMax = coordinates2[1];
                                }
                        }else if(geometryType == 'MultiPolygon'){
                                for(var n=0; n<coordinates2.length; n++){
                                        var coordinates3 = coordinates2[n];
                                        if(typeof(xMin) == 'undefined' || coordinates3[0]<xMin){
                                                xMin = coordinates3[0];
                                        }
                                        if(typeof(yMin) == 'undefined' || coordinates3[1]<yMin){
                                                yMin = coordinates3[1];
                                        }
                                        if(typeof(xMax) == 'undefined' || coordinates3[0]>xMax){
                                                xMax = coordinates3[0];
                                        }
                                        if(typeof(yMax) == 'undefined' || coordinates3[1]>yMax){
                                                yMax = coordinates3[1];
                                        }
                                        
                                }
                        }      
                }
            }
        }
    return {
            xMin: xMin,
            xMax: yMin,
            yMin: xMax,
            yMax: yMax,
            w: xMax - xMin,
            h: yMax - yMin,
            wh: (xMax - xMin) / (yMax - yMin)
    }
}

//经纬度坐标转换成对应的px坐标;
function getNewMapPosition(oldDots, mapRect, w){
    var x = parseInt((oldDots[0] - mapRect.xMin) / mapRect.w * w.width + w.left, 10);
    var y = parseInt((mapRect.yMax - oldDots[1]) / mapRect.h * w.height + w.top, 10);
    return [x, y];
}

//r 为圆弧半径，cx,cy 为圆弧圆心，startAngle,endAngle 为圆弧的起始角度和结束角度
//首先拟合的圆弧角度不能大于 90°，如果是大弧度的圆弧可以划分成多段拟合
function arcToBezier(r,cx,cy,startAngle,endAngle){
	var x0 = cx + Math.cos(startAngle)*r;
	var y0 = cy + Math.sin(startAngle)*r;
	var x3 = cx + Math.cos(endAngle)*r;
	var y3 = cy + Math.sin(endAngle)*r;
	var addAngle = endAngle - startAngle;
	var a = 4*Math.tan(addAngle/4)/3;
	var x1 = x0 - a*(y0 - cy);
	var y1 = y0 + a*(x0 - cx);
	var x2 = x3 + a*(y3 - cy);
	var y2 = y3 - a*(x3 - cx);
	return {"x0":x0,"y0":y0,"x1":x1,"y1":y1,"x2":x2,"y2":y2,"x3":x3,"y3":y3};
}

//角度转弧度, 弧度 = PI/180×角度;
function angleToRadian(angle){
    return Math.PI / 180 * angle;
}

//获取圆弧上某个点的坐标;
function getCircleDot(obj){
        var r = obj.r;
        var cx = obj.cx;
        var cy = obj.cy;
        var angle = obj.angle;
        
        var hudu =angleToRadian(angle);
        var xpos = Math.sin(hudu) * r;
        var ypos = Math.cos(hudu) * r;
        return [xpos, ypos]
}

//处理formatter里面的内容，主要是提取{name}\n{value};
function formatInnerContent(str, arr){
            var reg = /\{[^}]+}/;
            
            if(str.length > 0){
                if(reg.test(str)){
                     var regArr = str.match(reg);
                     if(regArr.index == 0){ //以{开头的内容;
                            //arr.push(regArr[0]);
                            arr.push({
                                prop: (regArr[0].replace('{','')).replace('}', '')
                            })
                            str = str.substr(regArr[0].length);
                            formatInnerContent(str, arr)
                            //console.log(str)
                     }else{//开头是字符串;
                            var index = str.indexOf('{');
                            arr.push(str.substr(0, index));
                            str = str.substr(index);
                            formatInnerContent(str, arr)
                     }
                }else{
                    arr.push(str);
                }
            }
}
//生成一个随机的IP;
function createRandomIp(){
    var num1 = createRangeNum(0, 255);
    var num2 = createRangeNum(0, 255);
    var num3 = createRangeNum(0, 255);
    var num4 = createRangeNum(0, 255);
    return num1 + "." + num2 + "." + num3 + "." + num4;
}
//生成一个随机的时间，例如：23:15:39
function createRandomTime(){
        return createRangeNum(0, 23, 2) + ":" + createRangeNum(0, 59, 2) + ":" + createRangeNum(0, 59, 2);
}
//生成一个日期，如果是2018年6月，就会生成一个2018年1月到6月之间的随机日期;
function createRandomDate(){
    var d = new Date();
    var year = d.getFullYear();
    var maxMonth = d.getMonth()+1;
    var month = createRangeNum(1, maxMonth, 2);
    var day = createRangeNum(1, 28, 2);
    return year + '-' + month + '-' + day;
}
//生成一个随机的时间+日期;
function createRandomDateTime(){
    return createRandomDate() + ' ' + createRandomTime();
}
//给数据加上一个level;
function putJsonLevel(arr, paraLevel){
    for(var i=0; i<arr.length; i++){
            if(arguments.length == 1){
                    arr[i].level = 0;
            }else{
                    arr[i].level = paraLevel;
            }
         if(arr[i].children){
                putJsonLevel(arr[i].children, arr[i].level+1)
         }
    }
}


/*======================================================================================================== 绘图类  绘图类函数包含大量的PS命令，这部分命令后期需要重构 */
//绘制矢量的线性渐变，但是渐变只能有2种颜色和角度;
/*
*   {
        startColor: 'ff0000',
        endColor: 'ffffff',
        angle: 20
    }
*
*/
function drawVectorLinearGradient(obj){
    var startColorArray = (hex2rgb(obj.startColor)).split(',');
    var endColorArray = (hex2rgb(obj.endColor)).split(',');
    var angle = correctAngle(obj.angle);

    var idsetd = charIDToTypeID( "setd" );
    var desc52 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref11 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref11.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc52.putReference( idnull, ref11 );
    var idT = charIDToTypeID( "T   " );
    var desc53 = new ActionDescriptor();
    var idFlCn = charIDToTypeID( "FlCn" );
    var desc54 = new ActionDescriptor();
    var idDthr = charIDToTypeID( "Dthr" );
    desc54.putBoolean( idDthr, true );
    var idRvrs = charIDToTypeID( "Rvrs" );
    desc54.putBoolean( idRvrs, false );
    var idAngl = charIDToTypeID( "Angl" );
    var idAng = charIDToTypeID( "#Ang" );
    desc54.putUnitDouble( idAngl, idAng, angle );
    var idType = charIDToTypeID( "Type" );
    var idGrdT = charIDToTypeID( "GrdT" );
    var idLnr = charIDToTypeID( "Lnr " );
    desc54.putEnumerated( idType, idGrdT, idLnr );
    var idAlgn = charIDToTypeID( "Algn" );
    desc54.putBoolean( idAlgn, true );
    var idScl = charIDToTypeID( "Scl " );
    var idPrc = charIDToTypeID( "#Prc" );
    desc54.putUnitDouble( idScl, idPrc, 100.000000 );
    var idOfst = charIDToTypeID( "Ofst" );
    var desc55 = new ActionDescriptor();
    var idHrzn = charIDToTypeID( "Hrzn" );
    var idPrc = charIDToTypeID( "#Prc" );
    desc55.putUnitDouble( idHrzn, idPrc, 0.000000 );
    var idVrtc = charIDToTypeID( "Vrtc" );
    var idPrc = charIDToTypeID( "#Prc" );
    desc55.putUnitDouble( idVrtc, idPrc, 0.000000 );
    var idPnt = charIDToTypeID( "Pnt " );
    desc54.putObject( idOfst, idPnt, desc55 );
    var idGrad = charIDToTypeID( "Grad" );
    var desc56 = new ActionDescriptor();
    var idNm = charIDToTypeID( "Nm  " );
    desc56.putString( idNm, "灰色、白色" );
    var idGrdF = charIDToTypeID( "GrdF" );
    var idGrdF = charIDToTypeID( "GrdF" );
    var idCstS = charIDToTypeID( "CstS" );
    desc56.putEnumerated( idGrdF, idGrdF, idCstS );
    var idIntr = charIDToTypeID( "Intr" );
    desc56.putDouble( idIntr, 4096.000000 );
    var idClrs = charIDToTypeID( "Clrs" );
    var list7 = new ActionList();
    var desc57 = new ActionDescriptor();
    var idClr = charIDToTypeID( "Clr " );
    var desc58 = new ActionDescriptor();
    var idRd = charIDToTypeID( "Rd  " );
    desc58.putDouble( idRd, startColorArray[0]);
    var idGrn = charIDToTypeID( "Grn " );
    desc58.putDouble( idGrn, startColorArray[1]);
    var idBl = charIDToTypeID( "Bl  " );
    desc58.putDouble( idBl, startColorArray[2]);
    var idRGBC = charIDToTypeID( "RGBC" );
    desc57.putObject( idClr, idRGBC, desc58 );
    var idType = charIDToTypeID( "Type" );
    var idClry = charIDToTypeID( "Clry" );
    var idUsrS = charIDToTypeID( "UsrS" );
    desc57.putEnumerated( idType, idClry, idUsrS );
    var idLctn = charIDToTypeID( "Lctn" );
    desc57.putInteger( idLctn, 0 );
    var idMdpn = charIDToTypeID( "Mdpn" );
    desc57.putInteger( idMdpn, 50 );
    var idClrt = charIDToTypeID( "Clrt" );
    list7.putObject( idClrt, desc57 );
    var desc59 = new ActionDescriptor();
    var idClr = charIDToTypeID( "Clr " );
    var desc60 = new ActionDescriptor();
    var idRd = charIDToTypeID( "Rd  " );
    desc60.putDouble( idRd, endColorArray[0]);
    var idGrn = charIDToTypeID( "Grn " );
    desc60.putDouble( idGrn, endColorArray[1]);
    var idBl = charIDToTypeID( "Bl  " );
    desc60.putDouble( idBl, endColorArray[2]);
    var idRGBC = charIDToTypeID( "RGBC" );
    desc59.putObject( idClr, idRGBC, desc60 );
    var idType = charIDToTypeID( "Type" );
    var idClry = charIDToTypeID( "Clry" );
    var idUsrS = charIDToTypeID( "UsrS" );
    desc59.putEnumerated( idType, idClry, idUsrS );
    var idLctn = charIDToTypeID( "Lctn" );
    desc59.putInteger( idLctn, 4096 );
    var idMdpn = charIDToTypeID( "Mdpn" );
    desc59.putInteger( idMdpn, 50 );
    var idClrt = charIDToTypeID( "Clrt" );
    list7.putObject( idClrt, desc59 );
    desc56.putList( idClrs, list7 );
    var idTrns = charIDToTypeID( "Trns" );
    var list8 = new ActionList();
    var desc61 = new ActionDescriptor();
    var idOpct = charIDToTypeID( "Opct" );
    var idPrc = charIDToTypeID( "#Prc" );
    desc61.putUnitDouble( idOpct, idPrc, 100.000000 );
    var idLctn = charIDToTypeID( "Lctn" );
    desc61.putInteger( idLctn, 0 );
    var idMdpn = charIDToTypeID( "Mdpn" );
    desc61.putInteger( idMdpn, 50 );
    var idTrnS = charIDToTypeID( "TrnS" );
    list8.putObject( idTrnS, desc61 );
    var desc62 = new ActionDescriptor();
    var idOpct = charIDToTypeID( "Opct" );
    var idPrc = charIDToTypeID( "#Prc" );
    desc62.putUnitDouble( idOpct, idPrc, 100.000000 );
    var idLctn = charIDToTypeID( "Lctn" );
    desc62.putInteger( idLctn, 4096 );
    var idMdpn = charIDToTypeID( "Mdpn" );
    desc62.putInteger( idMdpn, 50 );
    var idTrnS = charIDToTypeID( "TrnS" );
    list8.putObject( idTrnS, desc62 );
    desc56.putList( idTrns, list8 );
    var idGrdn = charIDToTypeID( "Grdn" );
    desc54.putObject( idGrad, idGrdn, desc56 );
    var idgradientLayer = stringIDToTypeID( "gradientLayer" );
    desc53.putObject( idFlCn, idgradientLayer, desc54 );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc63 = new ActionDescriptor();
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc63.putInteger( idstrokeStyleVersion, 2 );
    var idfillEnabled = stringIDToTypeID( "fillEnabled" );
    desc63.putBoolean( idfillEnabled, true );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc53.putObject( idstrokeStyle, idstrokeStyle, desc63 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc52.putObject( idT, idshapeStyle, desc53 );
    executeAction( idsetd, desc52, DialogModes.NO );
}
//绘制多个点的填充图形或者线条;
function drawShape(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        var arrDots = obj.dots;
        var subPathArr = [];
        for(var i=0; i<arrDots.length; i++){
            var p = new PathPointInfo();
            if(arrDots[i] instanceof Array){
                p.anchor = arrDots[i];
                p.leftDirection = arrDots[i];
                p.rightDirection = arrDots[i];
            }else if(arrDots[i].constructor === Object){ 
                p.anchor = arrDots[i].anchor;
                p.leftDirection = arrDots[i].left;
                p.rightDirection = arrDots[i].right;
            }  
            p.kind = PointKind.CORNERPOINT;
            subPathArr.push(p)
        }
        var spi = new SubPathInfo();
        spi.closed = true;
        spi.operation = ShapeOperation.SHAPEADD; 
        spi.entireSubPath=subPathArr;
        var tempShape = doc.pathItems.add("huiSeZhiZhao", [spi]); //这里line是pathItem对象;
        switch(obj.type){
            case "fill":
                var myColor = new SolidColor();
                myColor.rgb.hexValue = obj.hexValue;
                try{
                    tempShape.fillPath (myColor, ColorBlendMode.NORMAL, obj.opacity, false, 0, true, true);
                }catch(e){}
                break;
            case "stroke":
                tempShape.makeSelection(0, true, SelectionType.INTERSECT) ;
                var myColor = new SolidColor();
                myColor.rgb.hexValue = obj.hexValue;
                var location
                if(obj.location == 'inside'){
                    location = StrokeLocation.INSIDE;
                }else if(obj.location == 'outside'){
                    location = StrokeLocation.OUTSIDE;
                }else{ //center;
                    location = StrokeLocation.CENTER;
                }
                try{
                    doc.selection.stroke(myColor, obj.solidWidth, location);
                }catch(e){}
                doc.selection.deselect();
                break;
            case "selection":
                tempShape.makeSelection(0, true, SelectionType.INTERSECT);
                break;
            case "pencil":
             default:
                tempShape.strokePath(ToolType.PENCIL);
                break;
        }
        tempShape.remove();
}
//绘制多个路径，然后将他们一起转换成选区
//该功能主要是解决贝赛尔曲线不能一次性绘制大于90度的圆弧;
//drawMultShape({
//        shapes: [{
//                dots : [[100, 100], [100, 200], [200, 200], [200, 100]]
//        },{
//                dots :  [[512, 128], [600, 256], [684, 320], [600, 386], [686, 514], [512, 450],[340,512],[428,386],[340,320],[120,120]]
//        }]
//});
function drawMultShape(obj) {
       var doc = app.activeDocument;
       var shapes = obj.shapes;
       var allPath = []; 
       for(var  m=0; m<shapes.length; m++){
                var dots = shapes[m].dots;
                var lineArray = [];
                for (var i=0; i<dots.length; i++) {
                    lineArray[i] = new PathPointInfo;
                    lineArray[i].kind = PointKind.CORNERPOINT;
                    
                    if(dots[i] instanceof Array){
                        lineArray[i].anchor = dots[i];
                        lineArray[i].leftDirection = dots[i];
                        lineArray[i].rightDirection = dots[i];
                    }else if(dots[i].constructor === Object){ 
                        lineArray[i].anchor = dots[i].anchor;
                        lineArray[i].leftDirection = dots[i].left;
                        lineArray[i].rightDirection = dots[i].right;
                    }
                }
                var lineSubPathArray = new SubPathInfo();
                lineSubPathArray.closed = true;
                lineSubPathArray.operation = ShapeOperation.SHAPEADD; //区域全部加一起;
                lineSubPathArray.entireSubPath = lineArray;
                allPath.push(lineSubPathArray);
        }      
        var tempShape = doc.pathItems.add("huiSeZhiZhao", allPath);
        tempShape.makeSelection(0, true, SelectionType.EXTEND);
        tempShape.remove();
}

//在一个图层里面绘制多个形状。主要是给饼图用的;
//如果是镂空，改颜色前还需要挖掉中间圆圈！;
function drawSomeVectorShape(arr, obj, substractDots){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var rgbArray = hex2rgb(obj.hexValue || 'ffffff').split(",");
    var rColor = rgbArray[0];
    var gColor = rgbArray[1];
    var bColor = rgbArray[2];
    var rColorFill = 0;
    var gColorFill = 0;
    var bColorFill = 0;
    //都需要先确定好点;
    var allSubPath = [];
    for(m=0; m<arr.length; m++){
        var arrDots = arr[m];                
        var subPathArr = [];
        for(var i=0; i<arrDots.length; i++){
            var p = new PathPointInfo();
            if(arrDots[i] instanceof Array){
                p.anchor = correctPointsPosition(arrDots[i]);
                p.leftDirection = correctPointsPosition(arrDots[i]);
                p.rightDirection = correctPointsPosition(arrDots[i]);
            }else if(arrDots[i].constructor === Object){ 
                p.anchor = correctPointsPosition(arrDots[i].anchor);
                p.leftDirection = correctPointsPosition(arrDots[i].left);
                p.rightDirection = correctPointsPosition(arrDots[i].right);
            }                
            p.kind = PointKind.CORNERPOINT;
            subPathArr.push(p)
        }
        //alert(JSON.stringify(subPathArr))
        var spi = new SubPathInfo();
        spi.closed = obj.closed ? obj.closed : false; //是否是闭合的曲线;
        spi.operation = ShapeOperation.SHAPEADD; 
        spi.entireSubPath=subPathArr;
        allSubPath.push(spi);
    }    
    if(substractDots){
        for(n=0; n<substractDots.length; n++){
            var arrDots = substractDots[n];                
            var subPathArr2 = [];
            for(var j=0; j<arrDots.length; j++){
                var p = new PathPointInfo();
                if(arrDots[j] instanceof Array){
                    p.anchor = correctPointsPosition(arrDots[j]);
                    p.leftDirection = correctPointsPosition(arrDots[j]);
                    p.rightDirection = correctPointsPosition(arrDots[j]);
                }else if(arrDots[j].constructor === Object){ 
                    p.anchor = correctPointsPosition(arrDots[j].anchor);
                    p.leftDirection = correctPointsPosition(arrDots[j].left);
                    p.rightDirection = correctPointsPosition(arrDots[j].right);
                }                
                p.kind = PointKind.CORNERPOINT;
                subPathArr2.push(p);
            }
            var spi = new SubPathInfo();
            spi.closed = true;
            spi.operation = ShapeOperation.SHAPESUBTRACT; 
            spi.entireSubPath = subPathArr2;    
            allSubPath.push(spi);
        }
        
    }
    //如果是单色的;
    if(typeof(obj.hexValueFill) == 'string'){
        if(obj.type == 'all' || obj.type == 'fill'){
            var fillColorArray = hex2rgb(obj.hexValueFill).split(",");
            rColorFill = fillColorArray[0];
            gColorFill = fillColorArray[1];
            bColorFill = fillColorArray[2];
        }        
    }
    var tempShape = doc.pathItems.add("huiSeZhiZhao", allSubPath); //这里line是pathItem对象;        
    var desc88 = new ActionDescriptor();
    var ref60 = new ActionReference();
    ref60.putClass(stringIDToTypeID("contentLayer"));
    desc88.putReference(charIDToTypeID("null"), ref60);
    var desc89 = new ActionDescriptor();
    var desc90 = new ActionDescriptor();
    var desc91 = new ActionDescriptor();
    desc91.putDouble(charIDToTypeID("Rd  "), rColorFill); // R
    desc91.putDouble(charIDToTypeID("Grn "), gColorFill); // G
    desc91.putDouble(charIDToTypeID("Bl  "), bColorFill); // B
    var id481 = charIDToTypeID("RGBC");
    desc90.putObject(charIDToTypeID("Clr "), id481, desc91);
    desc89.putObject(charIDToTypeID("Type"), stringIDToTypeID("solidColorLayer"), desc90);
    desc88.putObject(charIDToTypeID("Usng"), stringIDToTypeID("contentLayer"), desc89);
    executeAction(charIDToTypeID("Mk  "), desc88, DialogModes.NO);    
    tempShape.remove();
     //如果是渐变;
     if(typeof(obj.hexValueFill) == 'object'){
        drawVectorLinearGradient(obj.hexValueFill);
    }
    //如果只是填充颜色，则不需要执行后面关于描边的所有内容;
    if(obj.type == "fill"){
        return;
    }
    var idsetd = charIDToTypeID( "setd" );
    var desc1150 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref401 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref401.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1150.putReference( idnull, ref401 );
    var idT = charIDToTypeID( "T   " );
    var desc1151 = new ActionDescriptor();
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc1152 = new ActionDescriptor();
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc1152.putInteger( idstrokeStyleVersion, 2 );
    //如果是描边，那么将颜色块取消掉;
    if(obj.type == 'stroke'){
        var idfillEnabled = stringIDToTypeID( "fillEnabled" );
        desc1152.putBoolean( idfillEnabled, false );
    }            
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc1151.putObject( idstrokeStyle, idstrokeStyle, desc1152 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1150.putObject( idT, idshapeStyle, desc1151 );
    executeAction( idsetd, desc1150, DialogModes.NO );


    var idsetd = charIDToTypeID( "setd" );
    var desc1169 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref417 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref417.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1169.putReference( idnull, ref417 );
    var idT = charIDToTypeID( "T   " );
    var desc1170 = new ActionDescriptor();
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc1171 = new ActionDescriptor();
    var idstrokeStyleContent = stringIDToTypeID( "strokeStyleContent" );
    var desc1172 = new ActionDescriptor();
    var idClr = charIDToTypeID( "Clr " );
    var desc1173 = new ActionDescriptor();
    var idRd = charIDToTypeID( "Rd  " );
    desc1173.putDouble( idRd, rColor );
    var idGrn = charIDToTypeID( "Grn " );
    desc1173.putDouble( idGrn, gColor );
    var idBl = charIDToTypeID( "Bl  " );
    desc1173.putDouble( idBl, bColor );
    var idRGBC = charIDToTypeID( "RGBC" );
    desc1172.putObject( idClr, idRGBC, desc1173 );
    var idsolidColorLayer = stringIDToTypeID( "solidColorLayer" );
    desc1171.putObject( idstrokeStyleContent, idsolidColorLayer, desc1172 );
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc1171.putInteger( idstrokeStyleVersion, 2 );
    var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
    desc1171.putBoolean( idstrokeEnabled, true );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc1170.putObject( idstrokeStyle, idstrokeStyle, desc1171 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1169.putObject( idT, idshapeStyle, desc1170 );
    executeAction( idsetd, desc1169, DialogModes.NO );

    var idsetd = charIDToTypeID( "setd" );
    var desc1177 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref419 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref419.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1177.putReference( idnull, ref419 );
    var idT = charIDToTypeID( "T   " );
    var desc1178 = new ActionDescriptor();
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc1179 = new ActionDescriptor();
    var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
    var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
    //默认线条的对齐方式是居中的,但是也会有特殊情况，绘制按钮的时候，需要绘制内边框，类似1px solid #000，内边框的样式是：strokeStyleAlignInside;
    //var idstrokeStyleAlignCenter = stringIDToTypeID( "strokeStyleAlignCenter" );
    var strokeAlignStr = obj.strokeStyleAlign || 'strokeStyleAlignCenter';
    var idstrokeStyleAlignCenter = stringIDToTypeID(strokeAlignStr);
    desc1179.putEnumerated( idstrokeStyleLineAlignment, idstrokeStyleLineAlignment, idstrokeStyleAlignCenter );
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc1179.putInteger( idstrokeStyleVersion, 2 );
    var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
    desc1179.putBoolean( idstrokeEnabled, true );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc1178.putObject( idstrokeStyle, idstrokeStyle, desc1179 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1177.putObject( idT, idshapeStyle, desc1178 );
    executeAction( idsetd, desc1177, DialogModes.NO );
    
    var idsetd = charIDToTypeID( "setd" );
    var desc42 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref7 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref7.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc42.putReference( idnull, ref7 );
    var idT = charIDToTypeID( "T   " );
    var desc43 = new ActionDescriptor();
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc44 = new ActionDescriptor();
    var idstrokeStyleLineWidth = stringIDToTypeID( "strokeStyleLineWidth" );
    var idPxl = charIDToTypeID( "#Pxl" );
    desc44.putUnitDouble( idstrokeStyleLineWidth, idPxl, obj.solidWidth );
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc44.putInteger( idstrokeStyleVersion, 2 );
    var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
    desc44.putBoolean( idstrokeEnabled, true );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc43.putObject( idstrokeStyle, idstrokeStyle, desc44 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc42.putObject( idT, idshapeStyle, desc43 );
    executeAction( idsetd, desc42, DialogModes.NO );

    var idsetd = charIDToTypeID( "setd" );
    var desc229 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref90 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref90.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc229.putReference( idnull, ref90 );
    var idT = charIDToTypeID( "T   " );
    var desc230 = new ActionDescriptor();
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc231 = new ActionDescriptor();
    var idstrokeStyleLineJoinType = stringIDToTypeID( "strokeStyleLineJoinType" );
    var idstrokeStyleLineJoinType = stringIDToTypeID( "strokeStyleLineJoinType" );
    var idstrokeStyleRoundJoin = stringIDToTypeID(obj.angleDot || 'idstrokeStyleRoundJoin');
    desc231.putEnumerated( idstrokeStyleLineJoinType, idstrokeStyleLineJoinType, idstrokeStyleRoundJoin );
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc231.putInteger( idstrokeStyleVersion, 2 );
    var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
    desc231.putBoolean( idstrokeEnabled, true );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc230.putObject( idstrokeStyle, idstrokeStyle, desc231 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc229.putObject( idT, idshapeStyle, desc230 );
    executeAction( idsetd, desc229, DialogModes.NO );    
    
    var idsetd = charIDToTypeID( "setd" );
    var desc1281 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref445 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref445.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1281.putReference( idnull, ref445 );
    var idT = charIDToTypeID( "T   " );
    var desc1282 = new ActionDescriptor();
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc1283 = new ActionDescriptor();
    var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
    var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
    //var idstrokeStyleAlignCenter = stringIDToTypeID( "strokeStyleAlignCenter" );
    var idstrokeStyleAlignCenter = stringIDToTypeID(strokeAlignStr);
    desc1283.putEnumerated( idstrokeStyleLineAlignment, idstrokeStyleLineAlignment, idstrokeStyleAlignCenter );
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc1283.putInteger( idstrokeStyleVersion, 2 );
    var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
    desc1283.putBoolean( idstrokeEnabled, true );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc1282.putObject( idstrokeStyle, idstrokeStyle, desc1283 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1281.putObject( idT, idshapeStyle, desc1282 );
    executeAction( idsetd, desc1281, DialogModes.NO );    
}


//绘制PS中的形状;
//暂时无法既绘制形状，又绘制边线,一次只能绘制一种;
function drawVectorShape(obj){ 
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        var rgbArray = hex2rgb(obj.hexValue || 'ffffff').split(",");
        var rColor = rgbArray[0];
        var gColor = rgbArray[1];
        var bColor = rgbArray[2];
        var rColorFill = 0;
        var gColorFill = 0;
        var bColorFill = 0;
        //无论是单色，还是边框，或者是又有背景色又有边框，都需要先确定好点;
        var arrDots = obj.dots;                
        var subPathArr = [];
        for(var i=0; i<arrDots.length; i++){
            var p = new PathPointInfo();
            if(arrDots[i] instanceof Array){
                p.anchor = correctPointsPosition(arrDots[i]);
                p.leftDirection = correctPointsPosition(arrDots[i]);
                p.rightDirection = correctPointsPosition(arrDots[i]);
            }else if(arrDots[i].constructor === Object){ 
                p.anchor = correctPointsPosition(arrDots[i].anchor);
                p.leftDirection = correctPointsPosition(arrDots[i].left);
                p.rightDirection = correctPointsPosition(arrDots[i].right);
            }                
            p.kind = PointKind.CORNERPOINT;
            subPathArr.push(p)
        }

        //如果是单色的;
        if(typeof(obj.hexValueFill) == 'string'){
            if(obj.type == 'all' || obj.type == 'fill'){
                var fillColorArray = hex2rgb(obj.hexValueFill).split(",");
                rColorFill = fillColorArray[0];
                gColorFill = fillColorArray[1];
                bColorFill = fillColorArray[2];
            }        
        }
        var spi = new SubPathInfo();
        spi.closed = obj.closed ? obj.closed : false; //是否是闭合的曲线;
        spi.operation = ShapeOperation.SHAPEADD; 
        spi.entireSubPath=subPathArr;
        var tempShape = doc.pathItems.add("huiSeZhiZhao", [spi]); //这里line是pathItem对象;        
        var desc88 = new ActionDescriptor();
        var ref60 = new ActionReference();
        ref60.putClass(stringIDToTypeID("contentLayer"));
        desc88.putReference(charIDToTypeID("null"), ref60);
        var desc89 = new ActionDescriptor();
        var desc90 = new ActionDescriptor();
        var desc91 = new ActionDescriptor();
        desc91.putDouble(charIDToTypeID("Rd  "), rColorFill); // R
        desc91.putDouble(charIDToTypeID("Grn "), gColorFill); // G
        desc91.putDouble(charIDToTypeID("Bl  "), bColorFill); // B
        var id481 = charIDToTypeID("RGBC");
        desc90.putObject(charIDToTypeID("Clr "), id481, desc91);
        desc89.putObject(charIDToTypeID("Type"), stringIDToTypeID("solidColorLayer"), desc90);
        desc88.putObject(charIDToTypeID("Usng"), stringIDToTypeID("contentLayer"), desc89);
        executeAction(charIDToTypeID("Mk  "), desc88, DialogModes.NO);    
        tempShape.remove();
        //如果是渐变;
        if(typeof(obj.hexValueFill) == 'object'){
            drawVectorLinearGradient(obj.hexValueFill);
        }
        //如果只是填充颜色，则不需要执行后面关于描边的所有内容;
        if(obj.type == "fill"){
            return;
        }

        
        var idsetd = charIDToTypeID( "setd" );
    var desc1150 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref401 = new ActionReference();
        var idcontentLayer = stringIDToTypeID( "contentLayer" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref401.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1150.putReference( idnull, ref401 );
    var idT = charIDToTypeID( "T   " );
        var desc1151 = new ActionDescriptor();
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        var desc1152 = new ActionDescriptor();
        var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
        desc1152.putInteger( idstrokeStyleVersion, 2 );
        //如果是描边，那么将颜色块取消掉;
        if(obj.type == 'stroke'){
            var idfillEnabled = stringIDToTypeID( "fillEnabled" );
            desc1152.putBoolean( idfillEnabled, false );
        }
            
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        desc1151.putObject( idstrokeStyle, idstrokeStyle, desc1152 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1150.putObject( idT, idshapeStyle, desc1151 );
    executeAction( idsetd, desc1150, DialogModes.NO );


var idsetd = charIDToTypeID( "setd" );
    var desc1169 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref417 = new ActionReference();
        var idcontentLayer = stringIDToTypeID( "contentLayer" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref417.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1169.putReference( idnull, ref417 );
    var idT = charIDToTypeID( "T   " );
        var desc1170 = new ActionDescriptor();
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
            var desc1171 = new ActionDescriptor();
            var idstrokeStyleContent = stringIDToTypeID( "strokeStyleContent" );
                var desc1172 = new ActionDescriptor();
                var idClr = charIDToTypeID( "Clr " );
                    var desc1173 = new ActionDescriptor();
                    var idRd = charIDToTypeID( "Rd  " );
                    desc1173.putDouble( idRd, rColor );
                    var idGrn = charIDToTypeID( "Grn " );
                    desc1173.putDouble( idGrn, gColor );
                    var idBl = charIDToTypeID( "Bl  " );
                    desc1173.putDouble( idBl, bColor );
                var idRGBC = charIDToTypeID( "RGBC" );
                desc1172.putObject( idClr, idRGBC, desc1173 );
            var idsolidColorLayer = stringIDToTypeID( "solidColorLayer" );
            desc1171.putObject( idstrokeStyleContent, idsolidColorLayer, desc1172 );
            var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
            desc1171.putInteger( idstrokeStyleVersion, 2 );
            var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
            desc1171.putBoolean( idstrokeEnabled, true );
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        desc1170.putObject( idstrokeStyle, idstrokeStyle, desc1171 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1169.putObject( idT, idshapeStyle, desc1170 );
executeAction( idsetd, desc1169, DialogModes.NO );
    

    var idsetd = charIDToTypeID( "setd" );
    var desc1177 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref419 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref419.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1177.putReference( idnull, ref419 );
    var idT = charIDToTypeID( "T   " );
        var desc1178 = new ActionDescriptor();
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
            var desc1179 = new ActionDescriptor();
            var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
            var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
            //默认线条的对齐方式是居中的,但是也会有特殊情况，绘制按钮的时候，需要绘制内边框，类似1px solid #000，内边框的样式是：strokeStyleAlignInside;
            //var idstrokeStyleAlignCenter = stringIDToTypeID( "strokeStyleAlignCenter" );
            var strokeAlignStr = obj.strokeStyleAlign || 'strokeStyleAlignCenter';
            var idstrokeStyleAlignCenter = stringIDToTypeID(strokeAlignStr);
            desc1179.putEnumerated( idstrokeStyleLineAlignment, idstrokeStyleLineAlignment, idstrokeStyleAlignCenter );
            var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
            desc1179.putInteger( idstrokeStyleVersion, 2 );
            var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
            desc1179.putBoolean( idstrokeEnabled, true );
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        desc1178.putObject( idstrokeStyle, idstrokeStyle, desc1179 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1177.putObject( idT, idshapeStyle, desc1178 );
    executeAction( idsetd, desc1177, DialogModes.NO );

    
    
    var idsetd = charIDToTypeID( "setd" );
    var desc42 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref7 = new ActionReference();
    var idcontentLayer = stringIDToTypeID( "contentLayer" );
    var idOrdn = charIDToTypeID( "Ordn" );
    var idTrgt = charIDToTypeID( "Trgt" );
    ref7.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc42.putReference( idnull, ref7 );
    var idT = charIDToTypeID( "T   " );
    var desc43 = new ActionDescriptor();
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    var desc44 = new ActionDescriptor();
    var idstrokeStyleLineWidth = stringIDToTypeID( "strokeStyleLineWidth" );
    var idPxl = charIDToTypeID( "#Pxl" );
    desc44.putUnitDouble( idstrokeStyleLineWidth, idPxl, obj.solidWidth );
    var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
    desc44.putInteger( idstrokeStyleVersion, 2 );
    var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
    desc44.putBoolean( idstrokeEnabled, true );
    var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
    desc43.putObject( idstrokeStyle, idstrokeStyle, desc44 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc42.putObject( idT, idshapeStyle, desc43 );
    executeAction( idsetd, desc42, DialogModes.NO );

    //绘制的线条是虚线;
    if(obj.strokeType && obj.strokeType == 'dashed'){
        var dashedLine_dots = UED.dashedLine[0]; //默认是4;
        var dashedLine_empty = UED.dashedLine[1]; //默认是2;

        var idsetd = charIDToTypeID( "setd" );
        var desc161 = new ActionDescriptor();
        var idnull = charIDToTypeID( "null" );
        var ref57 = new ActionReference();
        var idcontentLayer = stringIDToTypeID( "contentLayer" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref57.putEnumerated( idcontentLayer, idOrdn, idTrgt );
        desc161.putReference( idnull, ref57 );
        var idT = charIDToTypeID( "T   " );
        var desc162 = new ActionDescriptor();
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        var desc163 = new ActionDescriptor();
        var idstrokeStyleLineCapType = stringIDToTypeID( "strokeStyleLineCapType" );
        var idstrokeStyleLineCapType = stringIDToTypeID( "strokeStyleLineCapType" );
        var idstrokeStyleButtCap = stringIDToTypeID( "strokeStyleButtCap" );
        desc163.putEnumerated( idstrokeStyleLineCapType, idstrokeStyleLineCapType, idstrokeStyleButtCap );
        var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
        var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
        var idstrokeStyleAlignInside = stringIDToTypeID( "strokeStyleAlignInside" );
        desc163.putEnumerated( idstrokeStyleLineAlignment, idstrokeStyleLineAlignment, idstrokeStyleAlignInside );
        var idstrokeStyleLineDashSet = stringIDToTypeID( "strokeStyleLineDashSet" );
        var list18 = new ActionList();
        var idNne = charIDToTypeID( "#Nne" );
        list18.putUnitDouble( idNne, dashedLine_dots );
        var idNne = charIDToTypeID( "#Nne" );
        list18.putUnitDouble( idNne, dashedLine_empty );
        desc163.putList( idstrokeStyleLineDashSet, list18 );
        var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
        desc163.putInteger( idstrokeStyleVersion, 2 );
        var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
        desc163.putBoolean( idstrokeEnabled, true );
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        desc162.putObject( idstrokeStyle, idstrokeStyle, desc163 );
        var idshapeStyle = stringIDToTypeID( "shapeStyle" );
        desc161.putObject( idT, idshapeStyle, desc162 );
        executeAction( idsetd, desc161, DialogModes.NO );
    }

    
    var idsetd = charIDToTypeID( "setd" );
    var desc229 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref90 = new ActionReference();
        var idcontentLayer = stringIDToTypeID( "contentLayer" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref90.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc229.putReference( idnull, ref90 );
    var idT = charIDToTypeID( "T   " );
        var desc230 = new ActionDescriptor();
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
            var desc231 = new ActionDescriptor();
            var idstrokeStyleLineJoinType = stringIDToTypeID( "strokeStyleLineJoinType" );
            var idstrokeStyleLineJoinType = stringIDToTypeID( "strokeStyleLineJoinType" );
            var idstrokeStyleRoundJoin = stringIDToTypeID(obj.angleDot);
            desc231.putEnumerated( idstrokeStyleLineJoinType, idstrokeStyleLineJoinType, idstrokeStyleRoundJoin );
            var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
            desc231.putInteger( idstrokeStyleVersion, 2 );
            var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
            desc231.putBoolean( idstrokeEnabled, true );
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        desc230.putObject( idstrokeStyle, idstrokeStyle, desc231 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc229.putObject( idT, idshapeStyle, desc230 );
    executeAction( idsetd, desc229, DialogModes.NO );
    
    
    var idsetd = charIDToTypeID( "setd" );
    var desc1281 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref445 = new ActionReference();
        var idcontentLayer = stringIDToTypeID( "contentLayer" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref445.putEnumerated( idcontentLayer, idOrdn, idTrgt );
    desc1281.putReference( idnull, ref445 );
    var idT = charIDToTypeID( "T   " );
        var desc1282 = new ActionDescriptor();
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
            var desc1283 = new ActionDescriptor();
            var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
            var idstrokeStyleLineAlignment = stringIDToTypeID( "strokeStyleLineAlignment" );
            //var idstrokeStyleAlignCenter = stringIDToTypeID( "strokeStyleAlignCenter" );
            var idstrokeStyleAlignCenter = stringIDToTypeID(strokeAlignStr);
            desc1283.putEnumerated( idstrokeStyleLineAlignment, idstrokeStyleLineAlignment, idstrokeStyleAlignCenter );
            var idstrokeStyleVersion = stringIDToTypeID( "strokeStyleVersion" );
            desc1283.putInteger( idstrokeStyleVersion, 2 );
            var idstrokeEnabled = stringIDToTypeID( "strokeEnabled" );
            desc1283.putBoolean( idstrokeEnabled, true );
        var idstrokeStyle = stringIDToTypeID( "strokeStyle" );
        desc1282.putObject( idstrokeStyle, idstrokeStyle, desc1283 );
    var idshapeStyle = stringIDToTypeID( "shapeStyle" );
    desc1281.putObject( idT, idshapeStyle, desc1282 );
    executeAction( idsetd, desc1281, DialogModes.NO );

    
}
//拉一个A点到B点的渐变，目前只支持2个点，只支持前景色到背景色;
function lineGradient(obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var startX = obj.startX;
    var startY = obj.startY;
    var endX = obj.endX;
    var endY = obj.endY;
    setForegroundColor(obj.startColor);
    setBackgroundColor(obj.endColor)
    
    var idGrdn = charIDToTypeID( "Grdn" );
    var desc59 = new ActionDescriptor();
    var idFrom = charIDToTypeID( "From" );
        var desc60 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc60.putUnitDouble( idHrzn, idPxl, startX );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc60.putUnitDouble( idVrtc, idPxl, startY );
    var idPnt = charIDToTypeID( "Pnt " );
    desc59.putObject( idFrom, idPnt, desc60 );
    var idT = charIDToTypeID( "T   " );
        var desc61 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc61.putUnitDouble( idHrzn, idPxl, endX );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc61.putUnitDouble( idVrtc, idPxl, endY );
    var idPnt = charIDToTypeID( "Pnt " );
    desc59.putObject( idT, idPnt, desc61 );
    var idType = charIDToTypeID( "Type" );
    var idGrdT = charIDToTypeID( "GrdT" );
    var idRflc = charIDToTypeID( "Rflc" );
    desc59.putEnumerated( idType, idGrdT, idRflc );
    var idDthr = charIDToTypeID( "Dthr" );
    desc59.putBoolean( idDthr, true );
    var idUsMs = charIDToTypeID( "UsMs" );
    desc59.putBoolean( idUsMs, true );
    var idGrad = charIDToTypeID( "Grad" );
        var desc62 = new ActionDescriptor();
        var idNm = charIDToTypeID( "Nm  " );
        desc62.putString( idNm, """$$$/DefaultGradient/ForegroundToBackground=Foreground to Background""" );
        var idGrdF = charIDToTypeID( "GrdF" );
        var idGrdF = charIDToTypeID( "GrdF" );
        var idCstS = charIDToTypeID( "CstS" );
        desc62.putEnumerated( idGrdF, idGrdF, idCstS );
        var idIntr = charIDToTypeID( "Intr" );
        desc62.putDouble( idIntr, 4096.000000 );
        var idClrs = charIDToTypeID( "Clrs" );
            var list6 = new ActionList();
                var desc63 = new ActionDescriptor();
                var idType = charIDToTypeID( "Type" );
                var idClry = charIDToTypeID( "Clry" );
                var idFrgC = charIDToTypeID( "FrgC" );
                desc63.putEnumerated( idType, idClry, idFrgC );
                var idLctn = charIDToTypeID( "Lctn" );
                desc63.putInteger( idLctn, 0 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc63.putInteger( idMdpn, 50 );
            var idClrt = charIDToTypeID( "Clrt" );
            list6.putObject( idClrt, desc63 );
                var desc64 = new ActionDescriptor();
                var idType = charIDToTypeID( "Type" );
                var idClry = charIDToTypeID( "Clry" );
                var idBckC = charIDToTypeID( "BckC" );
                desc64.putEnumerated( idType, idClry, idBckC );
                var idLctn = charIDToTypeID( "Lctn" );
                desc64.putInteger( idLctn, 4096 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc64.putInteger( idMdpn, 50 );
            var idClrt = charIDToTypeID( "Clrt" );
            list6.putObject( idClrt, desc64 );
        desc62.putList( idClrs, list6 );
        var idTrns = charIDToTypeID( "Trns" );
            var list7 = new ActionList();
                var desc65 = new ActionDescriptor();
                var idOpct = charIDToTypeID( "Opct" );
                var idPrc = charIDToTypeID( "#Prc" );
                desc65.putUnitDouble( idOpct, idPrc, 100.000000 );
                var idLctn = charIDToTypeID( "Lctn" );
                desc65.putInteger( idLctn, 0 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc65.putInteger( idMdpn, 50 );
            var idTrnS = charIDToTypeID( "TrnS" );
            list7.putObject( idTrnS, desc65 );
                var desc66 = new ActionDescriptor();
                var idOpct = charIDToTypeID( "Opct" );
                var idPrc = charIDToTypeID( "#Prc" );
                desc66.putUnitDouble( idOpct, idPrc, 100.000000 );
                var idLctn = charIDToTypeID( "Lctn" );
                desc66.putInteger( idLctn, 4096 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc66.putInteger( idMdpn, 50 );
            var idTrnS = charIDToTypeID( "TrnS" );
            list7.putObject( idTrnS, desc66 );
        desc62.putList( idTrns, list7 );
    var idGrdn = charIDToTypeID( "Grdn" );
    desc59.putObject( idGrad, idGrdn, desc62 );
    executeAction( idGrdn, desc59, DialogModes.NO );
}
//创建一个线性渐变;
function createLineGradual(obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var idGrdn = charIDToTypeID( "Grdn" );
    var desc1062 = new ActionDescriptor();
    var idFrom = charIDToTypeID( "From" );
        var desc1063 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc1063.putUnitDouble( idHrzn, idPxl, 0.000000 );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc1063.putUnitDouble( idVrtc, idPxl, obj.startY );
    var idPnt = charIDToTypeID( "Pnt " );
    desc1062.putObject( idFrom, idPnt, desc1063 );
    var idT = charIDToTypeID( "T   " );
        var desc1064 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc1064.putUnitDouble( idHrzn, idPxl, 0.000000 );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc1064.putUnitDouble( idVrtc, idPxl, obj.endY );
    var idPnt = charIDToTypeID( "Pnt " );
    desc1062.putObject( idT, idPnt, desc1064 );
    var idType = charIDToTypeID( "Type" );
    var idGrdT = charIDToTypeID( "GrdT" );
    var idLnr = charIDToTypeID( "Lnr " );
    desc1062.putEnumerated( idType, idGrdT, idLnr );
    var idDthr = charIDToTypeID( "Dthr" );
    desc1062.putBoolean( idDthr, true );
    var idUsMs = charIDToTypeID( "UsMs" );
    desc1062.putBoolean( idUsMs, true );
    var idGrad = charIDToTypeID( "Grad" );
        var desc1065 = new ActionDescriptor();
        var idNm = charIDToTypeID( "Nm  " );
        desc1065.putString( idNm, """$$$/DefaultGradient/ForegroundToBackground=Foreground to Background""" );
        var idGrdF = charIDToTypeID( "GrdF" );
        var idGrdF = charIDToTypeID( "GrdF" );
        var idCstS = charIDToTypeID( "CstS" );
        desc1065.putEnumerated( idGrdF, idGrdF, idCstS );
        var idIntr = charIDToTypeID( "Intr" );
        desc1065.putDouble( idIntr, 4096.000000 );
        var idClrs = charIDToTypeID( "Clrs" );
            var list145 = new ActionList();
                var desc1066 = new ActionDescriptor();
                var idType = charIDToTypeID( "Type" );
                var idClry = charIDToTypeID( "Clry" );
                var idFrgC = charIDToTypeID( "FrgC" );
                desc1066.putEnumerated( idType, idClry, idFrgC );
                var idLctn = charIDToTypeID( "Lctn" );
                desc1066.putInteger( idLctn, 0 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc1066.putInteger( idMdpn, 50 );
            var idClrt = charIDToTypeID( "Clrt" );
            list145.putObject( idClrt, desc1066 );
                var desc1067 = new ActionDescriptor();
                var idType = charIDToTypeID( "Type" );
                var idClry = charIDToTypeID( "Clry" );
                var idBckC = charIDToTypeID( "BckC" );
                desc1067.putEnumerated( idType, idClry, idBckC );
                var idLctn = charIDToTypeID( "Lctn" );
                desc1067.putInteger( idLctn, 4096 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc1067.putInteger( idMdpn, 50 );
            var idClrt = charIDToTypeID( "Clrt" );
            list145.putObject( idClrt, desc1067 );
        desc1065.putList( idClrs, list145 );
        var idTrns = charIDToTypeID( "Trns" );
            var list146 = new ActionList();
                var desc1068 = new ActionDescriptor();
                var idOpct = charIDToTypeID( "Opct" );
                var idPrc = charIDToTypeID( "#Prc" );
                desc1068.putUnitDouble( idOpct, idPrc, 100.000000 );
                var idLctn = charIDToTypeID( "Lctn" );
                desc1068.putInteger( idLctn, 0 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc1068.putInteger( idMdpn, 50 );
            var idTrnS = charIDToTypeID( "TrnS" );
            list146.putObject( idTrnS, desc1068 );
                var desc1069 = new ActionDescriptor();
                var idOpct = charIDToTypeID( "Opct" );
                var idPrc = charIDToTypeID( "#Prc" );
                desc1069.putUnitDouble( idOpct, idPrc, 100.000000 );
                var idLctn = charIDToTypeID( "Lctn" );
                desc1069.putInteger( idLctn, 4096 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc1069.putInteger( idMdpn, 50 );
            var idTrnS = charIDToTypeID( "TrnS" );
            list146.putObject( idTrnS, desc1069 );
        desc1065.putList( idTrns, list146 );
    var idGrdn = charIDToTypeID( "Grdn" );
    desc1062.putObject( idGrad, idGrdn, desc1065 );
    executeAction( idGrdn, desc1062, DialogModes.NO );
}
//拉一个A点到B点的圆形渐变，目前只支持2个点，只支持前景色到背景色;
function radialGradient(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        var xpos = obj.x;
        var ypos = obj.y;
        var r = obj.r;
        
        setForegroundColor(obj.startColor);
        setBackgroundColor(obj.endColor);
        
        
        var idGrdn = charIDToTypeID( "Grdn" );
    var desc55 = new ActionDescriptor();
    var idFrom = charIDToTypeID( "From" );
        var desc56 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc56.putUnitDouble( idHrzn, idPxl, xpos );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc56.putUnitDouble( idVrtc, idPxl, ypos );
    var idPnt = charIDToTypeID( "Pnt " );
    desc55.putObject( idFrom, idPnt, desc56 );
    var idT = charIDToTypeID( "T   " );
        var desc57 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc57.putUnitDouble( idHrzn, idPxl, xpos+r );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc57.putUnitDouble( idVrtc, idPxl, ypos);
    var idPnt = charIDToTypeID( "Pnt " );
    desc55.putObject( idT, idPnt, desc57 );
    var idType = charIDToTypeID( "Type" );
    var idGrdT = charIDToTypeID( "GrdT" );
    var idRdl = charIDToTypeID( "Rdl " );
    desc55.putEnumerated( idType, idGrdT, idRdl );
    var idDthr = charIDToTypeID( "Dthr" );
    desc55.putBoolean( idDthr, true );
    var idUsMs = charIDToTypeID( "UsMs" );
    desc55.putBoolean( idUsMs, true );
    var idGrad = charIDToTypeID( "Grad" );
        var desc58 = new ActionDescriptor();
        var idNm = charIDToTypeID( "Nm  " );
        desc58.putString( idNm, """$$$/DefaultGradient/ForegroundToBackground=Foreground to Background""" );
        var idGrdF = charIDToTypeID( "GrdF" );
        var idGrdF = charIDToTypeID( "GrdF" );
        var idCstS = charIDToTypeID( "CstS" );
        desc58.putEnumerated( idGrdF, idGrdF, idCstS );
        var idIntr = charIDToTypeID( "Intr" );
        desc58.putDouble( idIntr, 4096.000000 );
        var idClrs = charIDToTypeID( "Clrs" );
            var list3 = new ActionList();
                var desc59 = new ActionDescriptor();
                var idType = charIDToTypeID( "Type" );
                var idClry = charIDToTypeID( "Clry" );
                var idFrgC = charIDToTypeID( "FrgC" );
                desc59.putEnumerated( idType, idClry, idFrgC );
                var idLctn = charIDToTypeID( "Lctn" );
                desc59.putInteger( idLctn, 0 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc59.putInteger( idMdpn, 50 );
            var idClrt = charIDToTypeID( "Clrt" );
            list3.putObject( idClrt, desc59 );
                var desc60 = new ActionDescriptor();
                var idType = charIDToTypeID( "Type" );
                var idClry = charIDToTypeID( "Clry" );
                var idBckC = charIDToTypeID( "BckC" );
                desc60.putEnumerated( idType, idClry, idBckC );
                var idLctn = charIDToTypeID( "Lctn" );
                desc60.putInteger( idLctn, 4096 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc60.putInteger( idMdpn, 50 );
            var idClrt = charIDToTypeID( "Clrt" );
            list3.putObject( idClrt, desc60 );
        desc58.putList( idClrs, list3 );
        var idTrns = charIDToTypeID( "Trns" );
            var list4 = new ActionList();
                var desc61 = new ActionDescriptor();
                var idOpct = charIDToTypeID( "Opct" );
                var idPrc = charIDToTypeID( "#Prc" );
                desc61.putUnitDouble( idOpct, idPrc, 100.000000 );
                var idLctn = charIDToTypeID( "Lctn" );
                desc61.putInteger( idLctn, 0 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc61.putInteger( idMdpn, 50 );
            var idTrnS = charIDToTypeID( "TrnS" );
            list4.putObject( idTrnS, desc61 );
                var desc62 = new ActionDescriptor();
                var idOpct = charIDToTypeID( "Opct" );
                var idPrc = charIDToTypeID( "#Prc" );
                desc62.putUnitDouble( idOpct, idPrc, 100.000000 );
                var idLctn = charIDToTypeID( "Lctn" );
                desc62.putInteger( idLctn, 4096 );
                var idMdpn = charIDToTypeID( "Mdpn" );
                desc62.putInteger( idMdpn, 50 );
            var idTrnS = charIDToTypeID( "TrnS" );
            list4.putObject( idTrnS, desc62 );
        desc58.putList( idTrns, list4 );
    var idGrdn = charIDToTypeID( "Grdn" );
    desc55.putObject( idGrad, idGrdn, desc58 );
    executeAction( idGrdn, desc55, DialogModes.NO );
}
//绘制矢量圆圈路径;
function drawVerctorCircle(obj){
    //如果没有背景颜色，就默认给个黑色;
    var rgbArr = (obj.bgColor && obj.bgColor != null) ? hex2rgb(obj.bgColor).split(',') : [0,0,0];
    var fillEnabled = (obj.bgColor && obj.bgColor != null) ? true : false;    
    var borderColorRgbArr = (obj.borderColor && obj.borderColor != null) ? hex2rgb(obj.borderColor).split(',') : [0,0,0];
    var borderEnable = (obj.borderColor && obj.borderColor != null) ? true : false;
    var borderWidth = borderEnable ? obj.borderWidth : 0;    
    var topPose = obj.top;
    var rightPose = obj.left + obj.diam;
    var bottomPose = topPose + obj.diam;
    var leftPose = obj.left;
    var strokeStyleAlign = obj.strokeStyleAlign || 'strokeStyleAlignInside';

    var idMk = charIDToTypeID("Mk  ");
    var desc12 = new ActionDescriptor();
    var idnull = charIDToTypeID("null");
    var ref3 = new ActionReference();
    var idcontentLayer = stringIDToTypeID("contentLayer");
    ref3.putClass(idcontentLayer);
    desc12.putReference(idnull, ref3);
    var idUsng = charIDToTypeID("Usng");
    var desc13 = new ActionDescriptor();
    var idType = charIDToTypeID("Type");
    var desc14 = new ActionDescriptor();
    var idClr = charIDToTypeID("Clr ");
    var desc15 = new ActionDescriptor();
    var idRd = charIDToTypeID("Rd  ");
    desc15.putDouble(idRd, rgbArr[0]);
    var idGrn = charIDToTypeID("Grn ");
    desc15.putDouble(idGrn, rgbArr[1]);
    var idBl = charIDToTypeID("Bl  ");
    desc15.putDouble(idBl, rgbArr[2]);
    var idRGBC = charIDToTypeID("RGBC");
    desc14.putObject(idClr, idRGBC, desc15);
    var idsolidColorLayer = stringIDToTypeID("solidColorLayer");
    desc13.putObject(idType, idsolidColorLayer, desc14);
    var idShp = charIDToTypeID("Shp ");
    var desc16 = new ActionDescriptor();
    var idunitValueQuadVersion = stringIDToTypeID("unitValueQuadVersion");
    desc16.putInteger(idunitValueQuadVersion, 1);
    var idTop = charIDToTypeID("Top ");
    var idPxl = charIDToTypeID("#Pxl");
    desc16.putUnitDouble(idTop, idPxl, topPose);
    var idLeft = charIDToTypeID("Left");
    var idPxl = charIDToTypeID("#Pxl");
    desc16.putUnitDouble(idLeft, idPxl, leftPose);
    var idBtom = charIDToTypeID("Btom");
    var idPxl = charIDToTypeID("#Pxl");
    desc16.putUnitDouble(idBtom, idPxl, bottomPose);
    var idRght = charIDToTypeID("Rght");
    var idPxl = charIDToTypeID("#Pxl");
    desc16.putUnitDouble(idRght, idPxl, rightPose);
    var idElps = charIDToTypeID("Elps");
    desc13.putObject(idShp, idElps, desc16);
    var idstrokeStyle = stringIDToTypeID("strokeStyle");
    var desc17 = new ActionDescriptor();
    var idstrokeStyleVersion = stringIDToTypeID("strokeStyleVersion");
    desc17.putInteger(idstrokeStyleVersion, 2);
    var idstrokeEnabled = stringIDToTypeID("strokeEnabled");
    desc17.putBoolean(idstrokeEnabled, borderEnable);    
    var idfillEnabled = stringIDToTypeID("fillEnabled");
    desc17.putBoolean(idfillEnabled, fillEnabled);
    var idstrokeStyleLineWidth = stringIDToTypeID("strokeStyleLineWidth");
    var idPxl = charIDToTypeID("#Pxl");
    desc17.putUnitDouble(idstrokeStyleLineWidth, idPxl, borderWidth);
    var idstrokeStyleLineDashOffset = stringIDToTypeID("strokeStyleLineDashOffset");
    var idPnt = charIDToTypeID("#Pnt");
    desc17.putUnitDouble(idstrokeStyleLineDashOffset, idPnt, 0.000000);
    var idstrokeStyleMiterLimit = stringIDToTypeID("strokeStyleMiterLimit");
    desc17.putDouble(idstrokeStyleMiterLimit, 100.000000);
    var idstrokeStyleLineCapType = stringIDToTypeID("strokeStyleLineCapType");
    var idstrokeStyleLineCapType = stringIDToTypeID("strokeStyleLineCapType");
    var idstrokeStyleButtCap = stringIDToTypeID("strokeStyleButtCap");
    desc17.putEnumerated(idstrokeStyleLineCapType, idstrokeStyleLineCapType, idstrokeStyleButtCap);
    var idstrokeStyleLineJoinType = stringIDToTypeID("strokeStyleLineJoinType");
    var idstrokeStyleLineJoinType = stringIDToTypeID("strokeStyleLineJoinType");
    var idstrokeStyleMiterJoin = stringIDToTypeID("strokeStyleMiterJoin");
    desc17.putEnumerated(idstrokeStyleLineJoinType, idstrokeStyleLineJoinType, idstrokeStyleMiterJoin);
    var idstrokeStyleLineAlignment = stringIDToTypeID("strokeStyleLineAlignment");
    var idstrokeStyleLineAlignment = stringIDToTypeID("strokeStyleLineAlignment");
    //var idstrokeStyleAlignInside = stringIDToTypeID("strokeStyleAlignInside");
    var idstrokeStyleAlignInside = stringIDToTypeID(strokeStyleAlign);
    desc17.putEnumerated(idstrokeStyleLineAlignment, idstrokeStyleLineAlignment, idstrokeStyleAlignInside);
    var idstrokeStyleScaleLock = stringIDToTypeID("strokeStyleScaleLock");
    desc17.putBoolean(idstrokeStyleScaleLock, false);
    var idstrokeStyleStrokeAdjust = stringIDToTypeID("strokeStyleStrokeAdjust");
    desc17.putBoolean(idstrokeStyleStrokeAdjust, false);
    var idstrokeStyleLineDashSet = stringIDToTypeID("strokeStyleLineDashSet");
    var list2 = new ActionList();
    desc17.putList(idstrokeStyleLineDashSet, list2);
    var idstrokeStyleBlendMode = stringIDToTypeID("strokeStyleBlendMode");
    var idBlnM = charIDToTypeID("BlnM");
    var idNrml = charIDToTypeID("Nrml");
    desc17.putEnumerated(idstrokeStyleBlendMode, idBlnM, idNrml);
    var idstrokeStyleOpacity = stringIDToTypeID("strokeStyleOpacity");
    var idPrc = charIDToTypeID("#Prc");
    desc17.putUnitDouble(idstrokeStyleOpacity, idPrc, 100.000000);
    var idstrokeStyleContent = stringIDToTypeID("strokeStyleContent");
    var desc18 = new ActionDescriptor();
    var idClr = charIDToTypeID("Clr ");
    var desc19 = new ActionDescriptor();
    var idRd = charIDToTypeID("Rd  ");
    desc19.putDouble(idRd, borderColorRgbArr[0]);
    var idGrn = charIDToTypeID("Grn ");
    desc19.putDouble(idGrn, borderColorRgbArr[1]);
    var idBl = charIDToTypeID("Bl  ");
    desc19.putDouble(idBl, borderColorRgbArr[2]);
    var idRGBC = charIDToTypeID("RGBC");
    desc18.putObject(idClr, idRGBC, desc19);
    var idsolidColorLayer = stringIDToTypeID("solidColorLayer");
    desc17.putObject(idstrokeStyleContent, idsolidColorLayer, desc18);
    var idstrokeStyleResolution = stringIDToTypeID("strokeStyleResolution");
    desc17.putDouble(idstrokeStyleResolution, 72.000000);
    var idstrokeStyle = stringIDToTypeID("strokeStyle");
    desc13.putObject(idstrokeStyle, idstrokeStyle, desc17);
    var idcontentLayer = stringIDToTypeID("contentLayer");
    desc12.putObject(idUsng, idcontentLayer, desc13);
    var idLyrI = charIDToTypeID("LyrI");
    desc12.putInteger(idLyrI, 2);
    executeAction(idMk, desc12, DialogModes.NO);
}



//绘制一个圆圈路径，转化为选区;
function drawCirclePath(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        var idsetd = charIDToTypeID( "setd" );
        var desc531 = new ActionDescriptor();
        var idnull = charIDToTypeID( "null" );
        var ref148 = new ActionReference();
        var idPath = charIDToTypeID( "Path" );
        var idWrPt = charIDToTypeID( "WrPt" );
        ref148.putProperty( idPath, idWrPt );
        desc531.putReference( idnull, ref148 );
        var idT = charIDToTypeID( "T   " );
        var desc532 = new ActionDescriptor();
        var idunitValueQuadVersion = stringIDToTypeID( "unitValueQuadVersion" );
        desc532.putInteger( idunitValueQuadVersion, 1 );
        var idTop = charIDToTypeID( "Top " );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc532.putUnitDouble( idTop, idPxl, obj.startY );
        var idLeft = charIDToTypeID( "Left" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc532.putUnitDouble( idLeft, idPxl, obj.startX );
        var idBtom = charIDToTypeID( "Btom" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc532.putUnitDouble( idBtom, idPxl, obj.endY );
        var idRght = charIDToTypeID( "Rght" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc532.putUnitDouble( idRght, idPxl, obj.endX );
        var idElps = charIDToTypeID( "Elps" );
        desc531.putObject( idT, idElps, desc532 );
        executeAction( idsetd, desc531, DialogModes.NO );
        
        var idsetd = charIDToTypeID( "setd" );
        var desc613 = new ActionDescriptor();
        var idnull = charIDToTypeID( "null" );
        var ref174 = new ActionReference();
        var idChnl = charIDToTypeID( "Chnl" );
        var idfsel = charIDToTypeID( "fsel" );
        ref174.putProperty( idChnl, idfsel );
        desc613.putReference( idnull, ref174 );
        var idT = charIDToTypeID( "T   " );
        var ref175 = new ActionReference();
        var idPath = charIDToTypeID( "Path" );
        var idWrPt = charIDToTypeID( "WrPt" );
        ref175.putProperty( idPath, idWrPt );
        desc613.putReference( idT, ref175 );    
        executeAction( idsetd, desc613, DialogModes.NO );
        
        switch(obj.type){
                case 'fill':
                    var myColor = new SolidColor();
                    myColor.rgb.hexValue = obj.hexValue;
                    doc.selection.fill(myColor, ColorBlendMode.NORMAL, obj.opacity, false);
                    doc.selection.deselect();
                break;
                case 'stroke':
                    var myColor = new SolidColor();
                    myColor.rgb.hexValue = obj.hexValue;
                    var location
                    if(obj.location == 'inside'){
                        location = StrokeLocation.INSIDE;
                    }else if(obj.location == 'outside'){
                        location = StrokeLocation.OUTSIDE;
                    }else{ //center;
                        location = StrokeLocation.CENTER;
                    }
                    doc.selection.stroke(myColor, obj.solidWidth, location);
                    doc.selection.deselect();
                    break;
                case 'selection':
                    break;
        }
}

//每隔4个像素创建一个2个像素的选区，然后删掉这个选区，再取消选择;
function vlineToDashed(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        for(var i=obj.top+6; i<obj.bottom+6; i+=6){
                var points = [[obj.left, i-2], [obj.right, i-2], [obj.right, i], [obj.left, i]]
                doc.selection.select(points,SelectionType.EXTEND, 0, false);
        }
        doc.selection.clear();
        doc.selection.deselect();
}

//横向4个实体像素，2个空白像素;
function hlineToDashed(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
      for(var j=obj.left+6; j<obj.right+6; j+=6){
              var points = [[j-2, obj.top], [j, obj.top], [j, obj.bottom], [j-2, obj.bottom]];
              doc.selection.select(points,SelectionType.EXTEND, 0, false);
              doc.selection.clear();
              doc.selection.deselect();
      }
}

//减去一个圆圈选区，在已经有一个选区的情况下，裁剪到圆圈部分。主要用于画空心饼图时使用;
function subtractCircle(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
            var idSbtF = charIDToTypeID( "SbtF" );
        var desc20 = new ActionDescriptor();
        var idnull = charIDToTypeID( "null" );
            var ref4 = new ActionReference();
            var idChnl = charIDToTypeID( "Chnl" );
            var idfsel = charIDToTypeID( "fsel" );
            ref4.putProperty( idChnl, idfsel );
        desc20.putReference( idnull, ref4 );
        var idT = charIDToTypeID( "T   " );
            var desc21 = new ActionDescriptor();
            var idTop = charIDToTypeID( "Top " );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc21.putUnitDouble( idTop, idPxl, obj.startY );
            var idLeft = charIDToTypeID( "Left" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc21.putUnitDouble( idLeft, idPxl, obj.startX );
            var idBtom = charIDToTypeID( "Btom" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc21.putUnitDouble( idBtom, idPxl, obj.endY );
            var idRght = charIDToTypeID( "Rght" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc21.putUnitDouble( idRght, idPxl, obj.endX );
        var idElps = charIDToTypeID( "Elps" );
        desc20.putObject( idT, idElps, desc21 );
        var idAntA = charIDToTypeID( "AntA" );
        desc20.putBoolean( idAntA, true );
        executeAction( idSbtF, desc20, DialogModes.NO );
}

//绘制多个弧形，然后一起转化成一个选区，获取其中一个弧形的点;
function getArcDots(obj){
        var r = (obj.roseType) ? obj.roseR : obj.r,  //如果是南丁格尔玫瑰图，则使用roseR;
              cx = obj.cx, 
              cy = obj.cy,
              start = obj.start - 90,
              end = obj.end - 90;
              
        var dots = [];
        if(end - start > 0){
            var pie = arcToBezier(r, cx, cy, angleToRadian(start), angleToRadian(end));
            dots = [{
                anchor: [pie.x0, pie.y0],
                left:      [pie.x1, pie.y1],
                right:    [pie.x0, pie.y0],
            },{
                anchor: [pie.x3, pie.y3],
                left:      [pie.x3, pie.y3],
                right:    [pie.x2, pie.y2],
            },[cx, cy]];
        }
        return dots;
}

//绘制圆弧的填充线条;
function drawPieBorder(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        var drawAngle = 80;
        var r = obj.r, 
              cx = obj.cx, 
              cy = obj.cy,
              start = obj.start,
              end = obj.end;
         var angle = end - start;
         var oldStart = obj.start;
         var circlePath = {
                    shapes: []
            }
            var num = Math.ceil(angle / drawAngle);
            for(var i=0; i<num; i++){
                    obj.start = oldStart + i*drawAngle;
                    var nowStart = obj.start;
                    if(i == num-1 && angle % drawAngle != 0){
                            drawAngle = angle % drawAngle;
                    }
                    obj.end = nowStart + drawAngle;
                    if(i>0){obj.start -= 5} //起始角度往回倒退5度，这样就有重合的，就不会有空隙;
                    var pathDots = getArcDots(obj);
                    circlePath.shapes.push({
                            dots: pathDots
                    });
            }
            drawMultShape(circlePath); //建立圆弧选区;
            //如果存在这个属性，则是绘制空心的饼图;
            if(obj.innerRadius){
                var radius,
                    reg = /%/,
                    reg2 = /px/;
                if(reg.test(obj.innerRadius)){
                    radius = parseInt(obj.innerRadius, 10) / 100 * obj.r * 2; //当是南丁格尔玫瑰图的时候，由于每个扇形半径不一样，所以这里不是乘以半径;
                }else if(reg2.test(obj.innerRadius)){
                    radius = parseInt(obj.innerRadius.replace('px', ''), 10);
                }else{
                    radius = parseInt(obj.innerRadius, 10);
                }
                var halfR = radius / 2;
                subtractCircle({
                    startX: obj.cx - halfR,
                    startY: obj.cy - halfR,
                    endX: obj.cx + halfR,
                    endY: obj.cy + halfR
                });
            }
            var borderWidth = parseInt(obj.border.borderWidth, 10),
                borderColor = obj.border.borderColor,
                //默认是填充到内侧;
                borderLocation = obj.border.borderLocation ? changeToStrokeLocation(obj.border.borderLocation): StrokeLocation.INSIDE; 
            
            var bColor = new SolidColor();
            bColor.rgb.hexValue = borderColor;
            doc.selection.stroke(bColor, borderWidth, borderLocation);
            if(obj.border.borderOpacity){
                    doc.activeLayer.fillOpacity = obj.border.borderOpacity;
            } 
            doc.selection.deselect();
}
//绘制一个某个范围内的数字，可以前面补零;
function createRangeNum(min, max, atLeastNum){
        var randomNum = Math.floor(Math.random() * (max - min + 1)) + min;
        if(arguments.length == 3){
                if(randomNum.toString().length < atLeastNum){
                        var zero = '';
                        var len = atLeastNum-(randomNum.toString().length)
                        for(var i=0; i<len; i++){
                                zero += '0';
                        }
                        return zero + randomNum;
                }else{
                        return randomNum;
                }
        }else{
            return randomNum;
        }
}
//绘制树时计算位置；
function getPostion(item, w) {    
    item.top = treeSetting.treeTop * treeSetting.top + w.top;
    item.left = treeSetting.left * item.level + w.left;
    treeSetting.treeTop += 1;
    if (item.children) {
        for (var i = 0; i < item.children.length; i++)
        getPostion(item.children[i], w)
    }
}
//绘制树时计算位置；
function getUedPostion(item, w) {
    item.top = uedTreeSetting.treeTop * uedTreeSetting.top + w.top;
    item.left = uedTreeSetting.left * item.level + w.left;
    uedTreeSetting.treeTop += 1;
    if (item.children) {
        for (var i = 0; i < item.children.length; i++)
        getUedPostion(item.children[i], w)
    }
}
//绘制饼图的一部分;
//从-90度开始绘制，因为echarts的饼图规则是从正北开始计算;
//每单大于90度的饼图，就需要分多部进行绘制;
function _drawPieItem(obj){
    var r = (obj.roseType) ? obj.roseR : obj.r,  //如果是南丁格尔玫瑰图，则使用roseR;
          cx = obj.cx, 
          cy = obj.cy,
          start = obj.start - 90,
          end = obj.end - 90;
          
    if(end - start > 0){
        var pie = arcToBezier(r, cx, cy, angleToRadian(start), angleToRadian(end));
        var dots = [{
            anchor: [pie.x0, pie.y0],
            left:      [pie.x1, pie.y1],
            right:    [pie.x0, pie.y0]
        },{
            anchor: [pie.x3, pie.y3],
            left:      [pie.x3, pie.y3],
            right:    [pie.x2, pie.y2]
        },[cx, cy]];
    
        drawShape({
            dots: dots,
            type: 'fill', 
            hexValue: obj.color,
            opacity: 100
        });
    }
    //如果存在这个属性，则是绘制空心的饼图;
    if(obj.innerRadius){
        var radius,
            reg = /%/,
            reg2 = /px/;
        if(reg.test(obj.innerRadius)){
            radius = parseInt(obj.innerRadius, 10) / 100 * obj.r * 2; //当是南丁格尔玫瑰图的时候，由于每个扇形半径不一样，所以这里不是乘以半径;
        }else if(reg2.test(obj.innerRadius)){
            radius = parseInt(obj.innerRadius.replace('px', ''), 10);
        }else{
            radius = parseInt(obj.innerRadius, 10);
        }
        var halfR = radius / 2;
        drawCirclePath({
            startX: obj.cx - halfR,
            startY: obj.cy - halfR,
            endX: obj.cx + halfR,
            endY: obj.cy + halfR,
            type: 'selection'
        });
        app.activeDocument.selection.clear();
        app.activeDocument.selection.deselect();
    }
    
}


//绘制圆弧，绘制大于90度的圆弧需要分多次;
function drawPieItem(obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var drawAngle = 80;
    var r = obj.r, 
          cx = obj.cx, 
          cy = obj.cy,
          start = obj.start,
          end = obj.end;
    var angle = end - start;
    var oldStart = obj.start;
    if(typeof(obj.color) == 'object'){ //如果是渐变色，大于90度的圆弧则必须绘制多个选区，然后再填充一次颜色;
        var circlePath = {
                shapes: []
        }
        var num = Math.ceil(angle / drawAngle);
        for(var i=0; i<num; i++){
            obj.start = oldStart + i*drawAngle;
            var nowStart = obj.start;
            if(i == num-1 && angle % drawAngle != 0){
                drawAngle = angle % drawAngle;
            }
            obj.end = nowStart + drawAngle;
            if(i>0){obj.start -= 5} //起始角度往回倒退5度，这样就有重合的，就不会有空隙;
            var pathDots = getArcDots(obj);
            circlePath.shapes.push({
                dots: pathDots
            });
        }
        drawMultShape(circlePath); //建立圆弧选区;
        //如果存在这个属性，则是绘制空心的饼图;
        if(obj.innerRadius){
            var radius,
                reg = /%/,
                reg2 = /px/;
            if(reg.test(obj.innerRadius)){
                radius = parseInt(obj.innerRadius, 10) / 100 * obj.r * 2; //当是南丁格尔玫瑰图的时候，由于每个扇形半径不一样，所以这里不是乘以半径;
            }else if(reg2.test(obj.innerRadius)){
                radius = parseInt(obj.innerRadius.replace('px', ''), 10);
            }else{
                radius = parseInt(obj.innerRadius, 10);
            }
            var halfR = radius / 2;
            subtractCircle({
                startX: obj.cx - halfR,
                startY: obj.cy - halfR,
                endX: obj.cx + halfR,
                endY: obj.cy + halfR
            });
            //app.activeDocument.selection.clear();
        }
            
        var sel = getSelectionBounds();
        var bgColor = obj.color, 
            x = bgColor.x,
            y = bgColor.y,
            x2 = bgColor.x2 || null,
            y2 = bgColor.y2 || null,
            rPos = bgColor.r || null,
            startColor = bgColor.colors[0],
            endColor = bgColor.colors[1];
                
        if(bgColor.type == 'linear'){ //线性渐变;
            lineGradient({
                startX: sel.width * x + sel.left,
                endX: sel.width * x2 + sel.left,
                startY: sel.height * y + sel.top,
                endY: sel.height * y2 + sel.top,
                startColor: startColor,
                endColor: endColor
            });
        }else if(bgColor.type == 'radial') {
            radialGradient({
                x: sel.width * x + sel.left,
                y: sel.height * y + sel.top,
                r: (sel.width * rPos),
                startColor: startColor,
                endColor: endColor
            });
        }
        doc.activeLayer.fillOpacity = obj.opacity;
        doc.selection.deselect();
            
    }else{
        if(angle > 90){ // 分多次绘制，每次绘制一个80度的圆弧，重叠5度，就不会有空隙;
            var num = Math.ceil(angle / drawAngle);
            for(var i=0; i<num; i++){
                obj.start = oldStart + i*drawAngle;
                var nowStart = obj.start;
                if(i == num-1 && angle % drawAngle != 0){
                    drawAngle = angle % drawAngle;
                }
                obj.end = nowStart + drawAngle;
                if(i>0){obj.start -= 5} //起始角度往回倒退5度，这样就有重合的，就不会有空隙;
                //$.writeln(obj.start +":"+obj.end)
                drawPieItem(obj);
            }
        }else{
            _drawPieItem(obj);
        }
    }
}


/*======================================================================================================== 宿主类，包含很多宿主对象的操作 */
//添加一个图层;
function addLayer(obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var l;
    l = doc.artLayers.add();
    /*
    //往文档里面增加一个图层;
    if(!obj.layerSetName){
        l = doc.artLayers.add();
    //往文件夹里面增加一个图层;
    }else{
        l = doc.layerSets.getByName(obj.layerSetName);
        if(l.length > 1){
            l = l[0];
        }
    }*/
    if(obj.name){
        l.name = obj.name;
    }
    if(obj.kind){
        l.kind = obj.kind;
    }
    if(obj.blendMode){
        l.blendMode = obj.blendMode;
    }
};

//添加一个文字图层;
function addTxtLayer(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        addLayer({name: obj.name, kind: LayerKind.TEXT});

        var txtItem = doc.activeLayer.textItem;
        //如果存在宽度，则说明是绘制文本框的文字;
        if(obj.width){
            txtItem.kind = TextType.PARAGRAPHTEXT;
            var docCoef = getNowReslution();
            txtItem.width = Math.ceil(obj.width / docCoef);
            //这里暂时有点简单粗暴;
            txtItem.height = Math.ceil(obj.size / docCoef) + 10;
        }
        if(obj.fauxBold){
            txtItem.fauxBold = true;
        }
        if(obj.leading){
            txtItem.useAutoLeading = false;
            txtItem.leading = obj.leading;
        }
        txtItem.contents = obj.content;
        txtItem.font = obj.font;
        txtItem.size = obj.size;
        txtItem.color.rgb.hexValue = obj.color;
        var txtAlign = Justification.LEFT;
        if(obj.txtAlign){
                switch(obj.txtAlign){
                        case 'left': 
                            txtAlign = Justification.LEFT;
                            break;
                        case 'right': 
                            txtAlign = Justification.RIGHT;
                            break;
                        case 'center': 
                            txtAlign = Justification.CENTER;
                            break;
                }
        }
        txtItem.justification = txtAlign;
        txtItem.position = obj.position;
}

//设置前景色;
function setForegroundColor(hexValue){
    var color = new SolidColor();
    color.rgb.hexValue = hexValue;
    app.foregroundColor = color;
}
//获取前景色;
function getForegroundColor(){
    return app.foregroundColor.rgb.hexValue;
}

//设置背景色;
function setBackgroundColor(hexValue){
    var color = new SolidColor();
    color.rgb.hexValue = hexValue;
    app.backgroundColor = color;
}
//创建文件夹，并且移动n个图层到文件夹内;
function createLayerSet(obj){
        var doc = app.activeDocument;
        var layer = doc.activeLayer;
        var ls = doc.layerSets.add();
        ls.name = obj.name;
        if(obj.number){
            for(var i=0; i<obj.number; i++){
                    doc.artLayers[0].move(ls, ElementPlacement.PLACEATEND);
            }
        }
}

//获取选区的bounds以及width和height;
function getSelectionBounds(){
        var sel = app.activeDocument.selection.bounds;
        var tempObj = {
                left: Math.ceil(sel[0]),
                top: Math.ceil(sel[1]),
                right: Math.ceil(sel[2]),
                bottom: Math.ceil(sel[3]),
                width: Math.ceil(sel[2]) - Math.ceil(sel[0]),
                height: Math.ceil(sel[3]) - Math.ceil(sel[1])
        }
        return tempObj;
}

//设置当前页面透明度;
function setActiveLayerOpacity(opacity){
    app.activeDocument.activeLayer.fillOpacity = opacity;
}

//如果出错了，路径中一般会有一个huiSeZhiZhao名字的路径，先干掉他;
function checkErrorPathItem(){
    try{
        app.activeDocument.pathItems.getByName("huiSeZhiZhao").remove();
    }catch(e){}
}
//检测分辨率和单位是否是px;
function getCheckUnitAndResolution(){
    var flag = false;
    var tip = '';
    var u = app.preferences.rulerUnits;
    if(u != Units.PIXELS){ //如果标尺单位不是px,则强制转成像素;
        oldRuleUnits = app.preferences.rulerUnits;
        app.preferences.rulerUnits = Units.PIXELS;
    }
    /*if(!(app.activeDocument.resolution && app.activeDocument.resolution == 72)){
        flag = true;
        tip += '拉框助手只支持文档分辨率为72像素的文件,您当前文档的分辨率为：' + app.activeDocument.resolution;
    }*/
    //界面上没有拉框;
    try{
        var sel = app.activeDocument.selection.bounds;
    }catch(e){
        var sel = undefined;     
    }
    if(!sel){
        flag = true;
        tip += '你必须使用 [矩形选框工具] 在界面上拉一个框（矩形选区）';
    }
    
    //这里我本来是要判断是否在参数里面传递了arguments[0].bounds的，但是死活都读不到;
    //但是arguments[0].length却能读取得到;
    if(arguments[0].length == 1){
        //alert(JSON.stringify(arguments[0]));
        if(/bounds/.test(JSON.stringify(arguments[0]))){
           flag = false;
        }
    }
    return {
        flag: flag,
        tip: tip
    }
}
//先删掉huiSeZhiZhao的路径图层，再检查单位和分辨率;
function checkError(){
    checkErrorPathItem();
    var arg = arguments;
    var o = getCheckUnitAndResolution(arg);
    return o;
}
//函数完成，标尺单位转回为原来的单位,发出bee的声音提示;
function completeFn(){
    app.preferences.rulerUnits = oldRuleUnits;
    app.beep();
}

/*======================================================================================================== charts绘制图表类 */
//创建图表的格子(数据是Y轴);
function drawChartGrid(obj, w){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var layerNum = 0;
    var grid = obj.grid;
    var xAxis = obj.xAxis;
    var xAxisSplitLineShow = (xAxis && xAxis.splitLine && xAxis.splitLine.show) || false;
    var yAxis = obj.yAxis;
    var yAxisSplitLineShow = (yAxis && yAxis.splitLine && yAxis.splitLine.show) || false;
    var yAxisSplitLineLineStyleWidth = (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.width) ? yAxis.splitLine.lineStyle.width : 1;
    var xAxisSplitLineLineStyleWidth = (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.width) ? xAxis.splitLine.lineStyle.width : 1;
    var seriesArr = obj.series;
    var xSplitNum = (obj.xAxis && obj.xAxis.splitNumber && typeof(obj.xAxis.splitNumber) == 'number') ? obj.xAxis.splitNumber : 'auto';
    var vBlank;
    if(xSplitNum == 'auto'){
        xSplitNum = xAxis.data.length;
    }
    //vBlank = ((w.width - grid.left - grid.right) - (xSplitNum+1)*xAxisSplitLineLineStyleWidth) / xSplitNum;
    vBlank = ((w.width - grid.left - grid.right) - xSplitNum * xAxisSplitLineLineStyleWidth) / (xSplitNum - 1);
    
    //y轴的分割段数(原本是分割段数，为了和echarts保持一致，现在改为线条的个数,个数比间距多1，所以强制-1);
    var ySplitNum = (obj.yAxis && obj.yAxis.splitNumber && typeof(obj.yAxis.splitNumber) == 'number') ? obj.yAxis.splitNumber : 5;
    ySplitNum -= 1;
    var hBlank = ((w.height-grid.top-grid.bottom - yAxisSplitLineLineStyleWidth) / ySplitNum);
    var bgSetting = {
        hBlank:  hBlank,  //格子的每根横线间距 = grid的高度 / splitNumber;
        vBlank:  vBlank,  //格子每根竖线间距;
        
        xLineColor: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.color) ? xAxis.splitLine.lineStyle.color : 'cccccc',
        xLineWidth: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.width) ? xAxis.splitLine.lineStyle.width : 1,
        xLineType: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.type) ? xAxis.splitLine.lineStyle.type : 'solid',
        xLineOpacity: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.opacity) ? xAxis.splitLine.lineStyle.opacity : 100,
        yLineColor: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.color) ? yAxis.splitLine.lineStyle.color : 'cccccc',
        yLineWidth: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.width) ? yAxis.splitLine.lineStyle.width : 1,
        yLineType: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.type) ? yAxis.splitLine.lineStyle.type : 'solid',
        yLineOpacity: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.opacity) ? yAxis.splitLine.lineStyle.opacity : 100,
    }
    var range = obj.range;
    var yMax = Math.max(range.split(",")[0], range.split(",")[1]);
    var yMin = Math.min(range.split(",")[0], range.split(",")[1]);
    var xTxtArray = obj.xAxis.data || [];
    
    grid.height = (bgSetting.hBlank) * ySplitNum + bgSetting.xLineWidth;
    var bgHLineColor = new SolidColor();
    bgHLineColor.rgb.hexValue = bgSetting.yLineColor;
    
    if(yAxisSplitLineShow){
        //先绘制背景的横线格子,其实是y坐标上的线条;        
        /*for(var i=0; i<=ySplitNum; i++){
            var points = [
                [w.left + grid.left,  w.bottom - grid.bottom - bgSetting.hBlank*i - bgSetting.yLineWidth],
                [w.left + grid.left,  w.bottom - grid.bottom - bgSetting.hBlank*i],
                [w.right - grid.right,  w.bottom - grid.bottom - bgSetting.hBlank*i],
                [w.right - grid.right,  w.bottom - grid.bottom - bgSetting.hBlank*i - bgSetting.yLineWidth]
            ];
            //绘制多个选区;
            doc.selection.select(points,SelectionType.EXTEND, 0, false);
        }
        doc.selection.fill(bgHLineColor);
        doc.activeLayer.fillOpacity = bgSetting.yLineOpacity;
        doc.selection.deselect();
        if(bgSetting.yLineType == 'dashed'){
            hlineToDashed(w);
        }*/
        for(var i=0; i<=ySplitNum; i++){
            var points = [
                [w.left + grid.left,  w.bottom - grid.bottom - bgSetting.hBlank*i - bgSetting.yLineWidth],
                [w.right - grid.right,  w.bottom - grid.bottom - bgSetting.hBlank*i - bgSetting.yLineWidth]
            ];
            addLayer({name: 'Chart-grid-'+'hLine' + (i+1)});
            layerNum++;
            drawVectorShape({
                dots: points,
                type: 'stroke', 
                hexValue: bgSetting.yLineColor || '000000',
                solidWidth: bgSetting.yLineWidth || 1,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: false,
                strokeType: bgSetting.yLineType
            });
            if(bgSetting.yLineOpacity != 100){
                setActiveLayerOpacity(bgSetting.yLineOpacity);
            }

        }
    }

    //要绘制的竖线的个数,其实是x坐标上的线条;
    var vNum = Math.floor((w.width - grid.left - grid.right) / (bgSetting.vBlank+bgSetting.xLineWidth));
    //var vNum = Math.floor((w.width - grid.left - grid.right) / (bgSetting.vBlank+bgSetting.lineWidth));
    grid.width = w.width - grid.right - grid.left;
    if(xAxisSplitLineShow){
        //绘制背景竖向格子;
        /*addLayer({name: 'Chart-grid-'+'vLine'});
        layerNum++;
        for(var i=0; i<=vNum; i++){
            var _tempTop = w.bottom - grid.bottom - bgSetting.hBlank * ySplitNum;
            var points = [
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), _tempTop],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), w.bottom - grid.bottom],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank) + bgSetting.xLineWidth, w.bottom - grid.bottom],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank) + bgSetting.xLineWidth, _tempTop]
            ];
            
            doc.selection.select(points,SelectionType.EXTEND, 0, false);
        }
        var fillColor = new SolidColor();
        fillColor.rgb.hexValue = bgSetting.xLineColor;
        doc.selection.fill(fillColor);
        doc.activeLayer.fillOpacity = bgSetting.xLineOpacity;
        doc.selection.deselect();
        if(bgSetting.xLineType == 'dashed'){
            vlineToDashed(w);//变成虚线;
        }*/
        for(var i=0; i<=vNum; i++){
            var _tempTop = w.bottom - grid.bottom - bgSetting.hBlank * ySplitNum;
            var points = [
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), _tempTop],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), w.bottom - grid.bottom]
            ];
            addLayer({name: 'Chart-grid-'+'vLine'+ (i+1)});
            layerNum++;
            drawVectorShape({
                dots: points,
                type: 'stroke', 
                hexValue: bgSetting.xLineColor || '000000',
                solidWidth: bgSetting.xLineWidth || 1,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignOuterSide',
                closed: false,
                strokeType: bgSetting.xLineType
            });
            if(bgSetting.xLineOpacity != 100){
                setActiveLayerOpacity(bgSetting.xLineOpacity);
            }
        }

    }
    //绘制左侧刻度文字;
    for(var i=0; i<=ySplitNum; i++){
        var _tempTop = w.bottom - grid.bottom - i * bgSetting.hBlank;
        var name = (yMax-yMin) / ySplitNum * i + yMin;
        switch(UED.decimal){
            case 0: 
                name = returnInterger(name);
                break;
            case 1:
                name = returnFloatOne(name);
                break;
            case 2:
                name = returnFloat(name);
                break;
        }

        if(obj.yAxis.axisLabel && obj.yAxis.axisLabel.show){
            var yLabel = obj.yAxis.axisLabel;
            var yAxisColor = yLabel.color || '000000';
            var yfontSize = yLabel.fontSize || 12;
            var yMargin = typeof(yLabel.margin) == 'number' ?  yLabel.margin : 2;
            var yPadding = typeof(yLabel.padding) == 'number' ? yLabel.padding : yfontSize/2;
            addTxtLayer({
                name: name,
                font: UED.font.family,
                color: yAxisColor,
                size: yfontSize,
                position: [w.left + grid.left - yMargin, _tempTop + yPadding],
                txtAlign: 'right',
                content: name
            });
            layerNum++;
        }
    }
    //绘制底部x轴文字;
    for(var i=0; i<xTxtArray.length; i++){
        var gap = obj.xAxis.boundaryGap ? xTxtArray.length : xTxtArray.length - 1;
        //如果底部没有填写文字，gap就等于0了，0不能做被除数，所以就强制成1;
        var txtWidth = (w.width - grid.left - grid.right) / ((gap == 0) ? 1 : gap);
        var leftPos = (obj.xAxis.boundaryGap) ? w.left + grid.left + i*txtWidth + txtWidth/2 : w.left + grid.left + i*txtWidth;
        var txtAlign = 'center';
        /*if(!obj.xAxis.boundaryGap){
            if(i == 0){ txtAlign = 'left';}
            if(i == xTxtArray.length-1){ txtAlign = 'right';}
        }*/
        if(obj.xAxis.axisLabel && obj.xAxis.axisLabel.show){
            var xLabel = obj.xAxis.axisLabel;
            var xAxisColor = xLabel.color || '000000';
            var xfontSize = xLabel.fontSize || 12;
            var xMargin = typeof(xLabel.margin) == 'number' ?  xLabel.margin : xfontSize/2;
            addTxtLayer({
                name: xTxtArray[i],
                font: UED.font.family,
                color: xAxisColor,
                size: xfontSize,
                position: [leftPos, w.bottom - xMargin],
                txtAlign: txtAlign,
                content: xTxtArray[i]
            });
            layerNum++;
        }
    }
    //存在底部文字少于柱子图的个数，那么gapWidth就不应该用文字来分段了;
    var gapStep = xTxtArray.length;
    for(var i=0; i<seriesArr.length; i++){
        if(seriesArr[i].data){ //存在特殊情况，绘制随机数据的时候没有data;
            if(gapStep < seriesArr[i].data.length){
                    gapStep = seriesArr[i].data.length;
            }
        }
    }
    if(!obj.xAxis.boundaryGap){ gapStep -= 1;}
    var gapWidth = (w.width - grid.left - grid.right) / gapStep;
    return {
        layerNum: layerNum,
        gapWidth: gapWidth,
        yMax: yMax,
        yMin: yMin
    };
}
//横柱子的格子，绘制数据是x轴的格子;
function drawChartGridX(obj, w){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var layerNum = 0;
    var grid = obj.grid;
    var xAxis = obj.xAxis;
    var xAxisSplitLineShow = (xAxis && xAxis.splitLine && xAxis.splitLine.show) || false;
    var yAxis = obj.yAxis;
    var yAxisSplitLineShow = (yAxis && yAxis.splitLine && yAxis.splitLine.show) || false;
    var yAxisSplitLineLineStyleWidth = (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.width) ? yAxis.splitLine.lineStyle.width : 1;
    var xAxisSplitLineLineStyleWidth = (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.width) ? xAxis.splitLine.lineStyle.width : 1;
    var seriesArr = obj.series;
    /**** 原本这块是根据像素，间隔多少像素绘制一条竖线，现在改成和echarts一致，读取splitNumber;
    //yAxis.splitLine.interval如果是auto就根据柱子的个数来绘制分割线,否则根据像素来间隔开,这里的interval是y轴的，普通的柱子是在x轴;
    var yAxisSplitLineInterval = (yAxis && yAxis.splitLine && yAxis.splitLine.interval) ?  yAxis.splitLine.interval : 'auto';
    var hBlank;
    if(yAxisSplitLineInterval == 'auto'){
        yAxisSplitLineInterval = 1;
        for(var i=0; i<seriesArr.length; i++){ //通过循环，得到了最多柱子的个数;
            if(yAxisSplitLineInterval < seriesArr[i].data.length){
                yAxisSplitLineInterval = seriesArr[i].data.length;
            }
        }
        hBlank = ((w.height - grid.top - grid.bottom) - (yAxisSplitLineInterval+1)*yAxisSplitLineLineStyleWidth) / yAxisSplitLineInterval;
    }else{
        hBlank = parseInt(yAxisSplitLineInterval, 10);
    }*/
    var ySplitNum = (obj.yAxis && obj.yAxis.splitNumber && typeof(obj.yAxis.splitNumber) == 'number') ? obj.yAxis.splitNumber : 'auto';
    var hBlank;
    if(ySplitNum == 'auto'){
        ySplitNum = yAxis.data.length + 1;
    }
    hBlank = ((w.height - grid.top - grid.bottom) - ySplitNum * yAxisSplitLineLineStyleWidth) / (ySplitNum - 1);



    //x轴的分割段数;
    var xSplitNum = (obj.xAxis && obj.xAxis.splitNumber && typeof(obj.xAxis.splitNumber) == 'number') ? obj.xAxis.splitNumber : 5;
    //xSplitNum -= 1;
    var bgSetting = {
        hBlank: hBlank, //格子的每根横线间距;
        //vBlank: (w.width - grid.left - grid.right) / xSplitNum - xAxisSplitLineLineStyleWidth  - xAxisSplitLineLineStyleWidth/2, //格子的每根竖线间距;
        //vBlank: ((w.width - grid.left - grid.right - xAxisSplitLineLineStyleWidth* (xSplitNum+1))) / xSplitNum,
        vBlank: ((w.width - grid.left - grid.right - xAxisSplitLineLineStyleWidth* (xSplitNum))) / (xSplitNum-1),
        
        xLineColor: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.color) ? xAxis.splitLine.lineStyle.color : 'cccccc',
        xLineWidth: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.width) ? xAxis.splitLine.lineStyle.width : 1,
        xLineType: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.type) ? xAxis.splitLine.lineStyle.type : 'solid',
        xLineOpacity: (xAxis && xAxis.splitLine && xAxis.splitLine.lineStyle && xAxis.splitLine.lineStyle.opacity) ? xAxis.splitLine.lineStyle.opacity : 100,
        yLineColor: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.color) ? yAxis.splitLine.lineStyle.color : 'cccccc',
        yLineWidth: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.width) ? yAxis.splitLine.lineStyle.width : 1,
        yLineType: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.type) ? yAxis.splitLine.lineStyle.type : 'solid',
        yLineOpacity: (yAxis && yAxis.splitLine && yAxis.splitLine.lineStyle && yAxis.splitLine.lineStyle.opacity) ? yAxis.splitLine.lineStyle.opacity : 100
    }
    var range = obj.range;
    var xMax = Math.max(range.split(",")[0], range.split(",")[1]);
    var xMin = Math.min(range.split(",")[0], range.split(",")[1]);
    var yTxtArray = obj.yAxis.data || [];
    
    
    //grid.width = (bgSetting.vBlank) * xSplitNum + bgSetting.yLineWidth;
    //grid.width = (bgSetting.vBlank) * (xSplitNum-1) + bgSetting.yLineWidth;
    grid.width = w.width - grid.left - grid.right;
    grid.height = w.height - grid.top - grid.bottom;
    var bgHLineColor = new SolidColor();
    bgHLineColor.rgb.hexValue = bgSetting.yLineColor;
    
    if(yAxisSplitLineShow){
        //先绘制背景的横线格子,其实是y坐标上的线条;
        /*addLayer({name: 'Chart-grid-'+'hLine'});
        layerNum++;
        var hNum = Math.floor((w.height - grid.top - grid.bottom) / (bgSetting.hBlank+bgSetting.yLineWidth));
        for(var i=0; i<=hNum; i++){
            var points = [
                [w.left + grid.left,  w.bottom - grid.bottom - (bgSetting.hBlank + bgSetting.yLineWidth)*i - bgSetting.yLineWidth],
                [w.left + grid.left,  w.bottom - grid.bottom - (bgSetting.hBlank + bgSetting.yLineWidth)*i],
                [w.right - grid.right,  w.bottom - grid.bottom - (bgSetting.hBlank + bgSetting.yLineWidth)*i],
                [w.right - grid.right,  w.bottom - grid.bottom - (bgSetting.hBlank + bgSetting.yLineWidth)*i - bgSetting.yLineWidth]
            ];
            //绘制多个选区;
            doc.selection.select(points,SelectionType.EXTEND, 0, false);
        }
        doc.selection.fill(bgHLineColor);
        doc.activeLayer.fillOpacity = bgSetting.yLineOpacity;
        doc.selection.deselect();
        if(bgSetting.yLineType == 'dashed'){
            hlineToDashed(w);
        }*/
        var hNum = Math.floor((w.height - grid.top - grid.bottom) / (bgSetting.hBlank+bgSetting.yLineWidth));
        for(var i=0; i<=hNum; i++){
            var points = [
                [w.left + grid.left,  w.bottom - grid.bottom - (bgSetting.hBlank + bgSetting.yLineWidth)*i - bgSetting.yLineWidth],
                [w.right - grid.right,  w.bottom - grid.bottom - (bgSetting.hBlank + bgSetting.yLineWidth)*i - bgSetting.yLineWidth]
            ];
            addLayer({name: 'Chart-grid-'+'hLine'+ (i+1)});
            layerNum++;
            drawVectorShape({
                dots: points,
                type: 'stroke', 
                hexValue: bgSetting.yLineColor || '000000',
                solidWidth: bgSetting.yLineWidth || 1,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignInside',
                closed: false,
                strokeType: bgSetting.yLineType
            });
            if(bgSetting.yLineOpacity != 100){
                setActiveLayerOpacity(bgSetting.yLineOpacity);
            }
        }
    }
    if(xAxisSplitLineShow){
        //绘制背景竖向格子;
        /*addLayer({name: 'Chart-grid-'+'vLine'});
        layerNum++;
        for(var i=0; i<=xSplitNum; i++){
            //var _tempTop = w.bottom - grid.bottom - bgSetting.hBlank * xSplitNum;
            var _tempTop = w.top;
            var points = [
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), _tempTop],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), w.bottom - grid.bottom],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank) + bgSetting.xLineWidth, w.bottom - grid.bottom],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank) + bgSetting.xLineWidth, _tempTop]
             ];
            
            doc.selection.select(points,SelectionType.EXTEND, 0, false);
        }
        var fillColor = new SolidColor();
        fillColor.rgb.hexValue = bgSetting.xLineColor;
        doc.selection.fill(fillColor);
        doc.activeLayer.fillOpacity = bgSetting.xLineOpacity;
        doc.selection.deselect();
        if(bgSetting.xLineType == 'dashed'){
            vlineToDashed(w);//变成虚线;
        }*/
        for(var i=0; i<xSplitNum; i++){
            var _tempTop = w.top;
            var points = [
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), _tempTop],
                [w.left+grid.left + i*(bgSetting.xLineWidth + bgSetting.vBlank), w.bottom - grid.bottom]
            ];
            addLayer({name: 'Chart-grid-'+'vLine' + (i+1)});
            layerNum++;
            drawVectorShape({
                dots: points,
                type: 'stroke', 
                hexValue: bgSetting.xLineColor || '000000',
                solidWidth: bgSetting.xLineWidth || 1,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignOuterSide',
                closed: false,
                strokeType: bgSetting.xLineType
            });
            if(bgSetting.xLineOpacity != 100){
                setActiveLayerOpacity(bgSetting.xLineOpacity);
            }
        }
    }
    //绘制左侧文字;
    var leftStep = grid.height / yTxtArray.length;
    var gap = obj.yAxis.boundaryGap ? yTxtArray.length : yTxtArray.length - 1;
    var txtWidth = (w.height - grid.top - grid.bottom) / gap;
    for(var i=0; i<yTxtArray.length; i++){
        var topPos = (obj.yAxis.boundaryGap) ? w.top + grid.top + i*txtWidth + txtWidth/2 : w.top + grid.top + i*txtWidth;                
        if(obj.yAxis.axisLabel && obj.yAxis.axisLabel.show){
            var yLabel = obj.yAxis.axisLabel;
            var yAxisColor = yLabel.color || '000000';
            var yfontSize = yLabel.fontSize || 12;
            var yMargin = typeof(yLabel.margin) == 'number' ?  yLabel.margin : 2;
            var yPadding = typeof(yLabel.padding) == 'number' ? yLabel.padding : yfontSize/2;
            addTxtLayer({
                name: yTxtArray[i],
                font: UED.font.family,
                color: yAxisColor,
                size: yfontSize,
                position: [w.left + grid.left - yMargin, parseInt(topPos+yPadding, 10)],
                txtAlign: 'right',
                content: yTxtArray[i]
            });
            layerNum++;
        }
    }
    
    //绘制底部x轴文字;
    for(var i=0; i<xSplitNum; i++){
        var gap = parseInt(grid.width / (xSplitNum-1), 10);
        var txtWidth = gap;
        //var leftPos = w.left + grid.left + i*txtWidth + i*bgSetting.xLineWidth + bgSetting.xLineWidth/2;
        var leftPos = w.left + grid.left + i*txtWidth + bgSetting.xLineWidth/2;
        var txtAlign = 'center';
        var txtName = (xMax - xMin) / (xSplitNum-1) * i + xMin;
        switch(UED.decimal){
            case 0: 
                txtName = returnInterger(txtName);
                break;
            case 1:
                txtName = returnFloatOne(txtName);
                break;
            case 2:
                txtName = returnFloat(txtName);
                break;
        }

        if(obj.xAxis.axisLabel && obj.xAxis.axisLabel.show){
            var xLabel = obj.xAxis.axisLabel;
            var xAxisColor = xLabel.color || '000000';
            var xfontSize = xLabel.fontSize || 12;
            var xMargin = typeof(xLabel.margin) == 'number' ?  xLabel.margin : xfontSize/2;
            addTxtLayer({
                name: txtName,
                font: UED.font.family,
                color: xAxisColor,
                size: xfontSize,
                position: [leftPos, w.bottom - xMargin],
                txtAlign: txtAlign,
                content: txtName
            });
            layerNum++;
        }
    }
    //存在底部文字少于柱子图的个数，那么gapWidth就不应该用文字来分段了;
    var gapStep = seriesArr[0].data.length;
    for(var i=0; i<seriesArr.length; i++){
        if(gapStep > seriesArr[i].data.length){
            gapStep = seriesArr[i].data.length;
        }
    }
    //var gapWidth = (w.height - grid.top - grid.bottom) / gapStep;
    var gapWidth = (grid.height - (gapStep+1) * bgSetting.yLineWidth) / gapStep;
    return {
        layerNum: layerNum,
        gapWidth: gapWidth,
        xMax: xMax,
        xMin: xMin
    };
}
//绘制折线图;
function drawLineChart(json){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(json);
    var grid = json.grid;
    var series = json.series;
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var layerNum = 0;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    //判断值是否在范围内，如果超出了范围，则改变范围;
    var jsonMax = Math.max(Number(json.range.split(',')[0]), Number(json.range.split(',')[1]));
    var jsonMin = Math.min(Number(json.range.split(',')[0]), Number(json.range.split(',')[1]));
    var changeRange = false;
    for(var i=0; i<series.length; i++){
        var data = series[i].data;
        if(typeof(data) != 'undefined'){
            for(var j=0; j<data.length; j++){
                var currentNum = Number(data[j]);
                if(currentNum > jsonMax){
                    jsonMax = currentNum;
                    changeRange = true;
                }
                if(currentNum < jsonMin){
                    jsonMin = currentNum;
                    changeRange = true;
                }
            }
        }
    }
    if(changeRange){
        json.range = jsonMin + "," +jsonMax;
    }

    var returnObj = drawChartGrid(json, w);
    var layerNum = returnObj.layerNum;
    var yMax = returnObj.yMax;
    var yMin = returnObj.yMin;    
    
    //循环整个data属性，看需要绘制几条线段;
    for(var i=0; i<series.length; i++){
        var dots = []; //将所有的点存储在这里，如果需要绘制面积图，则还需要使用;
        var lineDots = []; //要绘制的线条的所有点，存储在这里;
        var serie = series[i];
        var data = series[i].data;
        var dataRange = series[i].dataRange || false; //随机数据范围;
        var line = {
            color: (serie.lineStyle && serie.lineStyle.color) ? serie.lineStyle.color : '000000',
            width: (serie.lineStyle && serie.lineStyle.width) ? serie.lineStyle.width :  0,
            opacity: (serie.lineStyle && serie.lineStyle.opacity) ? serie.lineStyle.opacity : 100,
            //圆角idstrokeStyleRoundJoin 直角strokeStyleMiterJoin 斜角strokeStyleBevelJoin;
            angleDot: (serie.lineStyle && serie.lineStyle.angleDot) ? serie.lineStyle.angleDot : 'idstrokeStyleRoundJoin'
        }
        var area = {
            color : (serie.areaStyle && serie.areaStyle.color) ? serie.areaStyle.color : false,
            opacity: (serie.areaStyle && serie.areaStyle.opacity) ? serie.areaStyle.opacity : 100
        }
        var markPoint = {
            symbolSize: (serie.markPoint && serie.markPoint.symbolSize) ? serie.markPoint.symbolSize : 0,
            color: (serie.markPoint && serie.markPoint.color) ? serie.markPoint.color : 'eeeeee',
            opacity: (serie.markPoint && serie.markPoint.opacity) ? serie.markPoint.opacity : 100,
            borderWidth: (serie.markPoint && serie.markPoint.borderWidth) ? serie.markPoint.borderWidth : 0,
            borderColor: (serie.markPoint && serie.markPoint.borderColor) ? serie.markPoint.borderColor : '000000',
            borderOpacity: (serie.markPoint && serie.markPoint.borderOpacity) ? serie.markPoint.borderOpacity : 100
        }
        if(dataRange){ //不存在点数据，绘制范围内的随机数据;
            var gap = dataRange.gap || 10;
            var dotNum = Math.floor(grid.width / gap);
            var dataMin = dataRange.min || yMin;
            var dataMax = dataRange.max || yMax;
            data = [];
            for(var k=0; k<=dotNum; k++){
                var dataX =  w.left + grid.left + k * gap;
                var randomY = Math.floor(Math.random() * (dataMax - dataMin + 1)) + dataMin;
                data.push(randomY);
            }
        };//end if else;
        
        for(var n=0; n<data.length; n++){ //循环data数据，需要绘制多少个点;
            var x = w.left + grid.left + grid.width / (data.length-1) * n;
            var numberDataN = Number(data[n]);
            var y = Number(w.bottom - grid.bottom - (grid.height / (yMax - yMin) * (numberDataN-yMin)));
            if(!serie.smooth){ //不需要绘制圆滑曲线;
                dots.push([x, y]);
                lineDots.push([x, y]);
            }else{
                var smoothNum = serie.smoothNumber || 0.5;
                var twoDotsEdge = parseInt(grid.width / (data.length-1), 10);
                var leftPos = [x+twoDotsEdge * smoothNum, y];
                var rightPos = [x-twoDotsEdge * smoothNum, y];
                if(n == 0){ //第一个点，则不需要有弯曲度，否则面积图;
                    rightPos = [x, y];
                }
                //如果这是最后一个点，则也不需要有弯曲度;
                if(n ==(data.length-1)){
                    leftPos = [x, y];
                }
                dots.push({
                    anchor: [x, y],
                    left: leftPos,
                    right: rightPos
                });
                lineDots.push({
                    anchor: [x, y],
                    left: leftPos,
                    right: rightPos
                });
            };//end if else;
        };//end for;
        //需要绘制面积图;
        if(area.color){
            addLayer({name: 'Chart-Area'});
            layerNum++;
            dots.push([w.right-grid.right, w.bottom - grid.bottom]);
            dots.push([w.left + grid.left , w.bottom - grid.bottom]);
            /*if(typeof(area.color) == 'string'){ //绘制纯色;
                drawShape({
                    dots:  dots,
                    type: 'fill',  
                    hexValue: area.color,
                    opacity: 100
                });
            }else if(typeof(area.color) == 'object'){ //绘制渐变;
                drawShape({
                    dots:  dots,
                    type: 'selection'
                });
                var sel = getSelectionBounds();
                var x = area.color.x,
                    y = area.color.y,
                    x2 = area.color.x2 || null,
                    y2 = area.color.y2 || null,
                    rPos = area.color.r || null,
                    startColor = area.color.colors[0],
                    endColor = area.color.colors[1];
                        
                if(area.color.type == 'linear'){ //线性渐变;
                    lineGradient({
                        startX: sel.width * x + sel.left,
                        endX: sel.width * x2 + sel.left,
                        startY: sel.height * y + sel.top,
                        endY: sel.height * y2 + sel.top,
                        startColor: startColor,
                        endColor: endColor
                    });
                }else if(area.color.type == 'radial') {
                    radialGradient({
                        x: sel.width * x + sel.left,
                        y: sel.height * y + sel.top,
                        r: (sel.width * rPos),
                        startColor: startColor,
                        endColor: endColor
                    });
                }
                doc.selection.deselect();
            }*/
            if(typeof(area.color) == 'string'){ //绘制纯色;
                drawVectorShape({
                    dots: dots,
                    type: 'fill', 
                    hexValueFill: area.color,
                    closed: true,
                    angleDot: line.angleDot
                });
            }else if(typeof(area.color) == 'object'){ //绘制渐变;
                drawVectorShape({
                    dots: dots,
                    type: 'fill', 
                    hexValueFill: area.color,
                    closed: true,
                    angleDot: line.angleDot
                });
            }

            if(area.opacity != 100){ //不管是绘制纯色还是渐变，最后都设置透明度;
                setActiveLayerOpacity(area.opacity);
            }
        }
        if(line.width > 0){
            addLayer({name: 'Chart-Line'});
            layerNum++;                           
            drawVectorShape({
                dots: lineDots,
                type: 'stroke', 
                hexValue: line.color,
                solidWidth: line.width,
                angleDot: line.angleDot
            });
            if(line.opacity != 100){
                setActiveLayerOpacity(line.opacity);
            }
        }
        //曲线上如果要绘制点;
        if(markPoint.symbolSize > 0){            
            for(var n=0; n<lineDots.length; n++){
                var xpos = (lineDots[n] instanceof Array) ? lineDots[n][0] : lineDots[n].anchor[0];
                var ypos = (lineDots[n] instanceof Array) ? lineDots[n][1] : lineDots[n].anchor[1];
                var startX = xpos - markPoint.symbolSize/2;
                var startY = ypos - markPoint.symbolSize/2;
                var endX = xpos + markPoint.symbolSize/2;
                var endY = ypos + markPoint.symbolSize/2;
                
                addLayer({name: '标记小圆点 ' + data[n]});
                layerNum++;                
                var cirObj = {
                    bgColor: markPoint.color,
                    top: startY,
                    left: startX,
                    diam: markPoint.symbolSize
                }
                if(markPoint.borderWidth > 0){
                    cirObj.borderWidth = markPoint.borderWidth;
                    cirObj.borderColor = markPoint.borderColor;
                    cirObj.strokeStyleAlign = 'strokeStyleAlignOuterSide';
                }
                drawVerctorCircle(cirObj);
                if(markPoint.opacity != 100){ //不管是绘制纯色还是渐变，最后都设置透明度;
                    setActiveLayerOpacity(markPoint.opacity);
                }

                /*drawCirclePath({
                    startX: startX,
                    startY: startY,
                    endX: endX,
                    endY: endY,
                    type: 'fill',
                    hexValue: markPoint.color,
                    opacity: markPoint.opacity
                });
                if(markPoint.borderWidth > 0){
                    drawCirclePath({
                        startX: startX,
                        startY: startY,
                        endX: endX,
                        endY: endY,
                        type: 'stroke',
                        location: 'outside',
                        hexValue: markPoint.borderColor,
                        solidWidth: markPoint.borderWidth
                    });
                }*/
            };//end for;
        }
    };//end for;
    createLayerSet({
        name: 'Line Chart',
        number: layerNum
    });
    completeFn();
}
//绘制横柱状图;
function drawHColumnChart(json){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(json);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var jsonRangeArr = json.range.split(',');
    var maxRange = Math.max(Number(jsonRangeArr[0]), Number(jsonRangeArr[1]));
    var minRange = Math.min(Number(jsonRangeArr[0]), Number(jsonRangeArr[1]));
    var superposition = json.superposition || false; //是否是堆叠柱子图;
    //如果是堆叠图，正数全部加一起，负数全部加一起，最大值就是正数的和，最小值就是负数的和;
    if(superposition){
        //nowYposArr里面记录{yMin: 0, yMax: 100}类似这样的;
        var nowYposArr = [];
        var rangePositive = [];
        var rangeNegative = [];
        for(var i=0; i<json.series.length; i++){
            for(j=0; j<json.series[i].data.length; j++){
                var dataValue = Number(json.series[i].data[j].value);
                if(typeof(rangePositive[j]) == "undefined"){
                    _p = dataValue > 0 ? dataValue : 0;
                    _n = dataValue < 0 ? dataValue : 0;
                    rangePositive.push(_p);
                    rangeNegative.push(_n);
                }else{
                    if(dataValue > 0){
                        rangePositive[j] += dataValue;
                    }else{
                        rangeNegative[j] += dataValue;
                    }
                }
            }
        }
        var maxNumber = Math.max.apply(null, rangePositive);
        var minNumber = Math.min.apply(null, rangeNegative);
        if(maxNumber > maxRange || minNumber < minRange){
            var newMin = Math.min(minNumber, minRange);
            var newMax = Math.max(maxNumber, maxRange);
            json.range = newMin + ',' + newMax;
        }
    }else{
        //如果不是堆叠图，同样需要循环数据，看是否有数据超过了数据范围;
        var changeRangeFlag = false;
        for(var i=0; i<json.series.length; i++){
            for(j=0; j<json.series[i].data.length; j++){
                var dataValue = Number(json.series[i].data[j].value);
                if(dataValue > maxRange){
                    maxRange = dataValue;
                    changeRangeFlag = true;
                }
                if(dataValue < minRange){
                    minRange = dataValue;
                    changeRangeFlag = true;
                }  
            }
        }
        json.range = minRange + ',' + maxRange;
    }

    var returnObj = drawChartGridX(json, w);
    var yLineWidth = (json.yAxis.splitLine && json.yAxis.splitLine.lineStyle && json.yAxis.splitLine.lineStyle.width) ? json.yAxis.splitLine.lineStyle.width : 1;
    
    var layerNum = returnObj.layerNum;
    var xMax = returnObj.xMax;
    var xMin = returnObj.xMin;
    var grid = json.grid;
    //绘制柱子，先检测出要绘制的所有柱子的高度;
    var gapWidth = returnObj.gapWidth; //这里其实是高度;  
    var barGap = json.series[0].barGap || 0; //如果一栏里面有多个柱子，柱子与柱子之间的间距，单位px;
    var cloumnWidthArr = [];//每根柱子的高度是整个空白区域的宽度的10%;
    var oneGapWidth = 0; //其实是高度;
    var maxColumnWidth = 0; //当是堆叠图的时候，则要记录最大的柱子宽度(实际上堆叠图应该设置所有柱子宽度都一样);
    for(var i=0, len=json.series.length; i<len; i++){//先循环一次，主要是得到所有柱子高度的总和;
        var barWidth = json.series[i].barWidth;
        if(barWidth){
            //有可能是百分比，也可能是px;
            var reg = /[%]$/;
            var reg2 = /[px]$/;
            if(reg.test(barWidth)){
                var cloumnW = parseInt(barWidth.replace(/[%]$/, ''), 10) / 100;
                cloumnWidthArr.push(parseInt(gapWidth / len * cloumnW, 10));
                oneGapWidth += parseInt((gapWidth / len * cloumnW), 10);
            }else if(reg2.test(barWidth)){
                var cloumnW = parseInt(barWidth.replace(/[px]$/, ''), 10);
                cloumnWidthArr.push(cloumnW);
                oneGapWidth += cloumnW;
            }else{
                cloumnWidthArr.push(parseInt(barWidth,10));
                oneGapWidth += parseInt(barWidth,10);
            }
        }else{ //默认值是10%;
            cloumnWidthArr.push(parseInt(gapWidth / len * 0.1, 10));
            oneGapWidth += parseInt((gapWidth / len * 0.1), 10);
        }
        if(barWidth > maxColumnWidth){
            maxColumnWidth = parseInt(barWidth, 10);
        }
    }
    if(barGap != 0){
        oneGapWidth += barGap * (json.series.length -1);
    }
    //如果是堆叠图,两根柱子间的间距不需要;
    if(superposition){
        barGap = 0;
        oneGapWidth = maxColumnWidth;
    }
    //堆叠图主要是靠columnPositionArr数组里面记录up和down属性;
    //遇到正数就往up里面加，遇到负数就往down里面加。然后每个柱子分别移动up或down的值;
    var columnPositionArr = [];    
    //开始绘制柱子;
    for(var i=0; i<json.series.length; i++){
        for(var j=0; j<json.series[i].data.length; j++){
            var data = (json.series[i].data[j].constructor == Object) ? json.series[i].data[j].value : json.series[i].data[j];
            addLayer({name: 'Chart-Column' + j + "-" + data});
            var columnH = (gapWidth - oneGapWidth)/2 + gapWidth * j + yLineWidth*j + yLineWidth;
            var columnW = (grid.width / (xMax - xMin) *parseInt(data, 10)-xMin);
            var x = w.left + grid.left;
            var y = parseInt(w.top + grid.top + columnH + getArrayNumCount(cloumnWidthArr ,i), 10);
            //如果是堆叠图;
            if(superposition){
                y = parseInt(w.top + grid.top + columnH, 10);
            }
            if(i != 0 && barGap != 0){
                y += barGap * i
            }
            var numberDataN = Number(data);
            var dots; //要考虑负数的情况，也就是从0作为底部，而不是从最左侧开始绘制;
            if(xMin >= 0){
                dots = [
                    [x, y],
                    [x, y+cloumnWidthArr[i]],
                    [x+parseInt(columnW, 10),  y+cloumnWidthArr[i]],
                    [x+parseInt(columnW, 10), y]
                ];
                //如果是堆叠图;
                if(superposition){
                    var columnHeight = dots[2][0] - dots[1][0];
                    //不存在记录柱子位移信息;
                    if(typeof(columnPositionArr[j]) != 'object'){
                        columnPositionArr[j] = {
                            up: (numberDataN > 0) ?  columnHeight: 0,
                            down: (numberDataN < 0) ? columnHeight : 0
                        }
                    }else{
                        var movePos = (numberDataN >= 0) ? columnPositionArr[j].up : -columnPositionArr[j].down;
                        dots = arrayAddXNum(dots, movePos);
                        if(numberDataN > 0){
                            columnPositionArr[j].up += columnHeight;                            
                        }else{
                            columnPositionArr[j].down += columnHeight;
                        }
                    }
                }
            }else{
                var zeroNum = grid.width * (0-xMin) / (xMax - xMin);
                var columnW2 = grid.width * (Number(data)-xMin) / (xMax - xMin);
                if(data >= 0){
                    dots = [
                        [x + zeroNum, y],
                        [x + zeroNum, y+cloumnWidthArr[i]],
                        [x + columnW2,  y+cloumnWidthArr[i]],
                        [x + columnW2, y]
                    ];
                }else{
                    dots = [
                        [x + columnW2, y],
                        [x + columnW2,  y+cloumnWidthArr[i]],
                        [x + zeroNum, y+cloumnWidthArr[i]],
                        [x + zeroNum, y]
                    ];                    
                }
                //如果是堆叠图;
                if(superposition){
                    var columnHeight = dots[2][0] - dots[1][0];
                    if(typeof(columnPositionArr[j]) != 'object'){
                        columnPositionArr[j] = {
                            up: (numberDataN > 0) ? columnHeight : 0,
                            down: (numberDataN < 0) ? columnHeight : 0
                        }
                    }else{
                        var movePos = (numberDataN >= 0) ? columnPositionArr[j].up : -columnPositionArr[j].down;
                        dots = arrayAddXNum(dots, movePos);
                        if(numberDataN > 0){
                            columnPositionArr[j].up += columnHeight;                            
                        }else{
                            columnPositionArr[j].down += columnHeight;
                        }
                    }
                }
            }
            
            var borderRadiusFlag = false;
            //如果是圆角，则绘制圆角的;
            if(json.series[i].data[j].constructor == Object && json.series[i].data[j].itemStyle && json.series[i].data[j].itemStyle.barBorderRadius){
                var barBorderRadius = json.series[i].data[j].itemStyle.barBorderRadius;
                borderRadiusFlag = true;
                dots = getConnerDots(dots, barBorderRadius);
            }
            layerNum++;
            var fillColor = json.series[i].data[j].itemStyle.color;
            drawVectorShape({
                dots: dots,
                type: 'fill',
                hexValueFill: fillColor,
                closed: true
            });
            /*if((json.series[i].data[j].constructor == Object && json.series[i].data[j].itemStyle && json.series[i].data[j].itemStyle.color)){
                if(typeof(json.series[i].data[j].itemStyle.color) == 'string'){
                    fillColor = json.series[i].data[j].itemStyle.color;
                    drawShape({
                        dots: dots,
                        type: 'fill', 
                        hexValue: fillColor,
                        opacity: 100
                    });
                }
            }else{
                if((json.series[i].data[j].constructor == Object && json.series[i].data[j].itemStyle && json.series[i].data[j].itemStyle.colors)){
                    if(json.series[i].data[j].itemStyle.type == 'linear'){
                        drawShape({
                            dots: dots,
                            type: 'selection'
                        });
                        var _w = parseInt(columnW, 10);
                        var _h = cloumnWidthArr[i];
                        var _x = json.series[i].data[j].itemStyle.x;
                        var _y = json.series[i].data[j].itemStyle.y;
                        var _x2 = json.series[i].data[j].itemStyle.x2;
                        var _y2 = json.series[i].data[j].itemStyle.y2;
                        var _startColor =  json.series[i].data[j].itemStyle.colors[0];
                        var _endColor = json.series[i].data[j].itemStyle.colors[1];
                        lineGradient({
                            startX: x + _x * _w,
                            endX:  x  + _x2 * _w,
                            startY: y + _y * _h,
                            endY:  y + _y2 * _h,
                            startColor: _startColor,
                            endColor: _endColor
                        });
                        doc.selection.deselect();
                    }
                }else{
                    fillColor = json.areaColor[i];
                    drawShape({
                        dots: dots,
                        type: 'fill', 
                        hexValue: fillColor,
                        opacity: 100
                    });
                }
            }*/
        };//end for;
    };//end for;
    //文字不能被柱子压住，所以不得不多循环一次(当柱子高度比较细的时候，依旧会被遮挡，所以不得不多循环一次);
    columnPositionArr.length = 0;
    for(var i=0; i<json.series.length; i++){
        for(var j=0; j<json.series[i].data.length; j++){
            //绘制文字;
            if(json.series[i].label && json.series[i].label.show){ //要在柱子上面绘制文字;
                var data = (json.series[i].data[j].constructor == Object) ? json.series[i].data[j].value : json.series[i].data[j];
                var series = json.series[i];
                var columnH = (gapWidth - oneGapWidth)/2 + gapWidth * j + yLineWidth*j + yLineWidth;
                var txtColor = series.label.color || '333333';
                var distance = series.label.distance || 5;
                var fontSize = series.label.fontSize || 12;
                var numberDataN = Number(data);

                if(xMin >= 0){
                    var columnW = (grid.width / (xMax - xMin) *parseInt(data, 10)-xMin);
                }else{
                    var columnW = (Number(data) - xMin) * grid.width / (xMax - xMin);
                }                

                //如果是堆叠图;
                if(superposition){
                    var y = parseInt(w.top + grid.top + columnH, 10);
                    var txtY = y+cloumnWidthArr[i] - cloumnWidthArr[i]/2;
                    var txtYPos = txtY + fontSize/2;

                    var columnHeight = 0;
                    if(xMin >= 0){
                        columnHeight = (grid.width / (xMax - xMin) *parseInt(data, 10)-xMin);
                    }else{
                        var zeroNum = (0 - xMin) * grid.width / (xMax - xMin);
                        var _temp = (numberDataN - xMin) * grid.width / (xMax - xMin);
                        columnHeight = (numberDataN >= 0) ? _temp - zeroNum : zeroNum - _temp;
                    }

                    if(typeof(columnPositionArr[j]) != 'object'){
                        columnPositionArr[j] = {
                            up: (numberDataN > 0) ?  columnHeight: 0,
                            down: (numberDataN < 0) ? columnHeight : 0
                        }
                        var movePos = 0;
                    }else{
                        var movePos = (numberDataN >= 0) ? columnPositionArr[j].up : -columnPositionArr[j].down;                        
                        if(numberDataN > 0){
                            columnPositionArr[j].up += columnHeight;                            
                        }else{
                            columnPositionArr[j].down += columnHeight;
                        }
                    }       
                    var txtX = w.left + grid.left + columnW;
                    var txtXPos = (numberDataN >= 0) ? txtX + distance + movePos : txtX - distance + movePos;
                }else{
                    var txtX = w.left + grid.left + columnW;
                    var txtXPos = (data >= 0) ? txtX + distance : txtX - distance;
                    var y = parseInt(w.top + grid.top + columnH + getArrayNumCount(cloumnWidthArr ,i), 10);
                    var txtY = y+cloumnWidthArr[i] - cloumnWidthArr[i]/2;
                    var txtYPos = txtY + fontSize/2;
                }
                
                addTxtLayer({
                    name: data,
                    content: data,
                    font: UED.font.family,
                    size: fontSize,
                    color: txtColor,
                    txtAlign: (data >= 0) ? 'left' : 'right',
                    position:[txtXPos, txtYPos]
                });
                layerNum++;
            };//end if;
        }
    }
    createLayerSet({
        name: 'Column Chart',
        number: layerNum
    });
    completeFn();
}
//绘制柱状图;
function drawColumnChart(json){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(json);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    var jsonRangeArr = json.range.split(',');
    var maxRange = Math.max(Number(jsonRangeArr[0]), Number(jsonRangeArr[1]));
    var minRange = Math.min(Number(jsonRangeArr[0]), Number(jsonRangeArr[1]));
    var superposition = json.superposition || false; //是否是堆叠柱子图;
    //如果是堆叠图，正数全部加一起，负数全部加一起，最大值就是正数的和，最小值就是负数的和;
    if(superposition){
        //nowYposArr里面记录{yMin: 0, yMax: 100}类似这样的;
        var nowYposArr = [];
        var rangePositive = [];
        var rangeNegative = [];
        for(var i=0; i<json.series.length; i++){
            for(j=0; j<json.series[i].data.length; j++){
                var dataValue = Number(json.series[i].data[j].value);
                if(typeof(rangePositive[j]) == "undefined"){
                    _p = dataValue > 0 ? dataValue : 0;
                    _n = dataValue < 0 ? dataValue : 0;
                    rangePositive.push(_p);
                    rangeNegative.push(_n);
                }else{
                    if(dataValue > 0){
                        rangePositive[j] += dataValue;
                    }else{
                        rangeNegative[j] += dataValue;
                    }
                }
            }
        }
        var maxNumber = Math.max.apply(null, rangePositive);
        var minNumber = Math.min.apply(null, rangeNegative);        
        if(maxNumber > maxRange || minNumber < minRange){
            var newMin = Math.min(minNumber, minRange);
            var newMax = Math.max(maxNumber, maxRange);
            json.range = newMin + ',' + newMax;
        }
    }else{
        //如果不是堆叠图，则也要判断数据最大值和最小值是否超过了范围;
        var changeRangeFlag = false;
        for(var i=0; i<json.series.length; i++){
            for(j=0; j<json.series[i].data.length; j++){
                var dataValue = Number(json.series[i].data[j].value);
                if(dataValue > maxRange){
                    maxRange = dataValue;
                    changeRangeFlag = true;
                }
                if(dataValue < minRange){
                    minRange = dataValue;
                    changeRangeFlag = true;
                }                
            }
        }
        json.range = minRange + ',' + maxRange;
    }
    var returnObj = drawChartGrid(json, w);
    var layerNum = returnObj.layerNum;
    var yMax = returnObj.yMax;
    var yMin = returnObj.yMin;
    var grid = json.grid;
    
    //绘制柱子，先检测出要绘制的所有柱子的宽度;
    var gapWidth = returnObj.gapWidth;
    var barGap = json.series[0].barGap || 0; //如果一栏里面有多个柱子，柱子与柱子之间的间距，单位px;
    var cloumnWidthArr = [];//每根柱子的宽度是整个空白区域的宽度的10%;
    var oneGapWidth = 0;
    var maxColumnWidth = 0; //当是堆叠图的时候，则要记录最大的柱子宽度(实际上堆叠图应该设置所有柱子宽度都一样);
    for(var i=0, len=json.series.length; i<len; i++){//先循环一次，主要是得到所有柱子宽度的总和;
        var barWidth = json.series[i].barWidth;
        if(barWidth){
            //有可能是百分比，也可能是px;
            var reg = /[%]$/;
            var reg2 = /[px]$/;
            if(reg.test(barWidth)){
                var cloumnW = parseInt(barWidth.replace(/[%]$/, ''), 10) / 100;
                cloumnWidthArr.push(parseInt(gapWidth / len * cloumnW, 10));
                oneGapWidth += parseInt((gapWidth / len * cloumnW), 10);
            }else if(reg2.test(barWidth)){
                var cloumnW = parseInt(barWidth.replace(/[px]$/, ''), 10);
                cloumnWidthArr.push(cloumnW);
                oneGapWidth += cloumnW;
            }else{
                cloumnWidthArr.push(parseInt(barWidth,10));
                oneGapWidth += parseInt(barWidth,10);
            }
        }else{ //默认值是10%;
            barWidth = parseInt(gapWidth / len * 0.1, 10);
            cloumnWidthArr.push(parseInt(gapWidth / len * 0.1, 10));
            oneGapWidth += parseInt((gapWidth / len * 0.1), 10);
        }
        if(barWidth > maxColumnWidth){
            maxColumnWidth = parseInt(barWidth, 10);
        }
    }
    if(barGap != 0){
        oneGapWidth += barGap * (json.series.length -1);
    }
    //如果是堆叠图,两根柱子间的间距不需要;
    if(superposition){
        barGap = 0;
        oneGapWidth = maxColumnWidth;
    }
    //堆叠图主要是靠columnPositionArr数组里面记录up和down属性;
    //遇到正数就往up里面加，遇到负数就往down里面加。然后每个柱子分别移动up或down的值;
    var columnPositionArr = [];    
    //开始绘制柱子;
    for(var i=0; i<json.series.length; i++){
        for(j=0; j<json.series[i].data.length; j++){
            var data = (json.series[i].data[j].constructor == Object) ? json.series[i].data[j].value : json.series[i].data[j];
            addLayer({name: 'Chart-Column' + j + "-" + data});
            var startLeft = (gapWidth - oneGapWidth)/2 + gapWidth * j;
            var x = parseInt((startLeft + getArrayNumCount(cloumnWidthArr ,i) + w.left + grid.left), 10);
            
            //如果是堆叠图;
            if(superposition){
                x = parseInt((startLeft + w.left + grid.left), 10);
            }
            if(i != 0 && barGap != 0){
                x += barGap * i
            }
            var numberDataN = Number(data);
            var y = Number(w.bottom - grid.bottom - (grid.height / (yMax - yMin) * (numberDataN-yMin)));
            var dots; //要考虑负数的情况，也就是从0开始作为底部，而不是从拉框的底部开始绘制;
            if(yMin >= 0){
                var dots = [
                    [x, y],
                    [x, w.bottom-grid.bottom],
                    [x + cloumnWidthArr[i], w.bottom-grid.bottom],
                    [x + cloumnWidthArr[i], y]
                ];
                //如果是堆叠图;
                if(superposition){
                    var columnHeight = dots[1][1] - dots[0][1];
                    //不存在记录柱子位移信息;
                    if(typeof(columnPositionArr[j]) != 'object'){
                        columnPositionArr[j] = {
                            up: (numberDataN > 0) ?  columnHeight: 0,
                            down: (numberDataN < 0) ? columnHeight : 0
                        }
                    }else{
                        var movePos = (numberDataN >= 0) ? -columnPositionArr[j].up : columnPositionArr[j].down;
                        dots = arrayAddYNum(dots, movePos);
                        if(numberDataN > 0){
                            columnPositionArr[j].up += columnHeight;                            
                        }else{
                            columnPositionArr[j].down += columnHeight;
                        }
                    }
                }
            }else{
                var zeroNum = Number(w.bottom - grid.bottom - (grid.height / (yMax - yMin) * (0-yMin)));
                if(numberDataN >= 0){
                    var dots = [
                        [x, y],
                        [x, zeroNum],
                        [x + cloumnWidthArr[i], zeroNum],
                        [x + cloumnWidthArr[i], y]
                    ];
                }else{
                    var dots = [
                        [x, zeroNum],
                        [x, y],
                        [x + cloumnWidthArr[i], y],
                        [x + cloumnWidthArr[i], zeroNum]
                    ];
                }
                //如果是堆叠图;
                if(superposition){
                    var columnHeight = dots[1][1] - dots[0][1];
                    if(typeof(columnPositionArr[j]) != 'object'){
                        columnPositionArr[j] = {
                            up: (numberDataN > 0) ? columnHeight : 0,
                            down: (numberDataN < 0) ? columnHeight : 0
                        }
                    }else{
                        var movePos = (numberDataN >= 0) ? -columnPositionArr[j].up : columnPositionArr[j].down;
                        dots = arrayAddYNum(dots, movePos);
                        if(numberDataN > 0){
                            columnPositionArr[j].up += columnHeight;                            
                        }else{
                            columnPositionArr[j].down += columnHeight;
                        }
                    }
                }
            }


            var borderRadiusFlag = false;
            //如果是圆角，则绘制圆角的;
            if(json.series[i].data[j].constructor == Object && json.series[i].data[j].itemStyle && json.series[i].data[j].itemStyle.barBorderRadius){
                var barBorderRadius = json.series[i].data[j].itemStyle.barBorderRadius;
                borderRadiusFlag = true;
                dots = getConnerDots(dots, barBorderRadius);
            }
            layerNum++;
            var fillColor = json.series[i].data[j].itemStyle.color;
            drawVectorShape({
                dots: dots,
                type: 'fill',
                hexValueFill: fillColor,
                closed: true
            });

            /*if((json.series[i].data[j].constructor == Object && json.series[i].data[j].itemStyle && json.series[i].data[j].itemStyle.color)){
                if(typeof(json.series[i].data[j].itemStyle.color) == 'string'){
                    fillColor = json.series[i].data[j].itemStyle.color;
                    drawShape({
                        dots: dots,
                        type: 'fill', 
                        hexValue: fillColor,
                        opacity: 100
                    });
                }
            }else{
                if((json.series[i].data[j].constructor == Object && json.series[i].data[j].itemStyle && json.series[i].data[j].itemStyle.colors)){
                    if(json.series[i].data[j].itemStyle.type == 'linear'){
                        drawShape({
                            dots: dots,
                            type: 'selection'
                        });
                        var _w = cloumnWidthArr[i];
                        var _h = w.bottom-grid.bottom - y;
                        var _x = json.series[i].data[j].itemStyle.x;
                        var _y = json.series[i].data[j].itemStyle.y;
                        var _x2 = json.series[i].data[j].itemStyle.x2;
                        var _y2 = json.series[i].data[j].itemStyle.y2;
                        var _startColor =  json.series[i].data[j].itemStyle.colors[0];
                        var _endColor = json.series[i].data[j].itemStyle.colors[1];
                        lineGradient({
                            startX: x + _x * _w,
                            endX:  x  + _x2 * _w,
                            startY: y + _y * _h,
                            endY:  y + _y2 * _h,
                            startColor: _startColor,
                            endColor: _endColor
                        });
                        doc.selection.deselect();
                    }
                }else{
                    fillColor = json.areaColor[i];
                    drawShape({
                        dots: dots,
                        type: 'fill', 
                        hexValue: fillColor,
                        opacity: 100
                    });
                }
            };//end if else;*/
        }   
    }
    //文字不能被柱子压住，所以不得不多循环一次;
    columnPositionArr.length = 0;
    for(var i=0; i<json.series.length; i++){
        for(var j=0; j<json.series[i].data.length; j++){
            if(json.series[i].label && json.series[i].label.show){ //要在柱子上面绘制文字;
                var data = (json.series[i].data[j].constructor == Object) ? json.series[i].data[j].value : json.series[i].data[j];
                var series = json.series[i];
                var txtColor = series.label.color || '333333';
                var distance = series.label.distance || 5;
                var fontSize = series.label.fontSize || 12;
                var startLeft = (gapWidth - oneGapWidth)/2 + gapWidth * j;
                var numberDataN = Number(data);

                //如果是堆叠图;
                if(superposition){
                    var x = parseInt((startLeft + w.left + grid.left), 10);
                    var y = Number(w.bottom - grid.bottom - (grid.height / (yMax - yMin) * (numberDataN-yMin)));                
                    var columnHeight = 0;
                    if(yMin >= 0){
                        columnHeight = w.bottom - grid.bottom - y;
                    }else{
                        var zeroNum = Number(w.bottom - grid.bottom - (grid.height / (yMax - yMin) * (0-yMin)));
                        columnHeight = (numberDataN >= 0) ? zeroNum - y : y - zeroNum;
                    }

                    if(typeof(columnPositionArr[j]) != 'object'){
                        columnPositionArr[j] = {
                            up: (numberDataN > 0) ?  columnHeight: 0,
                            down: (numberDataN < 0) ? columnHeight : 0
                        }
                        var movePos = 0;
                    }else{
                        var movePos = (numberDataN >= 0) ? -columnPositionArr[j].up : columnPositionArr[j].down;                        
                        if(numberDataN > 0){
                            columnPositionArr[j].up += columnHeight;                            
                        }else{
                            columnPositionArr[j].down += columnHeight;
                        }
                    }                    
                    var txtXPos = x + cloumnWidthArr[i]/2;
                    var txtYPos = (numberDataN >= 0) ?  y - distance + movePos : y + distance + fontSize + movePos;     

                }else{
                    var x = parseInt(startLeft + getArrayNumCount(cloumnWidthArr ,i) + w.left + grid.left + barGap*i);                
                    var y = Number(w.bottom - grid.bottom - (grid.height / (yMax - yMin) * (numberDataN-yMin)));                
                    var txtXPos = x + cloumnWidthArr[i]/2;
                    var txtYPos = (numberDataN >= 0) ?  y - distance : y + distance + fontSize;     
                }
                addTxtLayer({
                        name: data,
                        content: data,
                        font: UED.font.family,
                        size: fontSize,
                        color: txtColor,
                        txtAlign: 'center',
                        position: [txtXPos, txtYPos]
                });
                layerNum++;
            }
        }
    }
    createLayerSet({
        name: 'Column Chart',
        number: layerNum
    });
    completeFn();
}
//绘制饼图;
function drawPieChart(json){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(json);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    //var r = w.width/2;
    var r = Math.min(w.width, w.height) / 2;
    var cx = w.left + r;
    var cy = w.top + r;
    drawVerctorPie(json, {
        r: r,
        cx: cx,
        cy: cy
    });
    completeFn();
}
function drawPie(json, obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var layerNum = 0;
    //得到百分比数据;
    var dataArray = json.series[0].data;
    var label = json.series[0].label;
    var count = 0;
    var maxData = -999999999999; //数据里面最大值，用来当做南丁格尔玫瑰图的100%的半径;
    for(var i=0; i<dataArray.length; i++){
        count += dataArray[i].value;
        if(dataArray[i].value > maxData){
            maxData = dataArray[i].value;
        }
    }
    var startAngle = 0;
    var txtPos = []; //存储饼图上文字要绘制的坐标;
    //将百分比放到json数据中去,并且开始绘制饼图的扇形;
    for(var i=0; i<dataArray.length; i++){
        layerNum++;
        var data = dataArray[i];
        data.percent = (data.value / count).toFixed(2); //百分比;
        data.percentShow =  (data.value / count * 100).toFixed(2) + '%'; //显示的百分比;
        data.angle = Math.floor(360 * data.value / count);
        //$.writeln(data.percent + "    " + data.percentShow + "   " + data.angle)
        addLayer({name: 'Pie-' + data.name + '-' + data.value + '-' + data.percentShow});
        var oldStartAngle = startAngle; //由于起始角度会变化，所以需要记录纠正一下;
        var para = {
            r: obj.r,
            cx: obj.cx,
            cy: obj.cy,
            start: startAngle,
            end: startAngle + data.angle,
            color: data.color,
            opacity: 100
        }
        if(json.series[0].radius){
            para.innerRadius = json.series[0].radius;
        }
        if(json.series[0].roseType){ //如果是南丁格尔玫瑰图;
            var roseR = obj.r * (data.value / maxData);
            if(json.series[0].radius){ //内部还需要绘制空心的;
                var innerGap,
                    innerRadius = json.series[0].radius,
                    reg = /%/,
                    reg2 = /px/;
                        
                if(reg.test(innerRadius)){
                    innerGap = parseInt(innerRadius, 10) / 100 * obj.r * 2;
                }else if(reg2.test(innerRadius)){
                    innerGap = parseInt(innerRadius.replace('px', ''), 10);
                }else{
                    innerGap = parseInt(innerRadius, 10);
                }
                roseR =  data.value * (obj.r - innerGap/2) / maxData + innerGap/2;
                //alert(roseR + ":" + innerGap)
            }
            para.roseType = true;
            para.roseR = roseR;
        }
        if(i == dataArray.length-1){para.end = 360}
        drawPieItem(para);
        if(data.selected){ //如果饼图是选中的，则需要偏移;
            var selectMove = getCircleDot({
                r: data.selectedGap ? data.selectedGap : obj.r * 0.1, //如果不存在偏移量，默认偏移半径的十分之一;
                cx: obj.cx,
                cy: obj.cy,
                angle: (startAngle + data.angle/2)
            });
            doc.activeLayer.translate(selectMove[0], -selectMove[1]);
        }
        //如果有需要绘制边框;
        if(data.border && data.border.borderWidth && data.border.borderColor && data.border.borderWidth > 0){
            layerNum++;
            addLayer({name: 'Pie-border'});
            para.border = data.border;
            //由于起始角度会变化，所以需要纠正一下;
            para.start = oldStartAngle;
            drawPieBorder(para);
            if(data.selected){ //如果饼图是选中的，则需要偏移;
                var selectMove = getCircleDot({
                    r: data.selectedGap ? data.selectedGap : obj.r * 0.1, //如果不存在偏移量，默认偏移半径的十分之一;
                    cx: obj.cx,
                    cy: obj.cy,
                    angle: (startAngle + data.angle/2)
                });
                doc.activeLayer.translate(selectMove[0], -selectMove[1]);
            }
        }
        var txtDistance = 0.6; //绘制的文字距离半径多少;
        if(label.position){
            switch(label.position){
                case 'inner':
                    txtDistance = 0.6;
                    break;
                case 'outer':
                
                    break;
                default:
                    txtDistance = label.position;
                    break;
            }
        }
        var txtDot = getCircleDot({
            r: obj.r * txtDistance,
            cx: obj.cx,
            cy: obj.cy,
            angle: (startAngle + data.angle/2)
        });
            
        var xpos = obj.cx + txtDot[0];
        var ypos = obj.cy - txtDot[1];
        if(data.selected){//如果是选中，那么文字也需要偏移;
            xpos += selectMove[0];
            ypos -= selectMove[1];
        }
        txtPos.push({
            x: xpos,
            y: ypos
        });
        startAngle = para.end;
    }
     
    if(label.show){
        //再循环一次，写上百分比文字;
        var txtShow = [];
        formatInnerContent(label.formatter, txtShow);
        //$.writeln(JSON.stringify(txtShow))
        
        for(var i=0; i<dataArray.length; i++){
            var data = dataArray[i];
            var txt = '';
            for(var j=0; j<txtShow.length; j++){
                if(typeof(txtShow[j]) == 'object'){
                    var prop = txtShow[j].prop;
                    txt += data[prop];
                }else{
                    txt += txtShow[j];
                }
            }
            addTxtLayer({
                name: data.percentShow,
                font: UED.font.family,
                color: label.color,
                size: label.fontSize,
                position: [txtPos[i].x, txtPos[i].y], //txtPos的length是肯定和dataArray一致的，所以这里共用i，没毛病;
                txtAlign: 'center',
                //content: data.name + '\r' + data.percentShow,
                content: txt,
                fauxBold: (label.border) ? true : false,
                leading: label.fontSize + 0.2 * label.fontSize
            });
            layerNum++;
        }
     }
     createLayerSet({
        name: 'Pie Chart',
        number: layerNum
     });
}
//将绘制分成2部分，先获取拉框bounds,然后到chrome里面去处理好点jsx只负责绘制;
function getMapBounds(){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var w = windowBounds = getSelectionBounds();
    return JSON.stringify(w);
}
//新重构的绘制地图，效率更高;
function drawGeoMap(json, settingsStr){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(json);
    var needDrawArr = json.needDrawArr;
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    doc.selection.deselect();
    if(settingsStr){
        var settings = JSON.parse(settingsStr);
        if(settings.itemStyle){
            if(typeof(settings.itemStyle.show) != 'undefined'){
                geoSetting.geo.itemStyle.show = settings.itemStyle.show;
                geoSetting.geo.itemStyle.color = settings.itemStyle.color; 
            }
            if(settings.itemStyle.borderWidth){
                geoSetting.geo.itemStyle.borderWidth = settings.itemStyle.borderWidth;
            }
            if(settings.itemStyle.borderColor){
                geoSetting.geo.itemStyle.borderColor = settings.itemStyle.borderColor;
            }
        }
        if(settings.label){
            if(typeof(settings.label.show) != 'undefined'){
                geoSetting.geo.label.show = settings.label.show;
                geoSetting.geo.label.color = settings.label.color;
                geoSetting.geo.label.fontSize = settings.label.fontSize;
            }
        }
    }
    var itemStyle = geoSetting.geo.itemStyle;
    var layerNum = 0;
    //开始利用这些新的点，绘制图案;
    for(var i =0; i<needDrawArr.length; i++){        
        //有底部图案;
        if(itemStyle.show){
            layerNum++;
            //有底部图案又有边框;
            if(itemStyle.borderWidth > 0){
                addLayer({name: 'map' + (i+1)});
                drawVectorShape({
                    dots: needDrawArr[i],
                    type: 'all', 
                    hexValue: itemStyle.borderColor || '000000',
                    hexValueFill: itemStyle.color || 'f6f4f2',
                    solidWidth: itemStyle.borderWidth || 1,
                    angleDot: 'idstrokeStyleRoundJoin',
                    closed: true
                });
            //只有底部图案,没有边框;
            }else{
                addLayer({name: 'map' + (i+1)});
                drawVectorShape({
                    dots: needDrawArr[i],
                    type: 'fill', 
                    hexValueFill: itemStyle.color || 'f6f4f2',
                    solidWidth: itemStyle.borderWidth || 1,
                    angleDot: 'idstrokeStyleRoundJoin',
                    closed: true
                });
            }
        //没有底部图案;
        }else{
            //有边框;
            if(itemStyle.borderWidth > 0){
                layerNum++;
                addLayer({name: 'map' + (i+1)});
                drawVectorShape({
                    dots: needDrawArr[i],
                    type: 'stroke', 
                    hexValue: itemStyle.borderColor || '000000',
                    solidWidth: itemStyle.borderWidth || 1,
                    angleDot: 'idstrokeStyleRoundJoin',
                    closed: true
                });
            }else{
            //没底部图案，也没边框,啥也不用做了;
                alert('您既没有勾选区域块面，也没有勾选区域边线，将无法绘制出地图')
            }
        }

        /*if(itemStyle.show){
            layerNum++;
            addLayer({name: 'Map' + i});
            drawShape({
                dots: needDrawArr[i],
                type: 'fill', 
                hexValue: itemStyle.color,
                opacity: 100
            });
        }        
        if(itemStyle.borderWidth > 0){
            layerNum++;
            addLayer({name: 'Map-border' + i});
            //TODO location还未实现,我当时写这个注释是什么意思？我是谁？我在哪？;
            drawVectorShape({
                dots: needDrawArr[i],
                type: 'stroke', 
                hexValue: itemStyle.borderColor || '000000',
                solidWidth: itemStyle.borderWidth || 1,
                angleDot: 'idstrokeStyleRoundJoin',
                closed: true
            });
        }*/
    }
    //绘制对应的文字;
    if(geoSetting.geo.label && geoSetting.geo.label.show){
        var txtCenterPoint = json.txtCenterPoint;
        var label = geoSetting.geo.label;
        var fontColor = label.color || '000000';
        var fontSize = label.fontSize || 12;
        for(var i=0; i<txtCenterPoint.length; i++){
            layerNum++;
            addTxtLayer({
                name: txtCenterPoint[i].name,
                font: UED.font.family,
                color: fontColor,
                size: fontSize,
                position: txtCenterPoint[i].position,
                txtAlign: 'center',
                content: txtCenterPoint[i].name 
            });         
        }     
    }    
    createLayerSet({
        name: 'Map',
        number: layerNum
    });
    completeFn();
}

//绘制地图[已废弃];
function drawMap(json, settingsStr){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(json);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    if(settingsStr){
        var settings = JSON.parse(settingsStr);
        if(settings.itemStyle){
            if(typeof(settings.itemStyle.show) != 'undefined'){
                geoSetting.geo.itemStyle.show = settings.itemStyle.show;
                geoSetting.geo.itemStyle.color = settings.itemStyle.color; 
            }
            if(settings.itemStyle.borderWidth){
                geoSetting.geo.itemStyle.borderWidth = settings.itemStyle.borderWidth;
            }
            if(settings.itemStyle.borderColor){
                geoSetting.geo.itemStyle.borderColor = settings.itemStyle.borderColor;
            }
        }
        if(settings.label){
            if(typeof(settings.label.show) != 'undefined'){
                geoSetting.geo.label.show = settings.label.show;
                geoSetting.geo.label.color = settings.label.color;
                geoSetting.geo.label.fontSize = settings.label.fontSize;
            }
        }
    }

    var mapRect = json.mapRect;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    //通过mapRect.wh的比例,如果大于1，则说明宽度大于高度;
    /*if(mapRect.wh >= 1){
        w.height = parseInt(w.width / mapRect.wh, 10);
        w.bottom = w.top + w.height;
    }else{
        w.width = parseInt(w.height * mapRect.wh, 10);
        w.right = w.left + w.width;
    }*/
    //用拉框的宽高来确定;
    if(w.width <= w.height){
        w.height = parseInt(w.width / mapRect.wh, 10);
        w.bottom = w.top + w.height;
    }else{
        w.width = parseInt(w.height * mapRect.wh, 10);
        w.right = w.left + w.width;
    }
    
    
    var needDrawArr = []; //存储所有要绘制的“矢量”区块;
    var features = json.features;
    for(var i=0; i<features.length; i++){
        var geometry = features[i].geometry;
        var geometryType = geometry.type;
        var coordinatesArr = geometry.coordinates;
        var onePart = []; //存储一个“矢量”区块所有的点集合;
        if(geometryType == 'Polygon'){
            var onePart = []; //存储一个“矢量”区块所有的点集合;
            var polyDots = coordinatesArr[0];
            for(var j=0; j<polyDots.length; j++){
                var newDots = getNewMapPosition(polyDots[j], mapRect, w);
                    if(polyDots[j-1]){
                        if(newDots[0] != onePart[onePart.length-1][0] && newDots[1] != onePart[onePart.length-1][1]){
                            onePart.push(newDots);
                        }
                    }else{
                        onePart.push(newDots);
                    }
            }
            needDrawArr.push(onePart);
        }else if(geometryType == 'MultiPolygon'){
            for(var k=0; k<coordinatesArr.length; k++){
                var multPart = coordinatesArr[k];
                var onePart = [];
                for(var m=0; m<multPart.length; m++){
                    var multDots = multPart[m];
                    for(var n=0; n<multDots.length; n++){
                        var newDots = getNewMapPosition(multDots[n], mapRect, w);
                        if(multDots[n-1]){
                            if((newDots[0] != onePart[onePart.length-1][0] && newDots[1] != onePart[onePart.length-1][1])){
                                onePart.push(newDots);
                            }
                        }else{
                            onePart.push(newDots);
                        }
                    }
                }
                needDrawArr.push(onePart);
            }
        }
    }
    var itemStyle = geoSetting.geo.itemStyle;
    var layerNum = 0;
    //开始利用这些新的点，绘制图案;
    for(var i =0; i<needDrawArr.length; i++){
        if(itemStyle.show){
            layerNum++;
            addLayer({name: 'Map' + i});
            drawShape({
                dots: needDrawArr[i],
                type: 'fill', 
                hexValue: itemStyle.color,
                opacity: 100
            });
        }        
        if(itemStyle.borderWidth > 0){
            layerNum++;
            addLayer({name: 'Map-border' + i});
            //TODO location还未实现,我当时写这个注释是什么意思？我是谁？我在哪？;
            drawVectorShape({
                dots: needDrawArr[i],
                type: 'stroke', 
                hexValue: itemStyle.borderColor || '000000',
                solidWidth: itemStyle.borderWidth || 1,
                angleDot: 'idstrokeStyleRoundJoin',
                closed: true
            });
        }
    }
    //绘制对应的文字;
    if(geoSetting.geo.label && geoSetting.geo.label.show){
        var label = geoSetting.geo.label;
        var fontColor = label.color || '000000';
        var fontSize = label.fontSize || 12;
        for(var i=0; i<features.length; i++){
            var properties = features[i].properties;
            var cp = getNewMapPosition(properties.cp, mapRect, w);
            layerNum++;
            addTxtLayer({
                name: properties.name,
                font: UED.font.family,
                color: fontColor,
                size: fontSize,
                position: cp,
                txtAlign: 'center',
                content: properties.name 
            });         
        }     
    }    
    createLayerSet({
        name: 'Map',
        number: layerNum
    });
    completeFn();
};//end drawMap;

//绘制雷达图;
function drawRadar(json){
    var errObj  = checkError();
    if(errObj.flag){
        alert(errObj.tip);
        return;
    }
    var json = JSON.parse(json);
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var layerNum = 0;
    var w = windowBounds = getSelectionBounds();
    doc.selection.deselect();
    //var r = w.width/2;
    var r = Math.min(w.width, w.height)/2;
    var cx = w.left + r;
    var cy = w.top + r;                
    //确定雷达要绘制成几边形,有几个顶点;
    var data = json.series[0].data;
    var radar = json.series[0].radar;
    var label = json.series[0].label;
    var indicator = radar.indicator;
    var angle = 360 / indicator.length;
    
    var lineColor = (radar.splitLine && radar.splitLine.lineStyle &&  radar.splitLine.lineStyle.color) ? radar.splitLine.lineStyle.color : 'c0c0c0';
    var lineWidth =(radar.splitLine && radar.splitLine.lineStyle &&  radar.splitLine.lineStyle.lineWidth) ? radar.splitLine.lineStyle.lineWidth : 1; 
    var allOuterDots = []; //记录最外层的那几个点;
    var splitNumber = radar.splitNumber || 5;
    var rStep = 1 / splitNumber;
    //无论是什么图形，都是默认分成5层，百度的echarts是5层;
    for(var m=0; m<splitNumber; m++){
        var dots = []; //多边形最外圈的n个点存储在这里;
        var innerDots = []; //比最外圈缩小一圈的n个点存储在这里，因为要删掉内圈;
        for(var i=0; i<indicator.length; i++){
            var posArray = getCircleDot({
                r: r - rStep*r*m, //半径每次减去1/5;
                cx: cx,
                cy: cy,
                angle: angle*i
            });
            dots.push([cx+posArray[0], cy-posArray[1]]);
            if(m == 0){
                allOuterDots.push([cx+posArray[0], cy-posArray[1]]);
            }
            if(m != (splitNumber-1)){ //最内侧的一个多边形不需要删除内圈。他自己就是内圈;
                var innerArray = getCircleDot({
                    r: r - rStep * r * (m+1),
                    cx: cx,
                    cy: cy,
                    angle: angle*i
                });
                innerDots.push([cx+innerArray[0], cy-innerArray[1]]);
            }
        }
        var shape = (radar.shape && radar.shape == 'circle') ? 'circle' : 'polygon';
        var color = (radar.splitArea && radar.splitArea.areaStyle && radar.splitArea.areaStyle.color && radar.splitArea.areaStyle.color[m] != undefined) ? radar.splitArea.areaStyle.color[m] : ((m%2 == 0) ? 'f5f5f5': 'e6e6e6');
        if(shape == 'circle'){
            var circleR =  parseInt(r - rStep*r*m, 10); 
            if(radar.splitArea && radar.splitArea.show){
                layerNum++;
                addLayer({name: '雷达-背景色块-' + (m+1)});
                /*drawCirclePath({
                    startX: cx - circleR,
                    startY: cy - circleR,
                    endX: cx + circleR,
                    endY: cy + circleR,
                    type: 'fill',
                    hexValue: color,
                    opacity: 100
                });
                //绘制内部小圆圈，将其删除;
                if(m != (splitNumber-1)){
                    var innerR =  parseInt(r - rStep*r*(m+1), 10);      
                    drawCirclePath({
                        startX: cx - innerR,
                        startY: cy - innerR,
                        endX: cx + innerR,
                        endY: cy + innerR,
                        type: 'selection'
                    });
                    doc.selection.clear();
                    doc.selection.deselect();
                }*/
                var outerPath = getCirclePath({
                    cx: cx,
                    cy: cy,
                    r : circleR
                });
                var innerR =  parseInt(r - rStep*r*(m+1), 10);
                var innerPath = getCirclePath({
                    cx: cx,
                    cy: cy,
                    r : innerR
                })
                drawSomeVectorShape([outerPath], {
                    type: 'fill',
                    hexValueFill: color,
                    closed: true
                }, [innerPath]);
            }
            if(radar.splitLine && radar.splitLine.show){
                layerNum++;
                addLayer({name: '雷达-背景分隔线-' + (m+1)});
                var outerPath = getCirclePath({
                    cx: cx,
                    cy: cy,
                    r : circleR
                });
                drawVectorShape({
                    dots: outerPath,
                    type: 'stroke',
                    hexValue: lineColor,
                    solidWidth: lineWidth,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignInside',
                    closed: true
                });

                /*drawCirclePath({
                    startX: cx - circleR,
                    startY: cy - circleR,
                    endX: cx + circleR,
                    endY: cy + circleR,
                    type: 'stroke',
                    location: 'inside',
                    hexValue: lineColor,
                    solidWidth: lineWidth
                });*/
            }            
        }else{
            //绘制面;
            if(radar.splitArea && radar.splitArea.show){
                //如果有颜色就读取颜色值，如果没有就以白色，灰色间隔填充;
                layerNum++;
                addLayer({name: '雷达-背景色块-' + (m+1)});
                drawSomeVectorShape([dots], {
                    type: 'fill',
                    hexValueFill: color,
                    closed: true
                }, [innerDots]);
                /*drawShape({
                    dots: dots,
                    type: 'fill', 
                    hexValue: color,
                    opacity: 100
                });
                //删除内侧小一圈的面;
                if(m != (splitNumber-1)){
                    drawShape({
                            dots: innerDots,
                            type: 'selection'
                    });
                    doc.selection.clear();
                    doc.selection.deselect();                    
                }*/

            }
            if(radar.splitLine && radar.splitLine.show){
                //绘制线;
                layerNum++;
                addLayer({name: '雷达-背景分隔线-' + (m+1)});
                drawVectorShape({
                    dots: dots,
                    type: 'stroke',
                    hexValue: lineColor,
                    solidWidth: lineWidth,
                    angleDot: 'idstrokeStyleRoundJoin',
                    strokeStyleAlign: 'strokeStyleAlignInside',
                    closed: true
                });
                /*drawShape({
                    dots: dots,
                    type: 'stroke', 
                    hexValue: lineColor,
                    solidWidth: lineWidth,
                    location: 'inside'
                });*/
            }            
        }
    }
    //绘制圆点到各个角的线段;
    for(var i=0; i<allOuterDots.length; i++){
        layerNum++;
        addLayer({name: '雷达-中心到顶点线-' + (m+1)});
        drawVectorShape({
            dots: [[cx, cy], allOuterDots[i]],
            type: 'stroke', 
            hexValue: lineColor,
            solidWidth: lineWidth,
            angleDot: 'idstrokeStyleRoundJoin'
        });
    }
    //绘制相关数据的面积图,面积图可能有多个;
    for(var n=0; n<data.length; n++){
        var earaPos = [];
        for(var j=0; j<data[n].value.length; j++){
            var currentData = data[n].value[j];
            //通过数据与max对比，得到数据的百分比;
            var dadaY = currentData / (indicator[j].max) * r;
            var temp = getCircleDot({
                r: dadaY, 
                cx: cx,
                cy: cy,
                angle: angle*j
            });
            earaPos.push([cx + temp[0], cy - temp[1]]);
        }
        if(data[n].areaStyle){ //如果需要绘制面积;
            var bgColor = data[n].areaStyle.color;
            var opacity = data[n].areaStyle.opacity || 100;
            addLayer({name: '雷达-背景色块-' + (n+1)});
            layerNum++;
            drawVectorShape({
                dots: earaPos,
                type: 'fill', 
                hexValueFill: bgColor,
                closed: true
            });
            setActiveLayerOpacity(opacity);

            /*if(typeof(bgColor) == 'string'){
                drawShape({
                    dots: earaPos,
                    type: 'fill', 
                    hexValue: bgColor,
                    opacity: opacity
                });
            }else if(typeof(bgColor) == 'object'){
                //要绘制渐变，先绘制出选区;
                drawShape({
                    dots: earaPos,
                    type: 'selection' 
                });                       
                var sel = getSelectionBounds();
                var x = bgColor.x,
                    y = bgColor.y,
                    x2 = bgColor.x2 || null,
                    y2 = bgColor.y2 || null,
                    rPos = bgColor.r || null,
                    startColor = bgColor.colors[0],
                    endColor = bgColor.colors[1];
                        
                if(bgColor.type == 'linear'){ //线性渐变;
                    lineGradient({
                        startX: sel.width * x + sel.left,
                        endX: sel.width * x2 + sel.left,
                        startY: sel.height * y + sel.top,
                        endY: sel.height * y2 + sel.top,
                        startColor: startColor,
                        endColor: endColor
                    });
                }else if(bgColor.type == 'radial') {
                    radialGradient({
                        x: sel.width * x + sel.left,
                        y: sel.height * y + sel.top,
                        r: (sel.width * rPos),
                        startColor: startColor,
                        endColor: endColor
                    });
                }
                doc.selection.deselect();
                doc.activeLayer.fillOpacity = opacity;
            }*/            
        }
        if(data[n].lineStyle){ //绘制数据产生的线;
            var lineColor = data[n].lineStyle.color;
            var opacity = data[n].lineStyle.opacity || 100;
            var lineWidth = data[n].lineStyle.lineWidth || 2;
            addLayer({name: '雷达-线条-' + (n+1)});
            layerNum++;
            /*drawShape({
                dots: earaPos,
                type: 'stroke', 
                hexValue: lineColor,
                solidWidth: lineWidth,
                location: 'inside'
            });*/
            drawVectorShape({
                dots: earaPos,
                type: 'stroke', 
                hexValue: lineColor,
                solidWidth: lineWidth,
                angleDot: 'idstrokeStyleRoundJoin',
                strokeStyleAlign: 'strokeStyleAlignCenter',
                closed: true
            });
            if(opacity < 100){
                doc.activeLayer.fillOpacity = opacity
            }
        }
        //如果需要绘制点;
        if(data[n].symbol){
            var symbol = data[n].symbol;
            var size = symbol.symbolSize || 5;
            var color = symbol.symbolColor || data[n].lineStyle.color || 'ff0000';
            var opacity = symbol.symbolOpacity || 100;
            var solidWidth = symbol.symbolSolidWidth || 0;
            var solidColor = symbol.symbolSolidColor || data[n].lineStyle.color || 'ff0000';
                        
            for(var i=0; i<earaPos.length; i++){
                var startX = parseInt(earaPos[i][0] - size/2, 10);
                var startY = parseInt(earaPos[i][1] - size/2, 10);
                var endX = parseInt(earaPos[i][0]  + size/2, 10);
                var endY = parseInt(earaPos[i][1] + size/2, 10);
                /*drawCirclePath({
                    startX: startX,
                    startY: startY,
                    endX: endX,
                    endY: endY,
                    type: 'fill',
                    hexValue: color,
                    opacity: opacity
                });
                if(solidWidth > 0){
                    drawCirclePath({
                        startX: startX,
                        startY: startY,
                        endX: endX,
                        endY: endY,
                        type: 'stroke',
                        location: 'outside',
                        hexValue: solidColor,
                        solidWidth: solidWidth
                    });
                }*/
                var cirObj = {
                    bgColor: color,
                    top: startY,
                    left: startX,
                    diam: size
                }
                if(solidWidth > 0){
                    cirObj.borderWidth = solidWidth;
                    cirObj.borderColor = solidColor;
                    cirObj.strokeStyleAlign = 'strokeStyleAlignOuterSide';
                }
                layerNum++;
                addLayer({name: '雷达-标记小圆点-' + (i+1)});
                drawVerctorCircle(cirObj);
                if(opacity != 100){
                    setActiveLayerOpacity(opacity);
                }

            };//end for;
        }                        
    };//end for;
    //绘制最外层的文字;
    if(label && label.show){
        var fontSize = label.fontSize || 12;
        var color = label.color;
        var border = label.border || false;
        
        var txtPosArr = [];
        for(var i=0; i<indicator.length; i++){
            var posArray = getCircleDot({
                r: r + fontSize,
                cx: cx,
                cy: cy,
                angle: angle*i
            });
            txtPosArr.push([cx+posArray[0], cy-posArray[1]]);
        }
        for(var i=0; i<txtPosArr.length; i++){
            var xpos = txtPosArr[i][0];
            var ypos = txtPosArr[i][1] + fontSize/2;
            var txtAlign = 'center'
            if(xpos == cx){
                txtAlign = 'center';
            }else if(xpos > cx){
                txtAlign = 'left';
            }else if(xpos < cx){
                txtAlign = 'right';
            }
            layerNum++;
            addTxtLayer({
                name: indicator[i].name,
                font: UED.font.family,
                color: color,
                size: fontSize,
                position: [xpos, ypos],
                txtAlign: txtAlign,
                content: indicator[i].name,
                fauxBold: border
            });                                
        }
    }
    createLayerSet({
        name: 'Column Radar',
        number: layerNum
    });
    completeFn();
}
function drawVerctorPie(json, obj){
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var layerNum = 0;
    //得到百分比数据;
    var dataArray = json.series[0].data;
    var label = json.series[0].label;
    var count = 0;
    var maxData = -999999999999; //数据里面最大值，用来当做南丁格尔玫瑰图的100%的半径;
    for(var i=0; i<dataArray.length; i++){
        count += dataArray[i].value;
        if(dataArray[i].value > maxData){
            maxData = dataArray[i].value;
        }
    }
    var startAngle = 0;
    var txtPos = []; //存储饼图上文字要绘制的坐标;
    //将百分比放到json数据中去,并且开始绘制饼图的扇形;
    for(var i=0; i<dataArray.length; i++){
        layerNum++;
        var data = dataArray[i];
        data.percent = (data.value / count).toFixed(2); //百分比;
        data.percentShow =  (data.value / count * 100).toFixed(2) + '%'; //显示的百分比;
        data.angle = Math.floor(360 * data.value / count);
        //$.writeln(data.percent + "    " + data.percentShow + "   " + data.angle)
        addLayer({name: 'Pie-' + data.name + '-' + data.value + '-' + data.percentShow});
        var oldStartAngle = startAngle; //由于起始角度会变化，所以需要记录纠正一下;
        var para = {
            r: obj.r,
            cx: obj.cx,
            cy: obj.cy,
            start: startAngle,
            end: startAngle + data.angle,
            color: data.color,
            opacity: 100
        }
        if(json.series[0].radius){
            para.innerRadius = json.series[0].radius;
        }
        if(json.series[0].roseType){ //如果是南丁格尔玫瑰图;
            var roseR = obj.r * (data.value / maxData);
            if(json.series[0].radius){ //内部还需要绘制空心的;
                var innerGap,
                    innerRadius = json.series[0].radius,
                    reg = /%/,
                    reg2 = /px/;
                        
                if(reg.test(innerRadius)){
                    innerGap = parseInt(innerRadius, 10) / 100 * obj.r * 2;
                }else if(reg2.test(innerRadius)){
                    innerGap = parseInt(innerRadius.replace('px', ''), 10);
                }else{
                    innerGap = parseInt(innerRadius, 10);
                }
                roseR =  data.value * (obj.r - innerGap/2) / maxData + innerGap/2;
                //alert(roseR + ":" + innerGap)
            }
            para.roseType = true;
            para.roseR = roseR;
        }
        if(data.border && data.border.borderWidth && data.border.borderColor && data.border.borderWidth > 0){
            para.border = data.border;
        }
        if(i == dataArray.length-1){para.end = 360}
        drawVerctorPieItem(para);
        if(data.selected){ //如果饼图是选中的，则需要偏移;
            var selectMove = getCircleDot({
                r: data.selectedGap ? data.selectedGap : obj.r * 0.1, //如果不存在偏移量，默认偏移半径的十分之一;
                cx: obj.cx,
                cy: obj.cy,
                angle: (startAngle + data.angle/2)
            });
            doc.activeLayer.translate(selectMove[0], -selectMove[1]);
        }
        //如果有需要绘制边框;
        /*if(data.border && data.border.borderWidth && data.border.borderColor && data.border.borderWidth > 0){
            layerNum++;
            addLayer({name: 'Pie-border'});
            para.border = data.border;
            //由于起始角度会变化，所以需要纠正一下;
            para.start = oldStartAngle;
            drawPieBorder(para);
            if(data.selected){ //如果饼图是选中的，则需要偏移;
                var selectMove = getCircleDot({
                    r: data.selectedGap ? data.selectedGap : obj.r * 0.1, //如果不存在偏移量，默认偏移半径的十分之一;
                    cx: obj.cx,
                    cy: obj.cy,
                    angle: (startAngle + data.angle/2)
                });
                doc.activeLayer.translate(selectMove[0], -selectMove[1]);
            }
        }*/
        var txtDistance = 0.6; //绘制的文字距离半径多少;
        if(label.position){
            switch(label.position){
                case 'inner':
                    txtDistance = 0.6;
                    break;
                case 'outer':
                
                    break;
                default:
                    txtDistance = label.position;
                    break;
            }
        }
        var txtDot = getCircleDot({
            r: obj.r * txtDistance,
            cx: obj.cx,
            cy: obj.cy,
            angle: (startAngle + data.angle/2)
        });
            
        var xpos = obj.cx + txtDot[0];
        var ypos = obj.cy - txtDot[1];
        if(data.selected){//如果是选中，那么文字也需要偏移;
            xpos += selectMove[0];
            ypos -= selectMove[1];
        }
        txtPos.push({
            x: xpos,
            y: ypos
        });
        startAngle = para.end;
    }
     
    if(label.show){
        //再循环一次，写上百分比文字;
        var txtShow = [];
        formatInnerContent(label.formatter, txtShow);
        //$.writeln(JSON.stringify(txtShow))
        
        for(var i=0; i<dataArray.length; i++){
            var data = dataArray[i];
            var txt = '';
            for(var j=0; j<txtShow.length; j++){
                if(typeof(txtShow[j]) == 'object'){
                    var prop = txtShow[j].prop;
                    txt += data[prop];
                }else{
                    txt += txtShow[j];
                }
            }
            addTxtLayer({
                name: data.percentShow,
                font: UED.font.family,
                color: label.color,
                size: label.fontSize,
                position: [txtPos[i].x, txtPos[i].y], //txtPos的length是肯定和dataArray一致的，所以这里共用i，没毛病;
                txtAlign: 'center',
                //content: data.name + '\r' + data.percentShow,
                content: txt,
                fauxBold: (label.border) ? true : false,
                leading: label.fontSize + 0.2 * label.fontSize
            });
            layerNum++;
        }
     }
     createLayerSet({
        name: 'Pie Chart',
        number: layerNum
     });
}

function drawVerctorPieItem(obj){
    //alert(JSON.stringify(obj))
    var doc = app.activeDocument;
    var layer = doc.activeLayer;
    var drawAngle = 80;
    var r = obj.r, 
        cx = obj.cx, 
        cy = obj.cy,
        start = obj.start,
        end = obj.end;
    
    var angle = end - start;
    var oldStart = obj.start;
    var innerCirclepath = [];

    //如果存在这个属性，则是绘制空心的饼图;
    if(obj.innerRadius){
        var radius,
            reg = /%/,
            reg2 = /px/;
        if(reg.test(obj.innerRadius)){
            radius = parseInt(obj.innerRadius, 10) / 100 * obj.r * 2; //当是南丁格尔玫瑰图的时候，由于每个扇形半径不一样，所以这里不是乘以半径;
        }else if(reg2.test(obj.innerRadius)){
            radius = parseInt(obj.innerRadius.replace('px', ''), 10);
        }else{
            radius = parseInt(obj.innerRadius, 10);
        }
        var halfR = radius / 2;
        innerCirclepath = getCirclePath({
            cx: cx,
            cy: cy,
            r : halfR
        });
    }
    //大于90度的圆弧则必须绘制多个形状，先都画出来;
    var num = Math.ceil(angle / drawAngle);    
    if(num > 1){
        var allPath = [];
        for(var i=0; i<num; i++){
            obj.start = oldStart + i*drawAngle;
            var nowStart = obj.start;
            if(i == num-1 && angle % drawAngle != 0){
                drawAngle = angle % drawAngle;
            }
            obj.end = nowStart + drawAngle;
            if(i>0){obj.start -= 5} //起始角度往回倒退5度，这样就有重合的，就不会有空隙;
            var pathDots = getArcDots(obj);
            allPath.push(pathDots);
        }
        //在一个图层里面绘制多个形状;
        //如果是镂空，改颜色前还需要挖掉中间圆圈;        
        //背景色是渐变;
        var baseObj = {
            type: 'fill',
            hexValueFill: obj.color,
            closed: true
        }
        //如果有需要绘制边框;
        if(obj.border && obj.border.borderWidth && obj.border.borderColor && obj.border.borderWidth > 0){
            baseObj.type = 'all';
            baseObj.hexValue = obj.border.borderColor;
            baseObj.solidWidth = obj.border.borderWidth;
            baseObj.strokeStyleAlign = changeToStrokeStyleAlign(obj.border.borderLocation);
            baseObj.angleDot= 'idstrokeStyleRoundJoin';
        }
        if(typeof(obj.color) == 'object'){
            if(obj.innerRadius){
                drawSomeVectorShape(allPath, baseObj, [innerCirclepath]);
            }else{
                drawSomeVectorShape(allPath, baseObj);
            }
        //背景色是单色;
        }else{
            if(obj.innerRadius){
                drawSomeVectorShape(allPath, baseObj, [innerCirclepath]);
            }else{
                drawSomeVectorShape(allPath, baseObj);
            }
        }
    }else{
        for(var i=0; i<num; i++){
            obj.start = oldStart + i*drawAngle;
            var nowStart = obj.start;
            if(i == num-1 && angle % drawAngle != 0){
                drawAngle = angle % drawAngle;
            }
            obj.end = nowStart + drawAngle;
            if(i>0){obj.start -= 5} //起始角度往回倒退5度，这样就有重合的，就不会有空隙;
            var pathDots = getArcDots(obj);         
            var baseObj = {
                dots: pathDots,
                type: 'fill',
                hexValueFill: obj.color,
                closed: true
            }
            //如果有需要绘制边框;
            if(obj.border && obj.border.borderWidth && obj.border.borderColor && obj.border.borderWidth > 0){
                baseObj.type = 'all';
                baseObj.hexValue = obj.border.borderColor;
                baseObj.solidWidth = obj.border.borderWidth;
                baseObj.strokeStyleAlign = changeToStrokeStyleAlign(obj.border.borderLocation);
                baseObj.angleDot= 'idstrokeStyleRoundJoin';
            }
            if(obj.innerRadius){
                drawSomeVectorShape([pathDots], baseObj, [innerCirclepath]);
            }else{
                drawVectorShape(baseObj);
            }
        }
    }
    
    
}
//绘制饼图的一部分;
//从-90度开始绘制，因为echarts的饼图规则是从正北开始计算;
//每单大于90度的饼图，就需要分多部进行绘制;
function _drawVerctorPieItem(obj){
    var r = (obj.roseType) ? obj.roseR : obj.r,  //如果是南丁格尔玫瑰图，则使用roseR;
          cx = obj.cx, 
          cy = obj.cy,
          start = obj.start - 90,
          end = obj.end - 90;
          
    if(end - start > 0){
        var pie = arcToBezier(r, cx, cy, angleToRadian(start), angleToRadian(end));
        var dots = [{
            anchor: [pie.x0, pie.y0],
            left:      [pie.x1, pie.y1],
            right:    [pie.x0, pie.y0]
        },{
            anchor: [pie.x3, pie.y3],
            left:      [pie.x3, pie.y3],
            right:    [pie.x2, pie.y2]
        },[cx, cy]];
    
        drawVectorShape({
            dots: dots,
            type: 'fill', 
            hexValueFill: obj.color,
            opacity: 100,
            closed: true
        });
    }
    //如果存在这个属性，则是绘制空心的饼图;
    if(obj.innerRadius){
        var radius,
            reg = /%/,
            reg2 = /px/;
        if(reg.test(obj.innerRadius)){
            radius = parseInt(obj.innerRadius, 10) / 100 * obj.r * 2; //当是南丁格尔玫瑰图的时候，由于每个扇形半径不一样，所以这里不是乘以半径;
        }else if(reg2.test(obj.innerRadius)){
            radius = parseInt(obj.innerRadius.replace('px', ''), 10);
        }else{
            radius = parseInt(obj.innerRadius, 10);
        }
        var halfR = radius / 2;
        drawCirclePath({
            startX: obj.cx - halfR,
            startY: obj.cy - halfR,
            endX: obj.cx + halfR,
            endY: obj.cy + halfR,
            type: 'selection'
        });
        app.activeDocument.selection.clear();
        app.activeDocument.selection.deselect();
    }    
}

/*======================================================================================================== 其他，需要重构的函数，V1.0.0.0版本临时用下 */
////////////////////////////////////////////////////////////////////////////////////添加背景外发光;
function addOuterLight(){
    var idsetd = charIDToTypeID( "setd" );
    var desc210 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref55 = new ActionReference();
        var idPrpr = charIDToTypeID( "Prpr" );
        var idLefx = charIDToTypeID( "Lefx" );
        ref55.putProperty( idPrpr, idLefx );
        var idLyr = charIDToTypeID( "Lyr " );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref55.putEnumerated( idLyr, idOrdn, idTrgt );
    desc210.putReference( idnull, ref55 );
    var idT = charIDToTypeID( "T   " );
        var desc211 = new ActionDescriptor();
        var idScl = charIDToTypeID( "Scl " );
        var idPrc = charIDToTypeID( "#Prc" );
        desc211.putUnitDouble( idScl, idPrc, 100.000000 );
        var idOrGl = charIDToTypeID( "OrGl" );
            var desc212 = new ActionDescriptor();
            var idenab = charIDToTypeID( "enab" );
            desc212.putBoolean( idenab, true );
            var idpresent = stringIDToTypeID( "present" );
            desc212.putBoolean( idpresent, true );
            var idshowInDialog = stringIDToTypeID( "showInDialog" );
            desc212.putBoolean( idshowInDialog, true );
            var idMd = charIDToTypeID( "Md  " );
            var idBlnM = charIDToTypeID( "BlnM" );
            var idNrml = charIDToTypeID( "Nrml" );
            desc212.putEnumerated( idMd, idBlnM, idNrml );
            var idClr = charIDToTypeID( "Clr " );
                var desc213 = new ActionDescriptor();
                var idRd = charIDToTypeID( "Rd  " );
                desc213.putDouble( idRd, 0.000000 );
                var idGrn = charIDToTypeID( "Grn " );
                desc213.putDouble( idGrn, 241.996307 );
                var idBl = charIDToTypeID( "Bl  " );
                desc213.putDouble( idBl, 255.000000 );
            var idRGBC = charIDToTypeID( "RGBC" );
            desc212.putObject( idClr, idRGBC, desc213 );
            var idOpct = charIDToTypeID( "Opct" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc212.putUnitDouble( idOpct, idPrc, 20.000000 );
            var idGlwT = charIDToTypeID( "GlwT" );
            var idBETE = charIDToTypeID( "BETE" );
            var idSfBL = charIDToTypeID( "SfBL" );
            desc212.putEnumerated( idGlwT, idBETE, idSfBL );
            var idCkmt = charIDToTypeID( "Ckmt" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc212.putUnitDouble( idCkmt, idPxl, 0.000000 );
            var idblur = charIDToTypeID( "blur" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc212.putUnitDouble( idblur, idPxl, 40.000000 );
            var idNose = charIDToTypeID( "Nose" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc212.putUnitDouble( idNose, idPrc, 0.000000 );
            var idShdN = charIDToTypeID( "ShdN" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc212.putUnitDouble( idShdN, idPrc, 0.000000 );
            var idAntA = charIDToTypeID( "AntA" );
            desc212.putBoolean( idAntA, false );
            var idTrnS = charIDToTypeID( "TrnS" );
                var desc214 = new ActionDescriptor();
                var idNm = charIDToTypeID( "Nm  " );
                desc214.putString( idNm, """线性""" );
            var idShpC = charIDToTypeID( "ShpC" );
            desc212.putObject( idTrnS, idShpC, desc214 );
            var idInpr = charIDToTypeID( "Inpr" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc212.putUnitDouble( idInpr, idPrc, 50.000000 );
        var idOrGl = charIDToTypeID( "OrGl" );
        desc211.putObject( idOrGl, idOrGl, desc212 );
    var idLefx = charIDToTypeID( "Lefx" );
    desc210.putObject( idT, idLefx, desc211 );
    executeAction( idsetd, desc210, DialogModes.NO );
}
//创建一个白色圆形渐变;
function createCirGradual(){
    
    var idGrdn = charIDToTypeID( "Grdn" );
        var desc1406 = new ActionDescriptor();
        var idFrom = charIDToTypeID( "From" );
            var desc1407 = new ActionDescriptor();
            var idHrzn = charIDToTypeID( "Hrzn" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc1407.putUnitDouble( idHrzn, idPxl, 120.000000 );
            var idVrtc = charIDToTypeID( "Vrtc" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc1407.putUnitDouble( idVrtc, idPxl, 120.000000 );
        var idPnt = charIDToTypeID( "Pnt " );
        desc1406.putObject( idFrom, idPnt, desc1407 );
        var idT = charIDToTypeID( "T   " );
            var desc1408 = new ActionDescriptor();
            var idHrzn = charIDToTypeID( "Hrzn" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc1408.putUnitDouble( idHrzn, idPxl, 240.000000 );
            var idVrtc = charIDToTypeID( "Vrtc" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc1408.putUnitDouble( idVrtc, idPxl, 120.000000 );
        var idPnt = charIDToTypeID( "Pnt " );
        desc1406.putObject( idT, idPnt, desc1408 );
        var idType = charIDToTypeID( "Type" );
        var idGrdT = charIDToTypeID( "GrdT" );
        var idRdl = charIDToTypeID( "Rdl " );
        desc1406.putEnumerated( idType, idGrdT, idRdl );
        var idDthr = charIDToTypeID( "Dthr" );
        desc1406.putBoolean( idDthr, true );
        var idUsMs = charIDToTypeID( "UsMs" );
        desc1406.putBoolean( idUsMs, true );
        var idGrad = charIDToTypeID( "Grad" );
            var desc1409 = new ActionDescriptor();
            var idNm = charIDToTypeID( "Nm  " );
            desc1409.putString( idNm, """$$$/DefaultGradient/ForegroundToTransparent=Foreground to Transparent""" );
            var idGrdF = charIDToTypeID( "GrdF" );
            var idGrdF = charIDToTypeID( "GrdF" );
            var idCstS = charIDToTypeID( "CstS" );
            desc1409.putEnumerated( idGrdF, idGrdF, idCstS );
            var idIntr = charIDToTypeID( "Intr" );
            desc1409.putDouble( idIntr, 4096.000000 );
            var idClrs = charIDToTypeID( "Clrs" );
                var list230 = new ActionList();
                    var desc1410 = new ActionDescriptor();
                    var idType = charIDToTypeID( "Type" );
                    var idClry = charIDToTypeID( "Clry" );
                    var idFrgC = charIDToTypeID( "FrgC" );
                    desc1410.putEnumerated( idType, idClry, idFrgC );
                    var idLctn = charIDToTypeID( "Lctn" );
                    desc1410.putInteger( idLctn, 0 );
                    var idMdpn = charIDToTypeID( "Mdpn" );
                    desc1410.putInteger( idMdpn, 50 );
                var idClrt = charIDToTypeID( "Clrt" );
                list230.putObject( idClrt, desc1410 );
                    var desc1411 = new ActionDescriptor();
                    var idType = charIDToTypeID( "Type" );
                    var idClry = charIDToTypeID( "Clry" );
                    var idFrgC = charIDToTypeID( "FrgC" );
                    desc1411.putEnumerated( idType, idClry, idFrgC );
                    var idLctn = charIDToTypeID( "Lctn" );
                    desc1411.putInteger( idLctn, 4096 );
                    var idMdpn = charIDToTypeID( "Mdpn" );
                    desc1411.putInteger( idMdpn, 50 );
                var idClrt = charIDToTypeID( "Clrt" );
                list230.putObject( idClrt, desc1411 );
            desc1409.putList( idClrs, list230 );
            var idTrns = charIDToTypeID( "Trns" );
                var list231 = new ActionList();
                    var desc1412 = new ActionDescriptor();
                    var idOpct = charIDToTypeID( "Opct" );
                    var idPrc = charIDToTypeID( "#Prc" );
                    desc1412.putUnitDouble( idOpct, idPrc, 100.000000 );
                    var idLctn = charIDToTypeID( "Lctn" );
                    desc1412.putInteger( idLctn, 0 );
                    var idMdpn = charIDToTypeID( "Mdpn" );
                    desc1412.putInteger( idMdpn, 50 );
                var idTrnS = charIDToTypeID( "TrnS" );
                list231.putObject( idTrnS, desc1412 );
                    var desc1413 = new ActionDescriptor();
                    var idOpct = charIDToTypeID( "Opct" );
                    var idPrc = charIDToTypeID( "#Prc" );
                    desc1413.putUnitDouble( idOpct, idPrc, 0.000000 );
                    var idLctn = charIDToTypeID( "Lctn" );
                    desc1413.putInteger( idLctn, 4096 );
                    var idMdpn = charIDToTypeID( "Mdpn" );
                    desc1413.putInteger( idMdpn, 50 );
                var idTrnS = charIDToTypeID( "TrnS" );
                list231.putObject( idTrnS, desc1413 );
            desc1409.putList( idTrns, list231 );
        var idGrdn = charIDToTypeID( "Grdn" );
        desc1406.putObject( idGrad, idGrdn, desc1409 );
        executeAction( idGrdn, desc1406, DialogModes.NO );
}
//弹出窗口上的一个发光;
function addCloseOuterLight(){

    var idsetd = charIDToTypeID( "setd" );
    var desc305 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref83 = new ActionReference();
        var idPrpr = charIDToTypeID( "Prpr" );
        var idLefx = charIDToTypeID( "Lefx" );
        ref83.putProperty( idPrpr, idLefx );
        var idLyr = charIDToTypeID( "Lyr " );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref83.putEnumerated( idLyr, idOrdn, idTrgt );
    desc305.putReference( idnull, ref83 );
    var idT = charIDToTypeID( "T   " );
        var desc306 = new ActionDescriptor();
        var idScl = charIDToTypeID( "Scl " );
        var idPrc = charIDToTypeID( "#Prc" );
        desc306.putUnitDouble( idScl, idPrc, 100.000000 );
        var idOrGl = charIDToTypeID( "OrGl" );
            var desc307 = new ActionDescriptor();
            var idenab = charIDToTypeID( "enab" );
            desc307.putBoolean( idenab, true );
            var idpresent = stringIDToTypeID( "present" );
            desc307.putBoolean( idpresent, true );
            var idshowInDialog = stringIDToTypeID( "showInDialog" );
            desc307.putBoolean( idshowInDialog, true );
            var idMd = charIDToTypeID( "Md  " );
            var idBlnM = charIDToTypeID( "BlnM" );
            var idNrml = charIDToTypeID( "Nrml" );
            desc307.putEnumerated( idMd, idBlnM, idNrml );
            var idClr = charIDToTypeID( "Clr " );
                var desc308 = new ActionDescriptor();
                var idRd = charIDToTypeID( "Rd  " );
                desc308.putDouble( idRd, 255.000000 );
                var idGrn = charIDToTypeID( "Grn " );
                desc308.putDouble( idGrn, 101.998444 );
                var idBl = charIDToTypeID( "Bl  " );
                desc308.putDouble( idBl, 0.000000 );
            var idRGBC = charIDToTypeID( "RGBC" );
            desc307.putObject( idClr, idRGBC, desc308 );
            var idOpct = charIDToTypeID( "Opct" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc307.putUnitDouble( idOpct, idPrc, 90.000000 );
            var idGlwT = charIDToTypeID( "GlwT" );
            var idBETE = charIDToTypeID( "BETE" );
            var idSfBL = charIDToTypeID( "SfBL" );
            desc307.putEnumerated( idGlwT, idBETE, idSfBL );
            var idCkmt = charIDToTypeID( "Ckmt" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc307.putUnitDouble( idCkmt, idPxl, 0.000000 );
            var idblur = charIDToTypeID( "blur" );
            var idPxl = charIDToTypeID( "#Pxl" );
            desc307.putUnitDouble( idblur, idPxl, 12.000000 );
            var idNose = charIDToTypeID( "Nose" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc307.putUnitDouble( idNose, idPrc, 0.000000 );
            var idShdN = charIDToTypeID( "ShdN" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc307.putUnitDouble( idShdN, idPrc, 0.000000 );
            var idAntA = charIDToTypeID( "AntA" );
            desc307.putBoolean( idAntA, false );
            var idTrnS = charIDToTypeID( "TrnS" );
                var desc309 = new ActionDescriptor();
                var idNm = charIDToTypeID( "Nm  " );
                desc309.putString( idNm, """线性""" );
            var idShpC = charIDToTypeID( "ShpC" );
            desc307.putObject( idTrnS, idShpC, desc309 );
            var idInpr = charIDToTypeID( "Inpr" );
            var idPrc = charIDToTypeID( "#Prc" );
            desc307.putUnitDouble( idInpr, idPrc, 50.000000 );
        var idOrGl = charIDToTypeID( "OrGl" );
        desc306.putObject( idOrGl, idOrGl, desc307 );
    var idLefx = charIDToTypeID( "Lefx" );
    desc305.putObject( idT, idLefx, desc306 );
    executeAction( idsetd, desc305, DialogModes.NO );
}


//所有绘制的点，都是依赖分辨率72来计算xy的，如果分辨率不是72，则需要“点的坐标/(当前分辨率/72)”;
//获取当前分辨率/72;
function getNowReslution(){
    var doc = app.activeDocument;
    return Number(doc.resolution / 72); 
}
function correctPointsPosition(arrPoint){
    var docCoef = getNowReslution();
    var newArr = [arrPoint[0] / docCoef, arrPoint[1] / docCoef];
    return newArr;
}


